/******************************************************************************
* @license
* Copyright (C) 2008-2014, Roman Lygin. All rights reserved.
* Copyright (C) 2014-2023, CADEX. All rights reserved.
*
* This file is part of the CAD Exchanger software.
*
* This file may be used under the terms and conditions of the License
* Agreement supplied with the software.
*
* This file is provided "AS IS" WITH NO WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF DESIGN,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
*****************************************************************************/


declare namespace cadex {

// https://github.com/Microsoft/TypeScript/issues/27024#issuecomment-421529650
type IfEquals<X, Y, A, B> =
    (<T>() => T extends X ? 1 : 2) extends
    (<T>() => T extends Y ? 1 : 2) ? A : B;

type FieldKeys<T> = ({
    [P in keyof T]: T[P] extends Function ? never : P
})[keyof T];

type WritableFieldKeys<T> = ({
    [P in keyof T]: T[P] extends Function
        ? never
        : IfEquals<{ [Q in P]: T[P] }, { -readonly [Q in P]: T[P] }, P & string, never>
})[keyof T];

type ModelPrs_FieldChangedEventMap<T extends ModelPrs_EventDispatcher> = {
  [p in `${WritableFieldKeys<T>}Changed`]: ModelPrs_Event<p, T>;
};

/**
 * Predefined list of angle units.
 */
enum Base_AngleUnit {
    Base_AU_Radians = 0,
    Base_AU_Degrees = 1,
}
class Base_Helper {
    static angleUnitSymbol(theUnit: Base_AngleUnit): string;
    static angleUnitCoef(theUnit: Base_AngleUnit): number;
    static lengthUnitSymbol(theUnit: Base_LengthUnit): string;
    /**
     * Returns coefficient for conversion from millimeters to target unit.
     */
    static lengthUnitCoef(theUnit: Base_LengthUnit): number;
}
/**
 * Predefined list of length units.
 */
enum Base_LengthUnit {
    Base_LU_Millimeters = 0,
    Base_LU_Centimeters = 1,
    Base_LU_Meters = 2,
    Base_LU_Inches = 3,
    Base_LU_Feets = 4,
    Base_LU_Yards = 5,
    Base_LU_Micrometers = 6,
    Base_LU_Decimeters = 7,
    Base_LU_Kilometers = 8,
    Base_LU_Mils = 9,
    Base_LU_Miles = 10,
}
/**
 * Represents a node in a hierarchy of progress scopes.
 *
 * Scopes can be used to recursively split the range of {@link Base_ProgressStatus} object.
 * {@link Base_ProgressStatus} internally maintains a stack of created scopes.
 * Whenever a scope is closed it notifies its parent by incrementing its value,
 * the parent notifies its own parent and so on up to the {@link Base_ProgressStatus} object itself.
 *
 * Each scope has a range which is by default [0, 100]. A child scope is created
 * as a portion of the parent scope (specified in the constructor).
 * This weight information is used to compute increment in the parent range
 * when the child scope's value changes.
 */
class Base_ProgressScope {
    /**
     * Creates a scope as a portion of the explicitly specified parent scope.
     *
     * `theScaleInParentRange` specifies a portion of the parent scope's remaining range,
     * which this scope corresponds to. `theScaleInParentRange` is specified in parent's range units.
     * If it is negative then the entire parent's range will be used by this child scope.
     *
     * Before creating a child scope a parent scope must already be set up with {@link Base_ProgressScope#setRange setRange}.
     */
    constructor(theParentScope: Base_ProgressScope, theScaleInParentRange?: number);
    /**
     * Min value set with `setRange` or 0 by default.
     */
    get minValue(): number;
    /**
     * Max value set with `setRange` or 100 by default.
     */
    get maxValue(): number;
    /**
     * The value is in the range.
     */
    get value(): number;
    /**
     * Indicates close the scope when the `value` reaches the `maxValue` or not.
     * Default is `true`.
     */
    get autoClose(): boolean;
    set autoClose(theAutoClose: boolean);
    /**
     * The progress status which the progress scope created for.
     */
    get owner(): Base_ProgressStatus;
    /**
     * The range must be set before creating child scopes.
     * Default range is [0, 100].
     */
    setRange(theMin: number, theMax: number): void;
    /**
     * Increments the value by specific value.
     */
    increment(theStep?: number): void;
    /**
     * Closes current scope. Changes after closing is not applied to the parent scope.
     */
    close(): void;
}
/**
 * Defines function which returns a boolean value whether the operation has been canceled.
 * Once the checker has returned `true`, the value is stored and no further calls are made.
 *
 * The function must be fast enough as it can be called very frequently.
 *
 * > __Note__: this property performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_base___progress_status_1_1_cancellation_checker.html CancellationChecker} class.
 */
type Base_ProgressStatusCancellationChecker = () => boolean;
/**
 * Provides progress status and notification mechanism.
 *
 * Base_ProgressStatus has a current `value` which belongs to the range [0, 100]. The
 * algorithms that support progress status update increment this value during their execution.
 *
 * Upon value change the status object notifies its observers (subclasses of the
 * {@link Base_ProgressStatusObserver} class) registered with the `register` method.
 * Thus, it implements the observer design pattern.
 *
 * In the case there are no observers, the progress status incurs minimum overhead to the algorithm.
 *
 * Base_ProgressScope objects can be used to split progress status range into smaller scopes.
 * The scopes can be nested.
 */
class Base_ProgressStatus {
    /**
     * Creates an instance of Base_ProgressStatus.
     */
    constructor();
    /**
     * Minimum value of the progress status. By convention it's 0.
     */
    static readonly minValue: number;
    /**
     * Maximum value of the progress status. By convention it's 100.
     */
    static readonly maxValue: number;
    /**
     * Cancellation checker.
     */
    cancellationChecker: Base_ProgressStatusCancellationChecker | null;
    /**
     * Root progress scope.
     */
    get rootScope(): Base_ProgressScope;
    /**
     * Current value. The value is in the range [0, 100].
     */
    get value(): number;
    /**
     * Adds an observer that will be notified when the progress status has changed.
     *
     * Registered observers will receive notification of a new value of this status object
     * if the both following conditions are true:
     * * the difference between a value used in previous notification is equal to or greater than `theValueThreshold` and
     * * the time passed since previous notification is equal to or greater `theTimeThreshold` (specified in milliseconds).
     *
     * Changing these thresholds you can manipulate frequency of the notifications for each observer.
     */
    register(theObserver: Base_ProgressStatusObserver, theValueThreshold?: number, theTimeThreshold?: number): void;
    unregister(theObserver: Base_ProgressStatusObserver): void;
    /**
     * Sets the flag that the operation has been canceled.
     */
    cancel(): void;
    /**
     * Returns `true` if the operation has been canceled.
     */
    wasCanceled(): boolean;
}
abstract class Base_ProgressStatusObserver {
    abstract changedValue(theStatus: Base_ProgressStatus): void;
    abstract completed(theStatus: Base_ProgressStatus): void;
    canceled(theStatus: Base_ProgressStatus): void;
}
/**
 * Defines a Universally Unique IDentifier (UUID), also known as GUIDs (Globally Unique IDentifier).
 */
class Base_Uuid {
    /**
     * Creates an instance of Base_Uuid.
     */
    constructor(theData?: Array<number> | string | null);
    /**
     * Returns `true` if this is the null UUID {00000000-0000-0000-0000-000000000000}, otherwise returns `false`.
     */
    isNull(): boolean;
    /**
     * Creates copy of the current object.
     */
    clone(): Base_Uuid;
    /**
     * Copies uuid data from another one.
     */
    copy(theUuid: Base_Uuid): this;
    /**
     * Returns string representation of the uuid.
     */
    toString(): string;
    /**
     * Returns JSON representation of the uuid.
     */
    toJSON(): string;
}
/**
 * Computes bounding box of the objects.
 */
class ModelAlgo_BoundingBox {
    /**
     * Computes a bounding box of a representation.
     */
    static computeRepresentationBoundingBox(theRep: ModelData_Representation, theTransformation?: ModelData_Transformation | null, theTarget?: ModelData_Box | null): Promise<ModelData_Box>;
    /**
     * Computes a bounding box of a body.
     *
     * Deprecated. Use {@link ModelAlgo_BoundingBox.computeShapeBoundingBox} instead.
     * @deprecated
     */
    static computeBodyBoundingBox(theBody: ModelData_Body, theTransformation?: ModelData_Transformation | null, theTarget?: ModelData_Box | null): ModelData_Box;
    /**
     * Computes a bounding box of a B-Rep shape.
     */
    static computeShapeBoundingBox(theShape: ModelData_Shape, theTransformation?: ModelData_Transformation | null, theTarget?: ModelData_Box | null): ModelData_Box;
    /**
     * Computes a bounding box of a polygonal shape.
     */
    static computePolyShapeBoundingBox(thePolyShape: ModelData_PolyShape, theTransformation?: ModelData_Transformation | null, theTarget?: ModelData_Box | null): ModelData_Box;
}
class ModelAlgo_CurveLength {
    /**
     * Computes a length of the edge.
     * Returns a zero value for degenerated edge and a negative value if the length could not be computed.
     */
    computeEdgeLength(theEdge: ModelData_Edge): number;
}
/**
 * Computes validation properties of the objects.
 */
class ModelAlgo_ValidationProperty {
    /**
     * Returns a surface area of a triangle set.
     */
    static computeITSSurfaceArea(theITS: ModelData_IndexedTriangleSet): number;
    /**
     * Calculates surface area of the body.
     *
     * Deprecated. Use {@link ModelAlgo_ValidationProperty.computeShapeSurfaceArea} instead.
     * @deprecated
     */
    static computeBodySurfaceArea(theBody: ModelData_Body): number;
    /**
     * Returns a surface area of a shape.
     */
    static computeShapeSurfaceArea(theShape: ModelData_Shape): number;
    /**
     * Returns a surface area of a representation.
     */
    static computeRepresentationSurfaceArea(theRep: ModelData_Representation): Promise<number>;
}
/**
 * Provides an interface to appearance which is a collection of visual styles.
 */
class ModelData_Appearance extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_Appearance.
     */
    constructor(theObject?: ModelData_ColorObject | ModelData_MaterialObject);
    /**
     * Generic color assigned to appearance.
     */
    get genericColor(): ModelData_ColorObject | null;
    set genericColor(theGenericColor: ModelData_ColorObject | null);
    /**
     * Material object assigned to appearance.
     */
    get material(): ModelData_MaterialObject | null;
    set material(theMaterial: ModelData_MaterialObject | null);
    /**
     * Texture set assigned to appearance
     */
    get textureSet(): ModelData_TextureSet | null;
    set textureSet(theTextureSet: ModelData_TextureSet | null);
    /**
     * The line properties of appearance.
     */
    get lineProperties(): ModelData_LineProperties | null;
    set lineProperties(theLineProperties: ModelData_LineProperties | null);
    /**
     * The point properties of appearance.
     */
    get pointProperties(): ModelData_PointProperties | null;
    set pointProperties(thePointProperties: ModelData_PointProperties | null);
    /**
     * Updates appearance.
     */
    set(theObject: ModelData_ColorObject | ModelData_MaterialObject): void;
    /**
     * Adds texture to the appearance.
     */
    add(theTexture: ModelData_Texture): void;
    /**
     * Returns `true` if there is any style item which allows to retrieve a color.
     *
     * If there is a {@link ModelData_Appearance#genericColor genericColor} then theColor will be assigned its value.
     * Otherwise, if there is a {@link ModelData_Appearance#material material} then theColor will be assigned its diffuse color value.
     *
     * Otherwise returns `false`.
     */
    toColor(theColor: ModelData_ColorObject): boolean;
    /**
     * Returns `true` if there is any style item which allows to convert this appearance to a material.
     *
     * If there is a {@link ModelData_Appearance#material material} then theMaterial will be assigned its value.
     * If there is a {@link ModelData_Appearance#genericColor genericColor} then theMaterial will have diffuse color of that value and other default colors will have default values.
     *
     * Otherwise returns `false`.
     */
    toMaterial(theMaterial: ModelData_MaterialObject): boolean;
    /**
     * Copies internal contents and combines with set attributes from another appearance.
     */
    copyAndCombineWith(theAppearance: ModelData_Appearance): ModelData_Appearance;
    /**
     * Combines appearance with current.
     */
    combineWith(theAppearance: ModelData_Appearance): this;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Appearance;
    /**
     * Copies appearance params from another appearance.
     */
    copy(theAppearance: ModelData_Appearance): this;
    /**
     * Equality operator.
     * Returns `true` if both appearances have equal colors, materials and textures.
     */
    isEqual(theAppearance: ModelData_Appearance | null | undefined): boolean;
}
/**
 * Defines a group of scene graph element.
 */
class ModelData_Assembly extends ModelData_SceneGraphElement {
    /**
     * Creates an instance of ModelData_Assembly.
     */
    constructor(theName?: string | null);
    /**
     * Number of instances in this assembly.
     */
    get numberOfInstances(): number;
    /**
     * Returns iterable to added instances.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const instance of assembly.instances()) {
     *   // Access to the instance
     * }
     * ```
     */
    instances(): Iterable<ModelData_Instance>;
    /**
     * Adds a new child element into the assembly.
     * Assemblies may only have instances as direct children. If theElement is not an instance (i.e. is a part
     * or an assembly) then an interim instance is created which will refer to theElement.
     */
    addInstance(theElement: ModelData_SceneGraphElement, theTransformation?: ModelData_Transformation, theName?: string): ModelData_Instance | null;
    override accept(theVisitor: ModelData_SceneGraphElementVisitor): Promise<void>;
}
/**
 * Defines an axis placement as a combination of point and direction.
 */
class ModelData_Axis1Placement {
    /**
     * Creates an axis from origin point and direction.
     */
    constructor(theLocation?: ModelData_Point, theDirection?: ModelData_Direction);
    /**
     * Location point of the axis placement.
     */
    get location(): ModelData_Point;
    /**
     * Direction of the axis placement.
     */
    get direction(): ModelData_Direction;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Axis1Placement;
    /**
     * Copies axis params from another axis.
     */
    copy(theAxis: ModelData_Axis1Placement): this;
}
/**
 * Defines a right-hand axis placement in 3D.
 *
 * An axis placement is defined by a location point, a Z-direction and a reference direction (XRef).
 *
 * The Z-direction defines a plane perpendicular to Z. Other directions are defined as follows:
 * * the Y-direction is defined as a cross-product of the Z-direction and the reference direction;
 * * the X-direction is defined as a cross-product of the Y-direction and the Z-direction.
 *
 * X,Y,Z axes always define a right-hand axis placement.
 */
class ModelData_Axis2Placement {
    /**
     * Creates an axis from origin point, direction and a referenced direction.
     */
    constructor(theLocation: ModelData_Point, theDirection: ModelData_Direction, theRefDirection: ModelData_Direction);
    /**
     * Location point of the axis placement.
     */
    get location(): ModelData_Point;
    /**
     * Z-direction of the axis placement.
     */
    get axis(): ModelData_Direction;
    /**
     * X-direction of the axis placement.
     */
    get xDirection(): ModelData_Direction;
    /**
     * Y-direction of the axis placement.
     */
    get yDirection(): ModelData_Direction;
}
/**
 * An axis placement is defined by a location point, an X-direction and a flag
 * whether the placement is right-handed or left-handed.
 */
class ModelData_Axis2Placement2d {
    constructor(theLocation?: ModelData_Point2d, theRefDirection?: ModelData_Direction2d, theCCW?: boolean);
    /**
     * The location point.
     */
    get location(): ModelData_Point2d;
    /**
     * X-direction.
     */
    get xDirection(): ModelData_Direction2d;
    /**
     * Y-direction.
     */
    get yDirection(): ModelData_Direction2d;
}
/**
 * Defines a right-handed or left-handed axis placement in 3D.
 *
 * An axis placement is defined similar to ModelData_Axis2Placement, however its directions
 * can be reversed independently.
 */
class ModelData_Axis3Placement {
    /**
     * Creates an axis from origin point, direction and a referenced direction.
     */
    constructor(theLocation?: ModelData_Point, theDirection?: ModelData_Direction, theRefDirection?: ModelData_Direction);
    /**
     * Location point of the axis placement.
     */
    get location(): ModelData_Point;
    /**
     * Z-direction of the axis placement.
     */
    get axis(): ModelData_Direction;
    /**
     * X-direction of the axis placement.
     */
    get xDirection(): ModelData_Direction;
    /**
     * Y-direction of the axis placement.
     */
    get yDirection(): ModelData_Direction;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Axis3Placement;
    /**
     * Copies axis params from another axis.
     */
    copy(theAx3: ModelData_Axis3Placement): this;
}
/**
 * Base class for all Objects.
 */
class ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_BaseObject.
     */
    constructor(theName?: string | null, theUuid?: Base_Uuid | null);
    /**
     * Object name. `null` if the object has no name (by default).
     */
    name: string | null;
    /**
     * Object uuid. `null` if the object has no uuid (by default).
     */
    uuid: Base_Uuid | null;
}
/**
 * Base class for other BIM elements, including host elements and construction elements.
 */
abstract class ModelData_BIMBaseObject {
    /**
     * An element name. `null` if the element has no name (by default).
     */
    name: string | null;
    /**
     * An element uuid. `null` if the object has no uuid (by default).
     */
    uuid: Base_Uuid | null;
    /**
     * The object for managing element properties.
     */
    get properties(): ModelData_BIMPropertyLibrary;
    /**
     * Accepts a visitor.
     */
    abstract accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Beam represents a structural element that resists bending or laterally loads.
 */
class ModelData_BIMBeam extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMBeam, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of beam.
     */
    get subType(): ModelData_BIMBeamSubType;
    set subType(theSubType: ModelData_BIMBeamSubType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the sub types of the BIM beam.
 */
enum ModelData_BIMBeamSubType {
    Undefined = 0,
    General = 1,
    Cornice = 2,
    Diaphragm = 3,
    Edgebeam = 4,
    GirderSegment = 5,
    Hatstone = 6,
    Hollowcore = 7,
    Joist = 8,
    Lintel = 9,
    Piercap = 10,
    Spandrel = 11,
    TBeam = 12,
}
/**
 * Building is an element of spatial structure hierarchy (together with site, storey, and space)
 * that represents a building within a construction project.
 */
class ModelData_BIMBuilding extends ModelData_BIMHost<ModelData_BIMStorey|ModelData_BIMElement> {
    /**
     * Adds an element to this building.
     */
    override add(theElement: ModelData_BIMStorey | ModelData_BIMElement): void;
    accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Column represents structural or architectural vertical element,
 * that can transmit loads from top elements to bottom.
 */
class ModelData_BIMColumn extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMColumn, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of column.
     */
    get subType(): ModelData_BIMColumnSubType;
    set subType(theSubType: ModelData_BIMColumnSubType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the sub types of the BIM column.
 */
enum ModelData_BIMColumnSubType {
    Undefined = 0,
    General = 1,
    Pierstem = 2,
    PierstemSegment = 3,
    Pilaster = 4,
    StandColumn = 5,
}
/**
 * Composite Element represents the complex BIM Element, consisting of several other BIM Elements.
 * Potentially it can contain other Composite Elements representing hierarchical structure.
 */
class ModelData_BIMCompositeElement extends ModelData_BIMElement {
    /**
     * The number of contained elements.
     */
    get numberOfElements(): number;
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
    /**
     * Adds an element to this composite element.
     */
    add(theElement: ModelData_BIMElement): void;
    /**
     * Returns iterator over contained elements.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const element of compositeElement.elements()) {
     *   // Access to the BIM element
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_i_m_composite_element_1_1_element_iterator.html ModelData_BIMCompositeElement::ElementIterator} class.
     */
    elements(): IterableIterator<ModelData_BIMElement>;
}
/**
 * Base class for concrete construction elements (wall, window, door, etc.),
 * that can have own geometry representation, assigned material and openings set.
 */
abstract class ModelData_BIMConstructionElement extends ModelData_BIMGeometryElement {
}
/**
 * CustomGeometryElement represents general bim element with geometry
 * for defining elements that have no more suitable BIM class for their description.
 */
class ModelData_BIMCustomGeometryElement extends ModelData_BIMGeometryElement {
    constructor(thePrototype?: ModelData_BIMCustomGeometryElement, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of custom geometry element.
     */
    get type(): string;
    set type(theCustomGeometryElement: string);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Door element can be used for represent standard
 * entrance or internal doors, gates, trapdoors, turnstiles.
 */
class ModelData_BIMDoor extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMDoor, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of door.
     */
    get subType(): ModelData_BIMDoorSubType;
    set subType(theSubType: ModelData_BIMDoorSubType);
    /**
     * The operation type of door.
     */
    get operationType(): ModelData_BIMDoorOperationType;
    set operationType(theOperationType: ModelData_BIMDoorOperationType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the operation types of the BIM door.
 */
enum ModelData_BIMDoorOperationType {
    Undefined = 0,
    DoublePanelDoubleSwing = 1,
    DoublePanelFolding = 2,
    DoublePanelLiftingVertical = 3,
    DoublePanelSingleSwing = 4,
    DoublePanelSingleSwingOppositeLeft = 5,
    DoublePanelSingleSwingOppositeRight = 6,
    DoublePanelSliding = 7,
    DoubleSwingLeft = 8,
    DoubleSwingRight = 9,
    FoldingToLeft = 10,
    FoldingToRight = 11,
    LiftingHorizontal = 12,
    LiftingVerticalLeft = 13,
    LiftingVerticalRight = 14,
    RevolvingHorizontal = 15,
    RevolvingVertical = 16,
    RollingUp = 17,
    SingleSwingLeft = 18,
    SingleSwingRight = 19,
    SlidingToLeft = 20,
    SlidingToRight = 21,
    SwingFixedLeft = 22,
    SwingFixedRight = 23,
}
/**
 * Defines the sub type of the BIM door.
 */
enum ModelData_BIMDoorSubType {
    Undefined = 0,
    General = 1,
    BoomBarrier = 2,
    Gate = 3,
    Trapdoor = 4,
    Turnstile = 5,
}
/**
 * Base class for element that represents a single BIM entity.
 */
abstract class ModelData_BIMElement extends ModelData_BIMBaseObject {
}
/**
 * Furniture represents element for defining furnishings such as a table, chair, cabinet, etc.
 */
class ModelData_BIMFurniture extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMFurniture, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of furniture.
     */
    get subType(): ModelData_BIMFurnitureSubType;
    set subType(theSubType: ModelData_BIMFurnitureSubType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the sub types of the BIM furniture.
 */
enum ModelData_BIMFurnitureSubType {
    Undefined = 0,
    Bed = 1,
    Chair = 2,
    Desk = 3,
    FileCabinet = 4,
    Shelf = 5,
    Sofa = 6,
    Table = 7,
    TechnicalCabinet = 8,
}
/**
 * Base class for geometry elements (wall, window, door, etc.),
 * that can have own geometry representation, assigned material and transformation.
 */
abstract class ModelData_BIMGeometryElement extends ModelData_BIMElement {
    /**
     * The element transformation.
     */
    transformation: ModelData_Transformation;
    /**
     * The element geometry.
     */
    get geometry(): ModelData_Model;
    set geometry(theGeometry: ModelData_Model);
    /**
     * The element material.
     */
    get material(): ModelData_BIMMaterial | null;
    set material(theMaterial: ModelData_BIMMaterial | null);
    /**
     * The element prototype.
     */
    get prototype(): ModelData_BIMGeometryElement | null;
}
/**
 * Host represents a base class for BIM elements that represent BIM structure by containing nested (child) elements.
 * Host is a base class for concrete host elements (site, building, storey).
 */
abstract class ModelData_BIMHost<T extends ModelData_BIMBaseObject = ModelData_BIMBaseObject> extends ModelData_BIMBaseObject {
    /**
     * The number of contained elements.
     */
    get numberOfElements(): number;
    /**
     * Adds an element.
     */
    abstract add(theElement: T): void;
    /**
     * Returns iterator over contained objects.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const object of hostElement.objects()) {
     *   // Access to the BIM object
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link http://docs/sdk/classcadex_1_1_model_data___b_i_m_host_1_1_object_iterator.html ModelData_BIMHost::ObjectIterator} class.
     */
    objects(): IterableIterator<T>;
}
/**
 * Material represents a substance that can be used to form physical elements,
 * e.g particular cement, steel, plastic, wood, etc.
 * Material entity can be attached to Construction Element instance.
 * Material can be defined with:
 * - text data (name, description and category);
 * - appearance data (with ModelData_Appearance approach);
 * - arbitrary data (with ModelData_BIMPropertyLibrary approach) -
 * for attaching different physical based properties (e.g. mass density, porosity, Young modulus, etc.)
 */
class ModelData_BIMMaterial {
    /**
     * The name of material.
     */
    name: string;
    /**
     * The description of material.
     */
    description: string;
    /**
     * The category of material.
     */
    category: string;
    /**
     * The material properties.
     */
    properties: ModelData_BIMPropertyLibrary;
    /**
     * The material appearance.
     * Material appearance can influence the elements with this material.
     */
    appearance: ModelData_Appearance | null;
}
/**
 * Provides CAD Exchanger BIM data model.
 *
 * The BIM data model encapsulates data about BIM elements and their hierarchy and is thus an entry
 * point to entire BIM model data, including elements, geometry, properties, materials.
 */
class ModelData_BIMModel {
    /**
     * An model name. `null` if the element has no name (by default).
     */
    name: string | null;
    /**
     * The root BIM element as entry point to BIM elements hierarchy.
     */
    root: ModelData_BIMSite;
    /**
     * BIM model properties.
     */
    get properties(): ModelData_BIMPropertyLibrary;
    /**
     * Adds BIM model properties.
     */
    addProperties(theProperties: ModelData_BIMPropertyLibrary): void;
    accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
    /**
     * Loads CDXBIMWEB model with specified file name.
     * All necessary data will be requested using `dataProvider`.
     */
    load(theFileName: string, dataProvider: ModelData_ExternalDataProvider, theProgressScope?: Base_ProgressScope): Promise<boolean>;
    /**
     * Loads CDXBIMWEB model from the buffer.
     * The external referenced data will be requested using `dataProvider`.
     */
    read(theFileData: ModelData_ExternalDataType, dataProvider: ModelData_ExternalDataProvider, theProgressScope?: Base_ProgressScope): boolean;
}
/**
 * Plate represents planar element, that is often add-on part for wall, roof or slab.
 */
class ModelData_BIMPlate extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMPlate, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of plate.
     */
    get subType(): ModelData_BIMPlateSubType;
    set subType(theSubType: ModelData_BIMPlateSubType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the sub types of the BIM plate.
 */
enum ModelData_BIMPlateSubType {
    Undefined = 0,
    BasePlate = 1,
    CoverPlate = 2,
    CurtainPanel = 3,
    FlangePlate = 4,
    GussetPlate = 5,
    Sheet = 6,
    SplicePlate = 7,
    StiffenerPlate = 8,
    WebPlate = 9,
}
/**
 * PropertyLibrary represents a way to manage properties of elements, materials or openings.
 * PropertyLibrary provides access to set of named property tables.
 */
class ModelData_BIMPropertyLibrary {
    /**
     * The number of property tables contained in.
     */
    numberOfTables: number;
    /**
     * Adds the property table.
     */
    add(theTable: ModelData_PropertyTable): void;
    /**
     * Adds all property tables from the library.
     */
    addLibrary(theLibrary: ModelData_BIMPropertyLibrary): void;
    /**
     * Gets the property table with specified name.
     * If property library has no a table with specified name, returns `undefined`;
     */
    table(theName: string): ModelData_PropertyTable | undefined;
    /**
     * Returns iterable to tables.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const table of propertyLibrary.tables()) {
     *   // Access to the property table
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_i_m_property_library.html ModelData_BIMPropertyLibrary::PropertyTableIterator} class.
     */
    tables(): IterableIterator<ModelData_PropertyTable>;
    /**
     * Finds the property with specified name in all property tables contained.
     *
     * Uses value of the first found occurrence.
     * For retrieving property from concrete table use `table()` method.
     */
    findProperty(theName: string): string | number | Date | ModelData_Point | ModelData_Box | undefined;
}
/**
 * Railing represents a frame assembly for physical support,
 * fall prevention and also decorating.
 */
class ModelData_BIMRailing extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMRailing, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of railing.
     */
    get subType(): ModelData_BIMRailingSubType;
    set subType(theSubType: ModelData_BIMRailingSubType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the sub types of the BIM railing.
 */
enum ModelData_BIMRailingSubType {
    Undefined = 0,
    Balustrade = 1,
    Fence = 2,
    GuardRail = 3,
    HandRail = 4,
}
/**
 * Roof represents covering element of the top part of a building.
 */
class ModelData_BIMRoof extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMRoof, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of roof.
     */
    get subType(): ModelData_BIMRoofSubType;
    set subType(theSubType: ModelData_BIMRoofSubType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the sub types of the BIM roof.
 */
enum ModelData_BIMRoofSubType {
    Undefined = 0,
    BarrelRoof = 1,
    ButterflyRoof = 2,
    DomeRoof = 3,
    FlatRoof = 4,
    Freeform = 5,
    GableRoof = 6,
    GambrelRoof = 7,
    HippedGableRoof = 8,
    HipRoof = 9,
    MansardRoof = 10,
    PavilionRoof = 11,
    RainbowRoof = 12,
    ShedRoof = 13,
}
/**
 * Site represents the area of land, on which the building construction is to be.
 * Often, the site element also includes the additional construction element with
 * geometry of the surrounding land.
 */
class ModelData_BIMSite extends ModelData_BIMHost<ModelData_BIMBuilding|ModelData_BIMElement> {
    /**
     * The elevation of site.
     */
    elevation: number;
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
    /**
     * Adds an element to this building.
     */
    override add(theElement: ModelData_BIMBuilding | ModelData_BIMElement): void;
}
/**
 * Slab represents an element that encloses a building space vertically.
 * The slab may provide the floor or roof support in building spaces.
 */
class ModelData_BIMSlab extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMSlab, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of slab.
     */
    get subType(): ModelData_BIMSlabSubType;
    set subType(theSubType: ModelData_BIMSlabSubType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the sub types of the BIM slab.
 */
enum ModelData_BIMSlabSubType {
    Undefined = 0,
    ApproachSlab = 1,
    BaseSlab = 2,
    Floor = 3,
    Landing = 4,
    Paving = 5,
    Roof = 6,
    Sidewalk = 7,
    TrackSlab = 8,
    Wearing = 9,
}
/**
 * Stair represents a passageway allowing persons to step
 * from one level to another level at a different elevation.
 */
class ModelData_BIMStair extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMStair, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of stair.
     */
    get subType(): ModelData_BIMStairSubType;
    set subType(theSubType: ModelData_BIMStairSubType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the sub types of the BIM stair.
 */
enum ModelData_BIMStairSubType {
    Undefined = 0,
    CurvedRunStair = 1,
    DoubleReturnStair = 2,
    HalfTurnStair = 3,
    HalfWindingStair = 4,
    Ladder = 5,
    QuarterTurnStair = 6,
    QuarterWindingStair = 7,
    SpiralStair = 8,
    StraightRunStair = 9,
    ThreeQuarterTurnStair = 10,
    ThreeQuarterWindingStair = 11,
    TwoCurvedRunStair = 12,
    TwoQuarterTurnStair = 13,
    TwoQuarterWindingStair = 14,
    TwoStraightRunStair = 15,
}
/**
 * Storey represents a horizontal aggregation of building spaces
 * that are vertically bound and are all at a specific elevation.
 * Storey is used to build the spatial structure of a building.
 */
class ModelData_BIMStorey extends ModelData_BIMHost<ModelData_BIMElement> {
    /**
     * The elevation of storey relative to building element.
     */
    elevation: number;
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
    /**
     * Adds an element to this storey.
     */
    override add(theElement: ModelData_BIMElement): void;
}
/**
 * Defines a BIM elements visitor.
 *
 * The Visitor follows a hierarchical visitor pattern (see
 * http://c2.com/cgi/wiki?HierarchicalVisitorPattern) what enables to track entering
 * and leaving composite entities - host elements (site, building, storey, space).
 *
 * Methods `visitEnter()` returns `true` if the child elements of the host element
 * should be visited. If the method returns `false` then they will not be visited.
 * The method `visitLeave()` will always be called.
 */
abstract class ModelData_BIMVisitor {
    abstract visitBeam(theElement: ModelData_BIMBeam): Promise<void> | void;
    abstract visitColumn(theElement: ModelData_BIMColumn): Promise<void> | void;
    abstract visitDoor(theElement: ModelData_BIMDoor): Promise<void> | void;
    abstract visitFurniture(theElement: ModelData_BIMFurniture): Promise<void> | void;
    abstract visitPlate(theElement: ModelData_BIMPlate): Promise<void> | void;
    abstract visitRailing(theElement: ModelData_BIMRailing): Promise<void> | void;
    abstract visitRoof(theElement: ModelData_BIMRoof): Promise<void> | void;
    abstract visitSlab(theElement: ModelData_BIMSlab): Promise<void> | void;
    abstract visitStair(theElement: ModelData_BIMStair): Promise<void> | void;
    abstract visitWall(theElement: ModelData_BIMWall): Promise<void> | void;
    abstract visitWindow(theElement: ModelData_BIMWindow): Promise<void> | void;
    abstract visitCustomGeometryElement(theElement: ModelData_BIMCustomGeometryElement): Promise<void> | void;
    abstract visitCompositeElementEnter(theElement: ModelData_BIMCompositeElement): Promise<boolean> | boolean;
    abstract visitCompositeElementLeave(theElement: ModelData_BIMCompositeElement): Promise<void> | void;
    abstract visitBuildingEnter(theElement: ModelData_BIMBuilding): Promise<boolean> | boolean;
    abstract visitBuildingLeave(theElement: ModelData_BIMBuilding): Promise<void> | void;
    abstract visitSiteEnter(theElement: ModelData_BIMSite): Promise<boolean> | boolean;
    abstract visitSiteLeave(theElement: ModelData_BIMSite): Promise<void> | void;
    abstract visitStoreyEnter(theElement: ModelData_BIMStorey): Promise<boolean> | boolean;
    abstract visitStoreyLeave(theElement: ModelData_BIMStorey): Promise<void> | void;
}
/**
 * Wall represents an element that bounds or subdivides building spaces.
 * Wall are usually vertical, planar structural element, that may bear loads.
 */
class ModelData_BIMWall extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMWall, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of wall.
     */
    get subType(): ModelData_BIMWallSubType;
    set subType(theSubType: ModelData_BIMWallSubType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the sub types of the BIM wall.
 */
enum ModelData_BIMWallSubType {
    Undefined = 0,
    Movable = 1,
    Parapet = 2,
    Partitioning = 3,
    PlumbingWall = 4,
    RetainingWall = 5,
    Shear = 6,
    SolidWall = 7,
    Standard = 8,
    WaveWall = 9,
}
/**
 * Window element can represent vertical, sloped or horizontal window in the wall or roof,
 * and also a window panel in a curtain wall.
 */
class ModelData_BIMWindow extends ModelData_BIMConstructionElement {
    constructor(thePrototype?: ModelData_BIMWindow, theTransformation?: ModelData_Transformation);
    /**
     * The subtype of window.
     */
    get subType(): ModelData_BIMWindowSubType;
    set subType(theSubType: ModelData_BIMWindowSubType);
    /**
     * The partition type of window.
     */
    get partitioningType(): ModelData_BIMWindowPartitioningType;
    set partitioningType(thePartitioningType: ModelData_BIMWindowPartitioningType);
    override accept(theVisitor: ModelData_BIMVisitor): Promise<void>;
}
/**
 * Defines the partitioning types of the BIM window.
 */
enum ModelData_BIMWindowPartitioningType {
    Undefined = 0,
    DoublePanelHorizontal = 1,
    DoublePanelVertical = 2,
    SinglePanel = 3,
    TriplePanelBottom = 4,
    TriplePanelHorizontal = 5,
    TriplePanelLeft = 6,
    TriplePanelRight = 7,
    TriplePanelTop = 8,
    TriplePanelVertical = 9,
}
/**
 * Defines the sub types of the BIM window.
 */
enum ModelData_BIMWindowSubType {
    Undefined = 0,
    General = 1,
    Lightdome = 2,
    Skylight = 3,
}
/**
 * Defines a root topological shape that can be owned by {@link ModelData_BRepRepresentation B-Rep representation}.
 *
 * Depending on its type, a body can hold either:
 * * acorn body - {@link ModelData_Vertex}
 * * solid body - {@link ModelData_Solid} (which can be a solid with voids),
 * * wireframe body - {@link ModelData_Edge}, {@link ModelData_Wire} or a group thereof,
 * * sheet body - {@link ModelData_Face}, {@link ModelData_Shell} or a group thereof
 */
abstract class ModelData_Body extends ModelData_Shape {
    /**
     * Body type.
     */
    get bodyType(): ModelData_BodyType;
    /**
     * Creates a body from an arbitrary shape.
     * If `theShape` is already a body then just returns it.
     * If `theShape` is a body list then returns its first body or empty, if the list is empty.
     * Otherwise creates a body wrapping `theShape`.
     */
    static create(theShape: ModelData_Shape): ModelData_Body | null;
    /**
     * Adds a shape to the body.
     * Returns `true` if the shape has been successfully added and `false` otherwise.
     * Result depends upon the existing body contents and the shape being added. The check are applied in the following order:
     * * if the body is either solid or acorn then returns `false`;
     * * if `theShape` is simple (i.e. from vertex to solid) and the body is either wireframe or sheet
     *      then either appends the shape and returns `true` (if the types of the shape and body are compatible)
     *      or returns `false` (if the types are incompatible).
     * * if `theShape` is a body of the same type as this body, then adds its components and returns `true`.
     *      Returns `false` if the types are incompatible.
     * * if `theShape` is a body list then tries to add its bodies using the above approach.
     *      Returns `true` if at least one body succeeded.
     */
    append(theShape: ModelData_Shape): boolean;
}
/**
 * Defines a list of bodies.
 * Body list is used by {@link ModelData_BRepRepresentation} to hold root bodies.
 */
class ModelData_BodyList extends ModelData_Shape {
    /**
     * Adds body to the body list.
     */
    append(theBody: ModelData_Body): void;
    /**
     * Returns the number of added bodies.
     */
    size(): number;
    /**
     * Retrieves body by index.
     */
    element(theIndex: number): ModelData_Body | undefined;
    [Symbol.iterator](): Iterator<ModelData_Body>;
}
/**
 * Defines poly presentation of body.
 * Contains multiple {@link ModelData_PolyVertexSet PolyVertexSets} for different sub-shape types.
 */
interface ModelData_BodyPrs {
    faces: Array<ModelData_IndexedTriangleSet> | undefined;
    edges: Array<ModelData_PolyLineSet> | undefined;
    vertexes: Array<ModelData_PolyPointSet> | undefined;
}
/**
 * Defines a body type.
 */
enum ModelData_BodyType {
    /**
     * Unknown body type
     */
    Undefined = 0,
    /**
     * Acorn body type
     */
    Acorn = 1,
    /**
     * Wireframe body type
     */
    Wireframe = 2,
    /**
     * Sheet body type
     */
    Sheet = 3,
    /**
     * Solid body type
     */
    Solid = 4,
}
/**
 * Defines a 3D axis-aligned bounding box.
 *
 * The box is defined via two corners - {@link ModelData_Box#minCorner} and {@link ModelData_Box#maxCorner}
 * - containing minimum and maximum coordinates respectively.
 */
class ModelData_Box {
    /**
     * Creates a box with specified minimum and maximum corners.
     */
    constructor(theMinCorner?: ModelData_XYZ, theMaxCorner?: ModelData_XYZ);
    /**
     * Minimum corner of the bounding box.
     */
    minCorner: ModelData_Point;
    /**
     * Maximum corner of the bounding box.
     */
    maxCorner: ModelData_Point;
    /**
     * Returns the length of range along X axis.
     */
    xRange(): number;
    /**
     * Returns the length of range along Y axis.
     */
    yRange(): number;
    /**
     * Returns the length of range along Z axis.
     */
    zRange(): number;
    /**
     * Returns a middle of the diagonal between `minCorner` and `maxCorner`.
     */
    getCenter(theTarget?: ModelData_Point): ModelData_Point;
    /**
     * Accumulates another bounding box or point.
     */
    add(theObject: ModelData_Point | ModelData_Box | Object): this;
    /**
     * Returns `true` if at least one coordinate is infinite.
     */
    isInfinite(): boolean;
    /**
     * Returns `true` if the box includes zero points within its bounds.
     * > __Note__: a box with equal min and max corners still includes one point, the one both corners share.
     */
    isEmpty(): boolean;
    /**
     * Resets the bounding box.
     */
    clear(): this;
    /**
     * Compares two boxes with specified tolerance.
     */
    isEqual(theOther: ModelData_Box, theTolerance?: number): boolean;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Box;
    /**
     * Copies values from another box.
     */
    copy(theBox: ModelData_Box | Object): this;
    /**
     * Returns string representation of the box.
     */
    toString(): string;
    /**
     * Returns JSON representation of the box.
     */
    toJSON(): Object;
}
/**
 * Defines a 2D bounding box.
 *
 * The box is defined via two corners - {@link ModelData_Box2d#minCorner} and {@link ModelData_Box2d#maxCorner}
 * - containing minimum and maximum coordinates respectively.
 */
class ModelData_Box2d {
    /**
     * Creates a box with specified minimum and maximum corners.
     */
    constructor(theMinCorner?: ModelData_XY, theMaxCorner?: ModelData_XY);
    /**
     * Minimum corner of the bounding box.
     */
    minCorner: ModelData_Point2d;
    /**
     * Maximum corner of the bounding box.
     */
    maxCorner: ModelData_Point2d;
    /**
     * Returns the length of range along X axis.
     */
    xRange(): number;
    /**
     * Returns the length of range along Y axis.
     */
    yRange(): number;
    /**
     * Returns a middle of the diagonal between `minCorner` and `maxCorner`.
     */
    getCenter(theTarget?: ModelData_Point2d): ModelData_Point2d;
    /**
     * Accumulates another bounding box or point.
     */
    add(theObject: ModelData_Point2d | ModelData_Box2d | Object): this;
    /**
     * Returns `true` if at least one coordinate is infinite.
     */
    isInfinite(): boolean;
    /**
     * Returns `true` if the box includes zero points within its bounds.
     * > __Note__: a box with equal min and max corners still includes one point, the one both corners share.
     */
    isEmpty(): boolean;
    /**
     * Resets the bounding box.
     */
    clear(): this;
    /**
     * Compares two boxes with specified tolerance.
     */
    isEqual(theOther: ModelData_Box2d, theTolerance?: number): boolean;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Box2d;
    /**
     * Copies values from another box.
     */
    copy(theBox: ModelData_Box2d | Object): this;
    /**
     * Returns string representation of the box.
     */
    toString(): string;
    /**
     * Returns JSON representation of the box.
     */
    toJSON(): Object;
}
/**
 * Defines a visitor of the subshapes.
 */
abstract class ModelData_BRepRepresentationSubshapeVisitor {
    abstract visit(theShape: ModelData_Shape): void;
}
/**
 * Provides {@link ModelData_BRepRepresentation B-Rep representation}.
 *
 * Used to defer construction of the B-Rep representation and thereby to reduce peak memory footprint.
 */
abstract class ModelData_BRepRepresentationProvider {
    /**
     * Feeds the representation.
     * This method can depend of the network request which makes asynchronously, so it should return Promise.
     * Subclasses must redefine this method to call {@link ModelData_BRepRepresentation#add add()}
     * of theRepresentation.
     */
    abstract feed(theRepresentation: ModelData_BRepRepresentation, theScope: Base_ProgressScope): Promise<void>;
}
/**
 * Defines B-Rep representation of part.
 * A part corresponds to mechanical part or a product in the product structure hierarchy.
 * A part contains one or several representations: zero or one B-Rep representation, and/or
 * zero, one or multiple polygonal representations.
 */
class ModelData_BRepRepresentation extends ModelData_Representation {
    /**
     * Creates an instance of ModelData_BRepRepresentation.
     */
    constructor(theShape?: ModelData_Shape);
    /**
     * Number of subshapes.
     */
    get numberOfSubshapes(): number;
    override accept(theVisitor: ModelData_RepresentationVisitor): Promise<void>;
    /**
     * Adds a root body (or bodies).
     *
     * If `theShape` is not a body or a body list then creates a body that holds `theShape`,
     * adds it to the list of root bodies and returns it.
     *
     * If `theShape` is a body then appends it to the list of root bodies,
     * if it is a body list then appends all its bodies to the root list;
     * returns `theShape` in either above case.
     */
    add(theShape: ModelData_Shape): ModelData_Shape;
    /**
     * Adds B-Rep representations provider.
     */
    addProvider(theProvider: ModelData_BRepRepresentationProvider): void;
    /**
     * Returns bodies list. This method flushes providers.
     */
    bodyList(theScope?: Base_ProgressScope): Promise<ModelData_BodyList>;
    /**
     * Returns a B-Rep shape by its id.
     * If `theId` is negative or there is no corresponding shape in this representation, then returns null.
     */
    shape(theId: number): ModelData_Shape | null;
    /**
     * Returns an id of B-Rep shape.
     * If the shape does not belong to B-Rep, then returns -1.
     */
    shapeId(theShape: ModelData_Shape): number;
    /**
     * Returns `true` if BRep representations has subshapes.
     */
    hasSubshapes(): boolean;
    /**
     * Returns iterable to subshapes.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const shape of bRepRep.subshapes()) {
     *   // Access to the subshape
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_rep_representation_1_1_subshape_iterator.html ModelData_BRepRepresentation::SubshapeIterator} class.
     */
    subshapes(): Promise<Iterable<ModelData_Shape>>;
    /**
     * Accepts a subshape visitor.
     */
    acceptSubshapeVisitor(theVisitor: ModelData_BRepRepresentationSubshapeVisitor): void;
    /**
     * Add the PMI data to the subshape.
     */
    addPMIData(theShape: ModelData_Shape, thePMIData: ModelData_PMIData): void;
    /**
     * Returns iterable to added PMI data associated with the subshapes.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const shape of bRepRep.subshapes()) {
     *   for (const pmiData of bRepRep.pmiData(shape)) {
     *      // Access to the PMI data
     *   }
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_rep_representation_1_1_p_m_i_data_iterator.html ModelData_BRepRepresentation::PMIDataIterator} class.
     */
    pmiData(theShape: ModelData_Shape): Iterable<ModelData_PMIData>;
    /**
     * Returns name associated with the subshape.
     *
     * This method corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_rep_representation.html ModelData_BRepRepresentation::Name} method.
     */
    shapeName(theShape: ModelData_Shape): string | undefined;
    /**
     * Associates name with the subshape.
     *
     * > *Note*: `theShape` may only be a {@link ModelData_Body} or its subshape. {@link ModelData_BodyList} will be ignored.
     *
     * This method corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_rep_representation.html ModelData_BRepRepresentation::SetName} method.
     */
    setShapeName(theShape: ModelData_Shape, name: string | undefined): void;
    /**
     * Returns appearance associated with the subshape.
     *
     * This method corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_rep_representation.html ModelData_BRepRepresentation::Appearance} method.
     */
    shapeAppearance(theShape: ModelData_Shape): ModelData_Appearance | undefined;
    /**
     * Associates appearance with the subshape.
     *
     * > *Note*: `theShape` may only be a {@link ModelData_Body} or its subshape. {@link ModelData_BodyList} will be iterated into its bodies.
     *
     * This method corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_rep_representation.html ModelData_BRepRepresentation::SetAppearance } method.
     */
    setShapeAppearance(theShape: ModelData_Shape, theAppearance: ModelData_Appearance | undefined): void;
    /**
     * Returns PropertyTable associated with the subshape.
     *
     * This method corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_rep_representation.html ModelData_BRepRepresentation::PropertyTable} method.
     */
    shapePropertyTable(theShape: ModelData_Shape): ModelData_PropertyTable | undefined;
    /**
     * Returns PropertyTable associated with the subshape.
     *
     * > *Note*: `theShape` may only be a {@link ModelData_Body} or its subshape. {@link ModelData_BodyList} will be iterated into its bodies.
     *
     * This method corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___b_rep_representation.html ModelData_BRepRepresentation::SetPropertyTable} method.
     */
    setShapePropertyTable(theShape: ModelData_Shape, thePropertyTable: ModelData_PropertyTable | undefined): void;
    /**
     * Returns body poly presentation.
     */
    bodyPrs(theBody: ModelData_Body): ModelData_BodyPrs;
}
/**
 * Defines an RGBA color (with alpha channel).
 * Components must be within a [0, 1] range.
 */
class ModelData_ColorObject extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_Color. Each value must be within [0, 1] range.
     */
    constructor(r?: number, g?: number, b?: number, a?: number);
    /**
     * Precision of color components.
     */
    static readonly precision: number;
    /**
     * Red component of the color. Value must be within [0, 1] range.
     */
    get r(): number;
    set r(theValue: number);
    /**
     * Green component of the color. Value must be within [0, 1] range.
     */
    get g(): number;
    set g(theValue: number);
    /**
     * Blue component of the color. Value must be within [0, 1] range.
     */
    get b(): number;
    set b(theValue: number);
    /**
     * Alpha component of the color. Value must be within [0, 1] range.
     */
    get a(): number;
    set a(theValue: number);
    /**
     * Creates color from integer which represents RGB.
     */
    static fromHex(theHex: number, a?: number): ModelData_ColorObject;
    /**
     * Returns component value by index.
     */
    value(theIndex: number): number;
    /**
     * Change component value by index.
     */
    setValue(theIndex: number, theValue: number): void;
    /**
     * Set color components.
     */
    set(r?: number, g?: number, b?: number, a?: number): this;
    /**
     * Set color from integer which represents RGB. Alpha component will not be changed.
     */
    setHex(theHex: number): this;
    /**
     * Returns the hexadecimal value of this color which represents RGB.
     */
    getHex(): number;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_ColorObject;
    /**
     * Copies values from another color to current.
     */
    copy(theColor: ModelData_ColorObject): this;
    /**
     * Returns string representation of the color.
     */
    toString(): string;
    /**
     * Returns JSON representation of the color.
     */
    toJSON(): Object;
    /**
     * Equality operator.
     * Returns `true` if all color components are the same or
     * differences between the component do not exceed {@link ModelData_ColorObject.precision precision}.
     */
    isEqual(theColor: ModelData_ColorObject | null | undefined): boolean;
}
/**
 * Base class for 3D curves.
 *
 * 3D curves are used to represent curves in 3D space. Each non-degenerated
 * {@link ModelData_Edge "edge"} must refer to a 3D curve.
 *
 * > __Note__: This class is defined for SDK API compatibility only.
 * Unlike {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___curve.html C++ SDK Curve}
 * it contains only type of the curve.
 */
abstract class ModelData_Curve {
    /**
     * Curve type.
     */
    get type(): ModelData_CurveType;
}
/**
 * Base class for 2D curves.
 *
 * 2D curves are used to represent curves in surface parametric domains (p-curves).
 * They are used by edges owned by faces.
 *
 * > __Note__: This class is defined for SDK API compatibility only.
 * Unlike {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___curve2d.html C++ SDK Curve 2D}
 * it contains only type of the curve.
 */
abstract class ModelData_Curve2d {
    /**
     * Curve type.
     */
    get type(): ModelData_CurveType;
}
/**
 * Defines curve type.
 */
enum ModelData_CurveType {
    Line = 0,
    Circle = 1,
    Ellipse = 2,
    Hyperbola = 3,
    Parabola = 4,
    Bezier = 5,
    BSpline = 6,
    Offset = 7,
    Trimmed = 8,
    Undefined = 100,
    UserDefine = 101,
}
/**
 * Represents direction in 3D space.
 */
class ModelData_Direction {
    /**
     * Creates Direction of object or three numbers.
     */
    constructor(x?: number, y?: number, z?: number);
    /**
     * A tolerance to compare coincidence of two direction.
     */
    static readonly coincidenceTolerance: number;
    /**
     * X coord.
     */
    get x(): number;
    set x(x: number);
    /**
     * Y coord.
     */
    get y(): number;
    set y(y: number);
    /**
     * Z coord.
     */
    get z(): number;
    set z(z: number);
    /**
     * Creates direction from object implementing {@link ModelData_XYZ} interface.
     */
    static fromXYZ(theXYZ: ModelData_XYZ): ModelData_Direction;
    /**
     * Sets coordinates.
     */
    setCoord(x: number, y: number, z: number): this;
    /**
     * Reverses the direction.
     */
    reverse(): this;
    /**
     * Returns a direction which is a reversed direction of this.
     */
    reversed(): ModelData_Direction;
    /**
     * Transforms the point with a transformation matrix.
     */
    transform(theTransformation: ModelData_Transformation): this;
    /**
     * Returns a new point which is transformed with a transformation matrix.
     */
    transformed(theTransformation: ModelData_Transformation): ModelData_Direction;
    /**
     * Returns a dot-product.
     */
    dot(theDirection: ModelData_XYZ): number;
    /**
     * Makes cross product with another direction.
     */
    cross(theDirection: ModelData_XYZ): this;
    /**
     * Makes cross product from current direction and with another one.
     */
    crossed(theDirection: ModelData_XYZ): ModelData_Direction;
    /**
     * Compares if two direction are parallel within a distance tolerance.
     */
    isParallel(theOther: ModelData_Direction, theTolerance: number): boolean;
    /**
     * Compares if two direction are equal within a distance tolerance.
     */
    isEqual(theXYZ: ModelData_XYZ, theTolerance?: number): boolean;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Direction;
    /**
     * Copies values from another direction.
     */
    copy(theDirection: ModelData_Direction): this;
    /**
     * Returns string representation of the direction.
     */
    toString(): string;
    /**
     * Returns JSON representation of the direction.
     */
    toJSON(): ModelData_XYZ;
}
/**
 * Represents direction in 2D space.
 */
class ModelData_Direction2d {
    /**
     * Creates instance of the ModelData_Direction2d.
     */
    constructor(x?: number, y?: number);
    /**
     * A tolerance to compare coincidence of two direction.
     */
    static readonly coincidenceTolerance: number;
    /**
     * X coord.
     */
    get x(): number;
    set x(x: number);
    /**
     * Y coord.
     */
    get y(): number;
    set y(y: number);
    /**
     * Creates direction from object implementing {@link ModelData_XY} interface.
     */
    static fromXY(theXY: ModelData_XY): ModelData_Direction2d;
    /**
     * Sets coordinates.
     */
    setCoord(x: number, y: number): this;
    /**
     * Reverses the direction.
     */
    reverse(): this;
    /**
     * Returns a direction which is a reversed direction of this.
     */
    reversed(): ModelData_Direction2d;
    /**
     * Returns a dot-product.
     */
    dot(theDirection: ModelData_Direction2d): number;
    /**
     * Makes cross product with another direction.
     */
    cross(theDirection: ModelData_Direction2d): number;
    /**
     * Compares if two direction are parallel within a distance tolerance.
     */
    isParallel(theOther: ModelData_Direction2d, theTolerance: number): boolean;
    /**
     * Compares if two direction are equal within a distance tolerance.
     */
    isEqual(theXY: ModelData_XY, theTolerance?: number): boolean;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Direction2d;
    /**
     * Copies values from another direction.
     */
    copy(theDirection: ModelData_Direction2d): this;
    /**
     * Returns string representation of the direction.
     */
    toString(): string;
    /**
     * Returns JSON representation of the direction.
     */
    toJSON(): ModelData_XY;
}
/**
 * Represents a single 2D drawing of a model.
 *
 * Encapsulates sheets, views and layers with various 2D geometry. An object of this class is a counterpart
 * to the ModelData_Model class, meaning that it's meant to represent all drawing-related information
 * for a single 3D model. In case the model is entirely 2D, an object corresponds to a single input file.
 */
class ModelData_Drawing extends ModelData_BaseObject {
    constructor(theSheet?: ModelData_DrawingSheet);
    /**
     * The number of sheets in the drawing.
     */
    get numberOfSheets(): number;
    /**
     * The number of layers in the drawing.
     */
    get numberOfLayers(): number;
    /**
     * Adds a sheet to the drawing.
     */
    addSheet(theSheet: ModelData_DrawingSheet): void;
    /**
     * Retrieves sheet by index.
     */
    sheet(theIndex: number): ModelData_DrawingSheet | undefined;
    /**
     * Returns iterable to added sheets.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const sheet of drawingSheet.sheets()) {
     *   // Access to the sheet
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_1_1_sheet_iterator.html ModelData_Drawing::SheetIterator} class.
     */
    sheets(): Iterable<ModelData_DrawingSheet>;
    /**
     * Adds a layer to the drawing.
     */
    addLayer(theLayer: ModelData_DrawingLayer): void;
    /**
     * Retrieves layer by index.
     */
    layer(theIndex: number): ModelData_DrawingLayer | undefined;
    /**
     * Returns iterable to added layers.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const layer of drawingSheet.layers()) {
     *   // Access to the layer
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_1_1_layer_iterator.html ModelData_Drawing::LayerIterator} class.
     */
    layers(): Iterable<ModelData_DrawingLayer>;
}
/**
 * Describes the angular dimension measures of for an angle given by three points.
 *
 * For more details refer to {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_angular_dimension.html#details C++ class documentation}.
 */
class ModelData_DrawingAngularDimension extends ModelData_DrawingDimension {
    constructor(theAnglePoint?: ModelData_Point2d, theFirstReferencePoint?: ModelData_Point2d, theSecondReferencePoint?: ModelData_Point2d, theDimensionArcLocation?: ModelData_Point2d, theMeasurementValue?: number);
    /**
     * The tip of the measured angle
     */
    anglePoint: ModelData_Point2d;
    /**
     * The first extension line reference point.
     */
    firstReferencePoint: ModelData_Point2d;
    /**
     * The second extension line reference point
     */
    secondReferencePoint: ModelData_Point2d;
    /**
     * The dimension arc reference point.
     */
    dimensionArcPoint: ModelData_Point2d;
    override accept(theVisitor: ModelData_DrawingElementVisitor): void;
}
/**
 * Describes drawing elements composed of 2D curves.
 *
 * This geometric element encapsulates the portion of the view's geometry described with parametric
 * 2D curves (more specifically, objects of classes derived from ModelData_Curve2d).
 */
class ModelData_DrawingCurveSet extends ModelData_DrawingGeometry {
    override accept(theVisitor: ModelData_DrawingElementVisitor): void;
}
/**
 * Describes a diametric dimension measures of circles.
 *
 * For more details refer to {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_diametric_dimension.html#details C++ class documentation}.
 */
class ModelData_DrawingDiametricDimension extends ModelData_DrawingDimension {
    constructor(theFirstChordPoint?: ModelData_Point2d, theSecondChordPoint?: ModelData_Point2d, theDimensionEndPoint?: ModelData_Point2d, theMeasurementValue?: number);
    /**
     * The first chord point on the drawing geometry that are being measured.
     */
    firstChordPoint: ModelData_Point2d;
    /**
     * The second chord point on the drawing geometry that are being measured.
     */
    secondChordPoint: ModelData_Point2d;
    /**
     * The dimension end point. Defines the end point of dimension line.
     */
    dimensionEndPoint: ModelData_Point2d;
    /**
     * Indicates whether the dimension line passes through the center of measured entity or not.
     */
    startsFromCenter: boolean;
    override accept(theVisitor: ModelData_DrawingElementVisitor): void;
}
/**
 * Defines the arrowhead placement.
 */
enum ModelData_DrawingDimensionArrowheadPlacementType {
    Internal = 0,
    External = 1,
}
/**
 * Base class for various dimension types.
 *
 * For more details refer to {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_dimension.html#details C++ class documentation}.
 */
abstract class ModelData_DrawingDimension extends ModelData_DrawingElement {
    /**
     * The arrowhead placement.
     */
    arrowheadPlacement: ModelData_DrawingDimensionArrowheadPlacementType;
    /**
     * Indicates whether the measurement text is align horizontally.
     */
    alignTextHorizontally: boolean;
    /**
     * The actual measurement value.
     */
    measurementValue: number;
    /**
     * The dimension text.
     */
    text: ModelData_DrawingText;
}
/**
 * Base class for drawing elements.
 *
 * Drawing elements are objects that are contained in views and define the contents
 * of a drawing. They include 2D geometry and view instances.
 */
abstract class ModelData_DrawingElement extends ModelData_BaseObject {
    /**
     * Accepts a drawing element visitor.
     */
    abstract accept(theVisitor: ModelData_DrawingElementVisitor): void;
}
/**
 * Defines a visitor of drawing elements.
 *
 * Provides an interface for visitors to implement to access elements stored in instances of {@link ModelData_DrawingView}.
 */
abstract class ModelData_DrawingElementVisitor {
    abstract visitDrawingAngularDimension(theElement: ModelData_DrawingAngularDimension): void;
    abstract visitDrawingDiametricDimension(theElement: ModelData_DrawingDiametricDimension): void;
    abstract visitDrawingLinearDimension(theElement: ModelData_DrawingLinearDimension): void;
    abstract visitDrawingRadialDimension(theElement: ModelData_DrawingRadialDimension): void;
    abstract visitDrawingCurveSet(theElement: ModelData_DrawingCurveSet): void;
    abstract visitDrawingPiecewiseContour(theElement: ModelData_DrawingPiecewiseContour): void;
    abstract visitDrawingPointSet(theElement: ModelData_DrawingPointSet): void;
    abstract visitDrawingHatch(theElement: ModelData_DrawingHatch): void;
    abstract visitDrawingText(theElement: ModelData_DrawingText): void;
}
/**
 * Base class for geometry drawing elements.
 */
abstract class ModelData_DrawingGeometry extends ModelData_DrawingElement {
}
/**
 * Represents pattern for drawing hatch.
 */
enum ModelData_DrawingHatchPatternType {
    Solid = 0,
    ANSI_31 = 1,
    ANSI_32 = 2,
    ANSI_33 = 3,
    ANSI_34 = 4,
    ANSI_35 = 5,
    ANSI_36 = 6,
    ANSI_37 = 7,
    ANSI_38 = 8,
}
/**
 * Represents style for drawing hatch.
 */
enum ModelData_DrawingHatchStyleType {
    Standard = 0,
    Mirrored = 1,
    Double = 2,
}
/**
 * Description
 */
enum ModelData_DrawingHatchContourType {
    Inner = 0,
    Outer = 1,
}
/**
 * Represents an area filled with a pattern.
 *
 * The hatch pattern can be composed of solid, dashed or dash-doted lines. The pattern can be mirrored or composed
 * with its mirror image to create double hatch. It follows the scheme used by AutoCAD to represent the set of
 * ANSI-standard hatch patterns.
 *
 * For more details refer to {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_hatch.html#details C++ class documentation}.
 */
class ModelData_DrawingHatch extends ModelData_DrawingElement {
    /**
     * The hatch pattern.
     */
    pattern: ModelData_DrawingHatchPatternType;
    /**
     * The style for the hatch pattern.
     */
    style: ModelData_DrawingHatchStyleType;
    /**
     * The scale of the hatch.
     */
    scale: number;
    /**
     * The number of hatch contours.
     */
    get numberOfContours(): number;
    override accept(theVisitor: ModelData_DrawingElementVisitor): void;
    /**
     * Adds the hatch contour.
     * The contour must be closed otherwise it will not be added and the function will return false.
     */
    addContour(theContour: ModelData_DrawingPiecewiseContour, theType?: ModelData_DrawingHatchContourType): boolean;
    /**
     * Returns the contour.
     */
    contour(theIndex: number): ModelData_DrawingPiecewiseContour | undefined;
    /**
     * Returns the type of hatch contour.
     */
    typeOfContour(theIndex: number): ModelData_DrawingHatchContourType | undefined;
}
/**
 * Provides logical grouping of {@link ModelData_DrawingElement} objects.
 *
 * Layers are commonly used to group objects in the drawing by function (e.g. walls, furniture, etc.),
 * but in principle they can represent any type of virtual grouping. All objects in a layer share appearance,
 * can be shown and hidden together. A drawing element can only belong to one layer.
 *
 * The drawing layer holds an `IsVisible()` attribute, which is `true` by default.
 */
class ModelData_DrawingLayer extends ModelData_BaseObject {
    /**
     * Creates a named layer, if `theName` is not empty.
     */
    constructor(theName?: string);
    /**
     * Indicates whether the drawing layer is visible or not.
     */
    get isVisible(): boolean;
    set isVisible(theIsVisible: boolean);
    /**
     * The drawing layer's own appearance.
     * The appearance for drawing element defines how this object should be visualized and contain the
     * following components:
     * * generic color - defines an RGBA color.
     * * line styles - define line properties.
     */
    get appearance(): ModelData_Appearance | null;
    set appearance(theAppearance: ModelData_Appearance | null);
    /**
     * Adds element to the drawing layer.
     */
    add(theElement: ModelData_DrawingElement): void;
    /**
     * Removes element from the drawing layer.
     * Returns `true` if the element has been in the list and `false` otherwise.
     */
    remove(theElement: ModelData_DrawingElement): boolean;
    /**
     * Returns `true` if element has already been added to the drawing layer.
     */
    contains(theElement: ModelData_DrawingElement): boolean;
    /**
     * Returns iterable to added elements.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const element of drawingLayer.elements()) {
     *   // Access to the element
     * }
     * ```
     */
    elements(): Iterable<ModelData_DrawingElement>;
    /**
     * Accepts the element visitor.
     */
    accept(theVisitor: ModelData_DrawingElementVisitor): void;
}
/**
 * Represents the rotation of linear dimension.
 */
enum ModelData_DrawingLinearDimensionRotationType {
    Horizontal = 0,
    Vertical = 1,
    Aligned = 2,
}
/**
 * Linear dimensions with horizontal, vertical, and aligned dimension lines.
 *
 * For more details refer to {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_linear_dimension.html#details C++ class documentation}.
 */
class ModelData_DrawingLinearDimension extends ModelData_DrawingDimension {
    constructor(theFirstReferencePoint?: ModelData_Point2d, theSecondReferencePoint?: ModelData_Point2d, theDimensionLinePoint?: ModelData_Point2d, theMeasurementValue?: number, theRotation?: ModelData_DrawingLinearDimensionRotationType);
    /**
     * The first reference point on the drawing geometry that are being measured.
     */
    firstReferencePoint: ModelData_Point2d;
    /**
     * The second reference point on the drawing geometry that are being measured.
     */
    secondReferencePoint: ModelData_Point2d;
    /**
     * The dimension line point. Specifies the position of dimension line (can be any point on the dimension line).
     */
    dimensionLinePoint: ModelData_Point2d;
    /**
     * The rotation of the dimension. Determines the type of linear dimension.
     */
    rotation: ModelData_DrawingLinearDimensionRotationType;
    override accept(theVisitor: ModelData_DrawingElementVisitor): void;
}
/**
 * Describes drawing elements composed connected 2D curves.
 *
 * Restriction - can only contain ModelData_TrimmedCurve2d and ModelData_BSplineCurve2d.
 */
class ModelData_DrawingPiecewiseContour extends ModelData_DrawingGeometry {
    override accept(theVisitor: ModelData_DrawingElementVisitor): void;
    /**
     * Returns `true` if the contour is closed.
     */
    isClosed(): boolean;
}
/**
 * Describes drawing elements composed of 2D points.
 *
 * This geometric element encapsulates the portion of the view's geometry described with 2D points.
 */
class ModelData_DrawingPointSet extends ModelData_DrawingGeometry {
    /**
     * The number of points currently composing the element.
     */
    get numberOfPoints(): number;
    override accept(theVisitor: ModelData_DrawingElementVisitor): void;
    /**
     * Adds a point to the element.
     */
    addPoint(thePoint: ModelData_Point2d): void;
    /**
     * Returns the specified point of the element.
     */
    point(theIndex: number): ModelData_Point2d | undefined;
    /**
     * Returns iterable to added points.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const point of drawingPointSet.points()) {
     *   // Access to the point
     * }
     * ```
     */
    points(): Iterable<ModelData_Point2d>;
}
/**
 * Describes a radial dimension measures of arcs and circles.
 *
 * For more details refer to {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_radial_dimension.html#details C++ class documentation}.
 */
class ModelData_DrawingRadialDimension extends ModelData_DrawingDimension {
    constructor(theCenterPoint?: ModelData_Point2d, theChordPoint?: ModelData_Point2d, theDimensionPoint?: ModelData_Point2d, theMeasurementValue?: number);
    /**
     * The center of the circle arc, which the dimension describes.
     */
    centerPoint: ModelData_Point2d;
    /**
     * The chord point of the radial dimension. Lies on the said circle arc.
     */
    chordPoint: ModelData_Point2d;
    /**
     * The end of the dimension line. Defines the length of the leader of this dimension.
     */
    dimensionEndPoint: ModelData_Point2d;
    /**
     * Indicates whether the radius dimension line starts from the center of the measured entity or not.
     */
    startsFromCenter: boolean;
    override accept(theVisitor: ModelData_DrawingElementVisitor): void;
}
/**
 * Represents supported paper sizes of drawing sheets.
 */
enum ModelData_DrawingSheetPaperSizeType {
    UnknownPaperSize = 0,
    A0 = 1,
    A1 = 2,
    A2 = 3,
    A3 = 4,
    A4 = 5,
    A = 6,
    B = 7,
    C = 8,
    D = 9,
    E = 10,
    UserDefinedPaperSize = 1000,
}
/**
 * Represents orientation of the drawing sheet.
 */
enum ModelData_DrawingSheetOrientationType {
    Landscape = 0,
    Portrait = 1,
}
/**
 * Represents a single sheet of a model drawing.
 *
 * An object of this class mimics a physical sheet of paper, hence the size and orientation settings.
 *
 * Normally a drawing sheet would have associated metadata, such as default scale, title block, etc.
 * This information is represented with objects of ModelData_DrawingView class.
 */
class ModelData_DrawingSheet extends ModelData_BaseObject {
    /**
     * The width of the sheet.
     *
     * The width and height of a sheet with a standard size depend on the orientation.
     * Changing the orientation swaps width and height.
     */
    get width(): number;
    /**
     * The height of the sheet.
     *
     * The width and height of a sheet with a standard size depend on the orientation. Changing the orientation
     * swaps width and height.
     */
    get height(): number;
    /**
     * The paper size of the sheet.
     */
    get paperSize(): ModelData_DrawingSheetPaperSizeType;
    /**
     * The orientation of the sheet.
     */
    get orientation(): ModelData_DrawingSheetOrientationType;
    /**
     * The number of child views.
     */
    get numberOfViews(): number;
    /**
     * Sets the sheet size to one of the standard sizes.
     */
    setStandardSheetSize(thePaperSize: ModelData_DrawingSheetPaperSizeType, theOrientation: ModelData_DrawingSheetOrientationType): void;
    /**
     * Sets the custom sheet size with specified dimensions.
     */
    setCustomSheetSize(theWidth: number, theHeight: number): void;
    /**
     * Adds a view to the sheet.
     */
    addView(theView: ModelData_DrawingView): void;
    /**
     * Returns iterable to added views.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const view of drawingSheet.views()) {
     *   // Access to the view
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_sheet_1_1_view_iterator.html ModelData_DrawingSheet::ViewIterator} class.
     */
    views(): Iterable<ModelData_DrawingView>;
}
/**
 * Defines a text for drawing.
 *
 * For more details refer to {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_text.html#details C++ class documentation}.
 */
class ModelData_DrawingText extends ModelData_DrawingElement {
    text: string;
    /**
     * The position of the text on the 2d plane.
     */
    origin: ModelData_Point2d;
    /**
     * The properties of dimension text.
     */
    textProperties: ModelData_TextProperties;
    override accept(theVisitor: ModelData_DrawingElementVisitor): void;
}
/**
 * Represents a view on a drawing sheet.
 *
 * Usually a view is a logical portion of the sheet, containing one of the literal views of
 * the model (e.g. top view, isometric view, cutaway view). Following this idea, the view is
 * defined by a position in the sheet and a scale (real-world model dimensions vs on-paper dimensions).
 *
 * Generally an object of this class can contain any collection of drawing elements (i.e. 2D geometry),
 * but it's expected that these elements together form a logically self-contained group.
 */
class ModelData_DrawingView extends ModelData_BaseObject {
    /**
     * The position of the view in the owning sheet.
     */
    position: ModelData_Axis2Placement2d;
    /**
     * The scale of the view.
     */
    scale: ModelData_Ratio;
    /**
     * The number of elements in the view.
     */
    get numberOfElements(): number;
    /**
     * Adds a element to the view.
     */
    add(theElement: ModelData_DrawingElement): void;
    /**
     * Returns iterable to added elements.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const element of drawingView.elements()) {
     *   // Access to the element
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___drawing_view_1_1_element_iterator.html ModelData_DrawingView::ElementIterator} class.
     */
    elements(): Iterable<ModelData_DrawingElement>;
    /**
     * Returns the appearance for the element.
     */
    elementAppearance(theElement: ModelData_DrawingElement): ModelData_Appearance | null;
    /**
     * Sets the appearance for the element.
     */
    setElementAppearance(theElement: ModelData_DrawingElement, theAppearance: ModelData_Appearance | null): void;
    /**
     * Accepts a drawing element visitor.
     */
    accept(theVisitor: ModelData_DrawingElementVisitor): void;
}
/**
 * Defines an edge.
 */
abstract class ModelData_Edge extends ModelData_Shape {
    /**
     * Edge 3D curve.
     */
    get curve(): ModelData_Curve | null;
    /**
     * Start vertex.
     */
    get startVertex(): ModelData_Vertex;
    /**
     * End vertex.
     */
    get endVertex(): ModelData_Vertex;
    [Symbol.iterator](): Iterator<ModelData_Vertex>;
}
/**
 * Base class for elementary surfaces.
 *
 * Elementary surface has an axis placement returned by Position().
 */
class ModelData_ElementarySurface {
    constructor(thePlacement: ModelData_Axis3Placement);
    /**
     * Surface axis.
     */
    get position(): ModelData_Axis3Placement;
    /**
     * Origin point.
     */
    get location(): ModelData_Point;
    /**
     * Z-direction of the axis placement.
     */
    get direction(): ModelData_Direction;
}
/**
 * Defines a topological face.
 *
 * Face resides on a geometrical surface and is bounded by one or several wires.
 */
abstract class ModelData_Face extends ModelData_Shape {
    /**
     * Underlying surface.
     */
    get surface(): ModelData_Surface | null;
    /**
     * Adds a wire to the face.
     */
    append(theWire: ModelData_Wire): boolean;
    [Symbol.iterator](): Iterator<ModelData_Wire>;
}
/**
 * Used to defer loading of image of the file texture.
 */
abstract class ModelData_FileTextureProvider {
    /**
     * Feeds the file texture.
     */
    abstract feed(theFileTexture: ModelData_FileTexture, theScope: Base_ProgressScope): Promise<void>;
}
/**
 * Provides a texture defined by file url.
 */
class ModelData_FileTexture extends ModelData_Texture {
    /**
     * Creates an instance of ModelData_FileTexture.
     */
    constructor(theFilePath?: string | null, theTextureType?: ModelData_TextureType, theTextureParameters?: ModelData_TextureParameters | null);
    /**
     * A file path to the file.
     */
    filePath: string | null;
    /**
     * Texture data provider.
     */
    get provider(): ModelData_FileTextureProvider | undefined;
    set provider(theProvider: ModelData_FileTextureProvider | undefined);
    /**
     * Creates the basic provider which uses external data provider for data fetching from server.
     */
    static createBasicProvider(dataProvider: ModelData_ExternalDataProvider): ModelData_FileTextureProvider;
    /**
     * Returns image. This method flushes provider.
     *
     * > __Note__: the file path should be relative of the model scenegraph location.
     */
    image(theScope?: Base_ProgressScope): Promise<(HTMLImageElement|HTMLCanvasElement|ImageBitmap|null)>;
    setImage(theImage: HTMLImageElement | HTMLCanvasElement | ImageBitmap | null): void;
    override accept(theVisitor: ModelData_TextureVisitor): Promise<void>;
}
/**
 * Defines a polygonal shape consisting of indexed poly lines.
 */
class ModelData_IndexedPolyLineSet extends ModelData_PolyVertexSet {
    /**
     * Creates an instance of ModelData_IndexedPolyLineSet.
     */
    constructor();
    /**
     * Indexes array.
     */
    indexes: Uint32Array | Uint16Array | null;
    /**
     * Creates a new instance of ModelData_IndexedPolyLineSet with the given attributes
     */
    static create(theCoords: Array<ModelData_Point>, theIndexes: Array<number>, theColors?: Array<ModelData_ColorObject>): ModelData_IndexedPolyLineSet;
}
/**
 * Defines a polygonal shape consisting of triangles.
 *
 * Triangles are defined via array of unique vertex coordinates and triplets of indices in this array.
 * This is to describe connectivity and to reduce data required to describe a triangle set.
 *
 * The triangle set can optionally contain normals which are defined per each vertex.
 */
class ModelData_IndexedTriangleSet extends ModelData_PolyVertexSet {
    /**
     * Creates an instance of ModelData_IndexedTriangleSet.
     */
    constructor();
    /**
     * Indexes array.
     */
    indexes: Uint32Array | Uint16Array | null;
    /**
     * Array of vertex normals in {x,y,z} format [x,y,z,x,y,z,...].
     */
    normals: Float32Array | null;
    /**
     * Array of vertex UV coordinates in {u,v} format [u,v,u,v,...].
     */
    uvCoordinates: Float32Array | null;
    /**
     * Number of faces (triangles).
     */
    get numberOfFaces(): number;
    /**
     * Total number of normals stored in {@link ModelData_IndexedTriangleSet#normals normals} array.
     */
    get numberOfNormals(): number;
    /**
     * Total number of UV coordinates stored in {@link ModelData_IndexedTriangleSet#uvCoordinates uvCoordinates} array.
     */
    get numberOfUVCoordinates(): number;
    /**
     * Creates a new instance of ModelData_IndexedTriangleSet with the given attributes
     */
    static create(theCoords: Array<ModelData_Point>, theIndexes: Array<number>, theNormals?: Array<ModelData_Direction>, theColors?: Array<ModelData_ColorObject>, theUv?: Array<ModelData_Point2d>): ModelData_IndexedTriangleSet;
    /**
     * Returns a coordinate index for a vertex in a face.
     */
    coordinateIndex(theFace: number, theVertexSlot: number): number;
    /**
     * Returns `true` if object has normal per vertex
     */
    hasNormals(): boolean;
    /**
     * Returns a normal index for a vertex in a face.
     */
    normalIndex(theFace: number, theVertexSlot: number): number;
    /**
     * Returns a normal per its index.
     * This method may only be used if `hasNormals()` returns `true`,
     * otherwise returns `undefined`.
     *
     * > __Note__: changing returned object does not change original normal.
     */
    normal(theIndex: number, theTarget?: ModelData_Direction): ModelData_Direction | undefined;
    /**
     * Returns `true` if object has UV coordinates associated with vertices.
     */
    hasUVCoordinates(): boolean;
    /**
     * Returns a UV coordinate index for a vertex in a face.
     * This method may only be used if `hasUVCoordinates()` returns `true`,
     * otherwise return -1.
     */
    uvCoordinateIndex(theFace: number, theVertexSlot: number): number;
    /**
     * Returns a UV coordinate per its index.
     * This method may only be used if `hasUVCoordinates()` returns `true`,
     * otherwise returns `undefined`.
     *
     * > __Note__: changing returned object does not change original UV point.
     */
    uvCoordinate(theIndex: number, theTarget?: ModelData_Point2d): ModelData_Point2d | undefined;
    /**
     * Returns an appearance index for the face.
     */
    appearanceIndex(theFace: number): number;
}
/**
 * Defines an occurrence of assembly or part in a scene graph.
 */
class ModelData_Instance extends ModelData_SceneGraphElement {
    /**
     * Creates an instance of ModelData_Instance.
     */
    constructor(theElement?: ModelData_SceneGraphElement, theTrsf?: ModelData_Transformation, theName?: string | null);
    /**
     * Referenced object.
     * Warning: there must be no cyclic references between theElement and this instance as otherwise there will
     * be memory leaks and infinite loop when visiting the scene graph.
     */
    get reference(): ModelData_SceneGraphElement | null;
    set reference(theRef: ModelData_SceneGraphElement | null);
    /**
     * The transformation of the reference. If the matrix has not been explicitly assigned with setter
     * then creates and explicitly assigns an identity matrix.
     */
    get transformation(): ModelData_Transformation;
    /**
     * > __Note__: Setter makes deep copy of the transformation.
     */
    set transformation(theTrsf: ModelData_Transformation);
    /**
     * Returns `true` if the instance has an explicitly attached transformation matrix.
     */
    hasTransformation(): boolean;
    override accept(theVisitor: ModelData_SceneGraphElementVisitor): Promise<void>;
}
/**
 * Provides a layer of objects.
 *
 * Layer is a logical group of elements.
 *
 * Scene Graph elements can be added and removed using Add() and Remove() methods respectively.
 *
 * Shapes can be added and removed using ModelData_BRepRepresentation::AddToLayer() and
 * ModelData_BRepRepresentation::RemoveFromLayer() methods respectively.
 *
 * ItemVisitor should be used to access items inside layer.
 *
 * > *Note*: Scene Graph elements can also be accessed using ModelData_Model::ElementVisitor.
 */
class ModelData_Layer extends ModelData_BaseObject {
    constructor(name?: string);
    /**
     * Layer visibility status
     */
    isVisible: boolean;
    /**
     * Adds element to the layer.
     */
    add(theElement: ModelData_SceneGraphElement): void;
    /**
     * Removes element from the layer.
     * Returns `true` if the element has been in the list and `false` otherwise.
     */
    remove(theElement: ModelData_SceneGraphElement): boolean;
    /**
     * Returns `true` if element has already been added to the layer.
     */
    find(theElement: ModelData_SceneGraphElement): boolean;
}
class ModelData_LineProperties {
    constructor(theType?: ModelData_LineType, theWidthScaleFactor?: number);
    /**
     * Line pattern.
     */
    type: ModelData_LineType;
    /**
     * A multiplicative value that scales nominal line width.
     * A value less than or equal to zero refers to the minimum available line width.
     */
    widthScaleFactor: number;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_LineProperties;
    /**
     * Copies parameters from another object.
     */
    copy(theLineProperties: ModelData_LineProperties): this;
    /**
     * Equality operator.
     */
    isEqual(theLineProperties: ModelData_LineProperties | null | undefined): boolean;
}
/**
 * Defines the line type in accordance with International Register of Graphical Items.
 */
enum ModelData_LineType {
    Solid = 1,
    Dashed = 2,
    Dotted = 3,
    DashedDotted = 4,
    DashDotDot = 5,
    Single = 6,
    SingleDot = 7,
    DoubleArrow = 8,
    StitchLine = 9,
    ChainLine = 10,
    CenterLine = 11,
    HiddenLine = 12,
    PhantomLine = 13,
    BreakLine1 = 14,
    BreakLine2 = 15,
    UserSpecifiedDashPattern = 16,
}
/**
 * Provides a data container for material.
 */
class ModelData_MaterialObject extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_MaterialObject.
     */
    constructor(theAmbientColor?: ModelData_ColorObject, theDiffuseColor?: ModelData_ColorObject, theSpecularColor?: ModelData_ColorObject, theEmissionColor?: ModelData_ColorObject, theShininess?: number);
    /**
     * Ambient color.
     */
    ambientColor: ModelData_ColorObject;
    /**
     * Diffuse color.
     */
    diffuseColor: ModelData_ColorObject;
    /**
     * Specular color.
     */
    specularColor: ModelData_ColorObject;
    /**
     * Emissive color.
     */
    emissiveColor: ModelData_ColorObject;
    /**
     * Shininess value. Must be within [0, 128] range.
     */
    get shininess(): number;
    set shininess(theShininess: number);
    /**
     * Creates deep copy of the current object.
     */
    clone(): ModelData_MaterialObject;
    /**
     * Copies another material properties to current.
     */
    copy(theMaterial: ModelData_MaterialObject): this;
    /**
     * Equality operator.
     * Returns `true` if all colors of two materials are equal.
     */
    isEqual(theMaterial: ModelData_MaterialObject | null | undefined): boolean;
}
/**
 * The external data type definition.
 */
type ModelData_ExternalDataType = ArrayBuffer;
/**
 * The external data provider function type.
 */
type ModelData_CDXFBBufferProvider = (theFileId: string, theObjectId: string, theProgressScope: Base_ProgressScope) => Promise<ModelData_ExternalDataType>;
/**
 * The external data provider function type.
 */
type ModelData_ExternalDataProvider = (theFileName: string, theProgressScope: Base_ProgressScope) => Promise<ModelData_ExternalDataType>;
/**
 * Result of {@link ModelData_Model#loadFile} and {@link ModelData_Model#loadSceneGraph} methods.
 */
type ModelData_ModelLoadResult = {
    /**
     * `true` if the loaded model contains a B-Rep representation for at least one part.
     */
    hasBRepRep: boolean;
    /**
     * The maximum number of poly representations per part in the model.
     */
    polyRepCount: number;
    /**
     * List of root elements added to the model.
     */
    roots: Array<ModelData_SceneGraphElement>;
}
/**
 * Provides CAD Exchanger data model.
 *
 * The model encapsulates a scene graph (hierarchy of assemblies and parts) and is thus an entry
 * point to entire 3D model data.
 *
 * The model object can be and restored from a file in native CAD Exchanger format.
 * Future versions may introduce model saving to CAD Exchanger native format.
 */
class ModelData_Model {
    /**
     * Creates an instance of ModelData_Model.
     */
    constructor(theName?: string);
    /**
     * The model name. `null` if the model has no name (by default).
     */
    name: string | null;
    /**
     * The drawing of the model.
     */
    drawing: ModelData_Drawing | null;
    /**
     * Number of root elements in the graph.
     */
    get numberOfRoots(): number;
    /**
     * Property table of the model.
     */
    get properties(): ModelData_PropertyTable | null;
    /**
     * Adds a property table to the model.
     * If the model did not have properties yet, then theProperties are just linked to it.
     * Otherwise, the elements of theProperties are appended to existing property table
     * (see {@link ModelData_PropertyTable#add ModelData_PropertyTable#add()} method).
     */
    addProperties(theProperties: ModelData_PropertyTable): void;
    /**
     * Adds new root element into the scene graph.
     */
    addRoot(theRoot: ModelData_SceneGraphElement): void;
    /**
     * Removes root scene graph element from the model.
     * Returns `true` if removal took place and `false` otherwise.
     */
    removeRoot(theRoot: ModelData_SceneGraphElement): boolean;
    /**
     * Returns root element in the graph by index.
     */
    root(theIndex: number): ModelData_SceneGraphElement | undefined;
    /**
     * Returns iterable to roots.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const root of model.roots()) {
     *   // Access to the root
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___model_1_1_element_iterator.html ModelData_Model::ElementIterator} class.
     */
    roots(): Iterable<ModelData_SceneGraphElement>;
    /**
     * Detaches the model contents.
     * Only cleans up contents of `this` object.
     */
    clear(): void;
    isEmpty(): boolean;
    /**
     * Adds layer to the model.
     */
    addLayer(theLayer: ModelData_Layer): void;
    /**
     * Returns iterable to layers.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const layer of model.layers()) {
     *   // Access to the layer
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___model.html ModelData_Model::LayerIterator} class.
     */
    layers(): Iterable<ModelData_Layer>;
    /**
     * Accepts an element visitor.
     */
    accept(theVisitor: ModelData_SceneGraphElementVisitor): Promise<void>;
    /**
     * Loads CDXFB or CDXWEB model with specified file identificator.
     * All necessary data will be requested using `dataProvider`.
     *
     * Deprecated. Use {@link ModelData_Model#load} instead.
     * @deprecated
     */
    loadFile(theFileId: string, dataProvider: ModelData_CDXFBBufferProvider, theAppend?: boolean, theProgressScope?: Base_ProgressScope): Promise<ModelData_ModelLoadResult>;
    /**
     * Reads CDXFB or CDXWEB model from buffer.
     * The external referenced data will be requested using `dataProvider`.
     *
     * Deprecated. Use {@link ModelData_Model#read} instead.
     * @deprecated
     */
    loadSceneGraph(theFileId: string, theFileData: ModelData_ExternalDataType, dataProvider: ModelData_CDXFBBufferProvider, theAppend?: boolean): ModelData_ModelLoadResult;
    /**
     * Loads CDXFB or CDXWEB model with specified file name.
     * All necessary data will be requested using `dataProvider`.
     */
    load(theFileName: string, dataProvider: ModelData_ExternalDataProvider, theAppend?: boolean, theProgressScope?: Base_ProgressScope): Promise<ModelData_ModelLoadResult>;
    /**
     * Reads CDXFB or CDXWEB model from buffer.
     * The external referenced data will be requested using `dataProvider`.
     */
    read(theFileData: ModelData_ExternalDataType, dataProvider: ModelData_ExternalDataProvider, theAppend?: boolean): ModelData_ModelLoadResult;
}
/**
 * Reads CDXFB, CDXWEB and CDXBIMWEB formats in a uniform fashion.
 *
 * This reader handles the task of determining the suitable reader
 * itself, the client code then receives the resulting model.
 */
class ModelData_ModelReader {
    constructor(theProgressScope?: Base_ProgressScope);
    /**
     * The progress scope for the reading process.
     */
    progressScope: Base_ProgressScope;
    /**
     * Loads the model with the specified name into the specified model.
     * All necessary data will be requested using `dataProvider`.
     *
     * Automatically selects a reader suitable for the specified file.
     * Returns `true` if the reading process succeeded, `false` otherwise.
     */
    loadModel(theFileName: string, theModel: ModelData_Model, dataProvider: ModelData_ExternalDataProvider): Promise<boolean>;
    /**
     * Reads the model data from buffer into the specified model.
     * The external referenced data will be requested using `dataProvider`.
     *
     * Automatically selects a reader suitable for the specified file.
     * Returns `true` if the reading process succeeded, `false` otherwise.
     */
    readModel(theFileData: ArrayBuffer, theModel: ModelData_Model, dataProvider: ModelData_ExternalDataProvider): Promise<boolean>;
    /**
     * Loads the model with the specified name into the specified model.
     * All necessary data will be requested using `dataProvider`.
     *
     * Automatically selects a reader suitable for the specified file.
     * Returns `true` if the reading process succeeded, `false` otherwise.
     */
    loadBIMModel(theFileName: string, theModel: ModelData_BIMModel, dataProvider: ModelData_ExternalDataProvider): Promise<boolean>;
    /**
     * Reads the model data from buffer into the specified model.
     * The external referenced data will be requested using `dataProvider`.
     *
     * Automatically selects a reader suitable for the specified file.
     * Returns `true` if the reading process succeeded, `false` otherwise.
     */
    readBIMModel(theFileData: ArrayBuffer, theModel: ModelData_BIMModel, dataProvider: ModelData_ExternalDataProvider): Promise<boolean>;
}
/**
 * Defines a leaf node in the scene graph hierarchy.
 * A part corresponds to mechanical part or a product in the product structure hierarchy.
 * A part contains one or several representations: zero or one B-Rep representation, and/or
 * zero, one or multiple polygonal representations.
 */
class ModelData_Part extends ModelData_SceneGraphElement {
    /**
     * Creates an instance of ModelData_Part.
     */
    constructor(theName?: string);
    /**
     * Number of representations.
     */
    get numberOfRepresentation(): number;
    /**
     * Returns iterable to added representation.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const representation of part.representations()) {
     *   // Access to the representation
     * }
     * ```
     */
    representations(): Iterable<ModelData_Representation>;
    /**
     * Adds a representation.
     */
    addRepresentation(theRepresentation: ModelData_Representation): void;
    /**
     * Returns a B-Rep representation.
     * Returns undefined if there is no B-Rep representation.
     */
    brepRepresentation(): ModelData_BRepRepresentation | undefined;
    /**
     * Returns a polygonal representation that matches the specified mask.
     * Returns undefined if there is no Poly representation.
     */
    polyRepresentation(theRepresentationMask: number): ModelData_PolyRepresentation | undefined;
    /**
     * Find representation by mask or representation name. Returns undefined if there is no representation.
     */
    representation(theMaskOrName: ModelData_RepresentationMask | string): ModelData_Representation | undefined;
    /**
     * Accepts a representation visitor.
     */
    acceptRepresentationVisitor(theVisitor: ModelData_RepresentationVisitor): Promise<void>;
    override accept(theVisitor: ModelData_SceneGraphElementVisitor): Promise<void>;
}
/**
 * Pixel format.
 */
enum ModelData_PixelFormat {
    ModelData_PF_None = 0,
    ModelData_PF_RGB = 1,
    ModelData_PF_RGBA = 2,
    ModelData_PF_Alpha = 3,
    ModelData_PF_Luminance = 4,
    ModelData_PF_LuminanceAlpha = 5,
}
/**
 * Provides store a graphical image as a rectangular array of pixel color values.
 */
class ModelData_PixMap {
    constructor(thePixelData: Uint8Array, theWidth: number, theHeight: number, thePixelFormat: ModelData_PixelFormat);
    /**
     * Pixmap width.
     */
    get width(): number;
    /**
     * Pixmap height.
     */
    get height(): number;
    /**
     * Pixel format.
     */
    get pixelFormat(): ModelData_PixelFormat;
    /**
     * Pixels raw data.
     */
    get pixelData(): Uint8Array;
}
/**
 * Used to defer loading of pixmap of the file texture.
 */
abstract class ModelData_PixMapTextureProvider {
    /**
     * Feeds the pixmap texture.
     */
    abstract feed(thePixMapTexture: ModelData_PixMapTexture): Promise<ModelData_PixMap>;
}
/**
 * Provides a texture defined by pixel map.
 */
class ModelData_PixMapTexture extends ModelData_Texture {
    /**
     * Creates an instance of ModelData_PixMapTexture.
     */
    constructor(thePixMap?: ModelData_PixMap | null, theTextureType?: ModelData_TextureType, theTextureParameters?: ModelData_TextureParameters | null);
    /**
     * Texture data provider.
     */
    get provider(): ModelData_PixMapTextureProvider | undefined;
    set provider(theProvider: ModelData_PixMapTextureProvider | undefined);
    /**
     * Returns pixmap. This method flushes provider.
     */
    pixmap(): Promise<(ModelData_PixMap|null)>;
    setPixMap(thePixMap: ModelData_PixMap | null): void;
    override accept(theVisitor: ModelData_TextureVisitor): Promise<void>;
}
/**
 * Defines a plane.
 *
 * A plane is defined by an axis placement.
 *
 * A plane is parameterized as follows:
 * S(u,v) = P + u * Dx + v * Dy, where
 * * P is an origin point,
 * * Dx and Dy are directions (unit vectors) of X and Y axes respectively,
 * * u, v belongs to (-infinity, +infinity).
 */
class ModelData_Plane extends ModelData_ElementarySurface {
    /**
     * Creates a plane located in 3D space with an axis placement three axis.
     */
    constructor(thePlacement: ModelData_Axis3Placement);
    /**
     * Creates a plane from origin point and the normal direction.
     */
    static fromPointAndNormal(thePoint: ModelData_Point, theNormal: ModelData_Direction): ModelData_Plane;
    /**
     * Creates a plane from tree points.
     */
    static fromThreePoints(thePoint1: ModelData_Point, thePoint2: ModelData_Point, thePoint3: ModelData_Point): ModelData_Plane;
    /**
     * Creates a plane from coefficients of cartesian equation:
     * `a * x + b * y + c * z + d = 0.0`
     */
    static fromEquationCoefficients(a: number, b: number, c: number, d: number): ModelData_Plane;
    /**
     * Calculates value at specific parameters values.
     */
    value(theParameterU: number, theParameterV: number, theTarget: ModelData_Point): void;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Plane;
    /**
     * Copies plane params from another plane.
     */
    copy(thePlane: ModelData_Plane): this;
}
/**
 * Defines an angle unit.
 */
class ModelData_PMIAngleUnitAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIAngleUnitAttribute.
     */
    constructor(theUnit?: Base_AngleUnit);
    /**
     * Angle unit. Default is {@link Base_AngleUnit Base_AU_Degrees}.
     */
    unit: Base_AngleUnit;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a camera (point of view) in a scene.
 */
class ModelData_PMICamera extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_PMICamera.
     */
    constructor(theLocation?: ModelData_Point, theTargetPoint?: ModelData_Point, theUpDirection?: ModelData_Direction, theName?: string);
    /**
     * Position of the camera.
     */
    location: ModelData_Point;
    /**
     * The point that the camera is looking at.
     */
    targetPoint: ModelData_Point;
    /**
     * "up" direction of the camera
     */
    upDirection: ModelData_Direction;
}
/**
 * Defines a collection of outlines.
 */
class ModelData_PMICompositeOutline extends ModelData_PMIOutline {
    /**
     * Creates an instance of ModelData_PMICompositeOutline.
     */
    constructor();
    /**
     * Number of added outlines.
     */
    get numberOfOutlines(): number;
    /**
     * Returns iterable to outlines.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const outline of compositeOutline.outlines()) {
     *   // Access to the outline
     * }
     * ```
     */
    outlines(): Iterable<ModelData_PMIOutline>;
    /**
     * Appends an outline.
     */
    append(theOutline: ModelData_PMIOutline): void;
    /**
     * Returns `true` if no outlines have been added.
     */
    isEmpty(): boolean;
    /**
     * Returns a specified outline.
     */
    outline(theIndex: number): ModelData_PMIOutline | undefined;
    override accept(theVisitor: ModelData_PMIOutlineVisitor): void;
}
/**
 * Defines an outline consist of 2d curves.
 */
class ModelData_PMICurve2dOutline extends ModelData_PMIOutline {
    /**
     * Creates an instance of ModelData_PMICurve2dOutline.
     */
    constructor();
    override accept(theVisitor: ModelData_PMIOutlineVisitor): void;
}
/**
 * Defines an outline consist of curves.
 */
class ModelData_PMICurveOutline extends ModelData_PMIOutline {
    /**
     * Creates an instance of ModelData_PMICurveOutline.
     */
    constructor();
    override accept(theVisitor: ModelData_PMIOutlineVisitor): void;
}
/**
 * Defines a complete PMI element. Encapsulate a PMI graphical element
 * and other PMI related information, if there is any.
 */
class ModelData_PMIData extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_PMIData.
     */
    constructor(theType?: ModelData_PMIType, theName?: string);
    /**
     * PMI data type.
     */
    type: ModelData_PMIType;
    /**
     * Graphical element.
     */
    graphicalElement: ModelData_PMIGraphicalElement | null;
    /**
     * Semantic element.
     */
    semanticElement: ModelData_PMISemanticElement | null;
    /**
     * Property table.
     */
    get properties(): ModelData_PropertyTable | null;
    /**
     * Adds a property table to the data.
     *
     * If this object did not have properties yet, then theProperties are just linked to this object.
     * Otherwise, the elements of theProperties are appended to existing property table (see
     * {@link ModelData_PropertyTable#add add()}).
     */
    addProperties(theProperties: ModelData_PropertyTable): void;
    /**
     * Returns `true` if no graphical element was set.
     */
    isEmpty(): boolean;
}
/**
 * Defines a component represented by a datum, datum feature symbol or datum target.
 *
 * Datum has a label (A, B, etc.) and optionally additional attributes like index, datum target description, etc.
 */
class ModelData_PMIDatumComponent extends ModelData_PMISemanticElementComponent {
    /**
     * Creates an instance of ModelData_PMIDatumComponent.
     */
    constructor(theLabel?: string);
    /**
     * Associated label.
     */
    label: string;
    override accept(theVisitor: ModelData_PMISemanticElementComponentVisitor): void;
}
/**
 * Defines a datum reference.
 * Stores a label of the datum and its precedence in a compartment of a geometric tolerance.
 */
class ModelData_PMIDatumRefAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIDatumRefAttribute.
     */
    constructor(thePrecedence?: number, theTargetLabel?: string);
    /**
     * Precedence value. Default is `1`.
     */
    precedence: number;
    /**
     * Target label.
     */
    targetLabel: string;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a compartment of datum references or compartments.
 * Stores a collection of datum references or compartments and optionally a collection of modifiers, which is applied to each added compartment.
 */
class ModelData_PMIDatumRefCompartmentAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIDatumRefCompartmentAttribute.
     */
    constructor();
    /**
     * Number of added references.
     */
    get numberOfReferences(): number;
    /**
     * Number of added modifier attributes.
     */
    get numberOfModifierAttributes(): number;
    /**
     * Adds a reference.
     */
    addReference(theReference: ModelData_PMIDatumRefAttribute | ModelData_PMIDatumRefCompartmentAttribute): void;
    /**
     * Returns a specified reference.
     */
    reference(theIndex: number): ModelData_PMISemanticAttribute | undefined;
    /**
     * Returns iterable to references.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const reference of attribute.modifierAttributes()) {
     *   // Access to the reference
     * }
     * ```
     */
    references(): Iterable<(ModelData_PMIDatumRefAttribute|ModelData_PMIDatumRefCompartmentAttribute)>;
    /**
     * Adds a modifier attribute.
     */
    addModifierAttribute(theAttribute: ModelData_PMIModifierAttribute): void;
    /**
     * Returns a specified modifier attribute.
     */
    modifierAttribute(theIndex: number): ModelData_PMIModifierAttribute | undefined;
    /**
     * Returns iterable to modifier attributes.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const modifierAttribute of attribute.modifierAttributes()) {
     *   // Access to the modifier attribute
     * }
     * ```
     */
    modifierAttributes(): Iterable<ModelData_PMIModifierAttribute>;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a datum target data.
 * Contains a description (e.g. a point, line or limited area of the part surface) and an index (if the datum is composite).
 */
class ModelData_PMIDatumTargetAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIDatumTargetAttribute.
     */
    constructor(theIndex?: number, theDescription?: string);
    /**
     * The index. Default is `1`.
     */
    index: number;
    /**
     * The description.
     */
    description: string;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a component represented by a dimensional tolerance.
 *
 * Dimensional tolerance has a nominal value and optionally additional attributes like modifiers, plus-minus bounds etc.
 *
 * Examples of dimensional tolerance: dimensional size, dimensional location, angular size, etc.
 */
class ModelData_PMIDimensionComponent extends ModelData_PMISemanticElementComponent {
    /**
     * Creates an instance of ModelData_PMIDimensionComponent.
     */
    constructor(theNominalValue?: number);
    /**
     * Nominal value of a dimension. Default is `0`.
     */
    nominalValue: number;
    /**
     * Type of dimension. Default is {@link ModelData_PMIDimensionType.Undefined}.
     */
    typeOfDimension: ModelData_PMIDimensionType;
    override accept(theVisitor: ModelData_PMISemanticElementComponentVisitor): void;
}
/**
 * Defines type of dimensional tolerance.
 */
enum ModelData_PMIDimensionType {
    Undefined = 0,
    CurveLength = 1,
    Diameter = 2,
    SphericalDiameter = 3,
    Radius = 4,
    SphericalRadius = 5,
    ToroidalMinorDiameter = 6,
    ToroidalMajorDiameter = 7,
    ToroidalMinorRadius = 8,
    ToroidalMajorRadius = 9,
    ToroidalHighMajorDiameter = 10,
    ToroidalLowMajorDiameter = 11,
    ToroidalHighMajorRadius = 12,
    ToroidalLowMajorRadius = 13,
    Thickness = 14,
    Angular = 15,
    LinearDistance = 16,
    CurvedDistance = 17,
}
/**
 * Defines a displacement value for an unequally disposed geometric tolerance.
 */
class ModelData_PMIDisplacementAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIDisplacementAttribute.
     */
    constructor(theDisplacement?: number);
    /**
     * Displacement value. Default is `0`.
     */
    displacement: number;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a component represented by a geometric tolerance.
 *
 * Geometric tolerance has a magnitude, a type and optionally additional attributes like modifiers, datum references, etc.
 *
 * Examples of geometric tolerance: flatness tolerance, position tolerance, etc.
 */
class ModelData_PMIGeometricToleranceComponent extends ModelData_PMISemanticElementComponent {
    /**
     * Creates an instance of ModelData_PMIGeometricToleranceComponent.
     */
    constructor(theMagnitude?: number, theTypeOfTolerance?: ModelData_PMIGeometricToleranceType);
    /**
     * Magnitude value of a tolerance.
     */
    magnitude: number;
    /**
     * Type of tolerance
     */
    typeOfTolerance: ModelData_PMIGeometricToleranceType;
    /**
     * Tolerance zone form.
     */
    toleranceZoneForm: ModelData_PMIGeometricToleranceZoneFormType;
    override accept(theVisitor: ModelData_PMISemanticElementComponentVisitor): void;
}
/**
 * Defines type of tolerance.
 */
enum ModelData_PMIGeometricToleranceType {
    UndefinedTolerance = 0,
    Angularity = 1,
    CircularRunout = 2,
    Coaxiality = 3,
    Concentricity = 4,
    Cylindricity = 5,
    Flatness = 6,
    LineProfile = 7,
    Parallelism = 8,
    Perpendicularity = 9,
    Position = 10,
    Roundness = 11,
    Straightness = 12,
    SurfaceProfile = 13,
    Symmetry = 14,
    TotalRunout = 15,
}
/**
 * Defines type of tolerance zone form.
 */
enum ModelData_PMIGeometricToleranceZoneFormType {
    Undefined = 0,
    CylindricalOrCircular = 1,
    Spherical = 2,
    WithinACircle = 3,
    BetweenTwoConcentricCircles = 4,
    BetweenTwoEquidistantCurves = 5,
    WithinACylinder = 6,
    BetweenTwoCoaxialCylinders = 7,
    BetweenTwoEquidistantSurfaces = 8,
    NonUniform = 9,
}
/**
 * Defines PMI graphical element type.
 *
 * An element is complete elementary PMI entity such as dimension or tolerance measurement.
 *
 * An element stores a presentation of PMI element or annotation in graphical form. The information
 * to be presented can be stored in different ways: outlines, text or triangulation. Therefore different
 * components ({@link ModelData_PMIGraphicalElementComponent} subclasses) were introduced to enable complete
 * description of element's graphical presentation.
 *
 * An element contains components, plane, property table and its type.
 */
class ModelData_PMIGraphicalElement extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_PMIGraphicalElement.
     */
    constructor(theName?: string);
    /**
     * The plane on which the 2d components are placed.
     */
    plane: ModelData_PMIPlane | null;
    /**
     * Number of added components.
     */
    get numberOfComponents(): number;
    /**
     * Adds a component.
     */
    addComponent(theComponent: ModelData_PMIGraphicalElementComponent): void;
    /**
     * Returns `true` if no graphical element components were added.
     */
    isEmpty(): boolean;
    /**
     * Returns iterable to added components.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const component of graphicalElement.components()) {
     *   // Access to the component
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___p_m_i_graphical_element_1_1_component_iterator.html ModelData_PMIGraphicalElement::ComponentIterator} class.
     */
    components(): Iterable<ModelData_PMIGraphicalElementComponent>;
    /**
     * Accepts a component visitor.
     */
    accept(theVisitor: ModelData_PMIGraphicalElementComponentVisitor): void;
}
/**
 * Base class for various component types.
 */
abstract class ModelData_PMIGraphicalElementComponent {
    /**
     * Accepts a visitor.
     */
    abstract accept(theVisitor: ModelData_PMIGraphicalElementComponentVisitor): void;
}
/**
 * Defines a visitor of the components.
 */
abstract class ModelData_PMIGraphicalElementComponentVisitor {
    abstract visitOutlinedComponent(theComponent: ModelData_PMIOutlinedComponent): void;
    abstract visitTextComponent(theComponent: ModelData_PMITextComponent): void;
    abstract visitTriangulatedComponent(theComponent: ModelData_PMITriangulatedComponent): void;
}
/**
 * Defines a length unit.
 */
class ModelData_PMILengthUnitAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMILengthUnitAttribute.
     */
    constructor(theUnit?: Base_LengthUnit);
    /**
     * Length unit. Default is {@link Base_LengthUnit Base_LU_Millimeters}.
     */
    unit: Base_LengthUnit;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a kind of a tolerance class dimension such as form variance, zone variance, grade, etc. (ISO 286)
 */
class ModelData_PMILimitsAndFitsAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMILimitsAndFitsAttribute.
     */
    constructor(theType?: string, theValue?: string);
    /**
     * Attribute type.
     */
    type: string;
    /**
     * Attribute value.
     */
    value: string;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a machining allowance and its bound (deviations).
 */
class ModelData_PMIMachiningAllowanceAttribute extends ModelData_PMISemanticAttribute {
    constructor(theValue?: number);
    /**
     * Machining allowance value. Default is `0`.
     */
    value: number;
    /**
     * Upper bound. Default is `0`.
     */
    upperBound: number;
    /**
     * Lower bound. Default is `0`.
     */
    lowerBound: number;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a maximum value.
 */
class ModelData_PMIMaximumValueAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIMaximumValueAttribute.
     */
    constructor(theMaxValue?: number);
    /**
     * Maximum value. Default is `0`.
     */
    maxValue: number;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines the type of modification applied to a dimensional/geometric tolerance or a datum reference (e.g. statistical, free state, maximum material requirement, etc.).
 */
class ModelData_PMIModifierAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIModifierAttribute.
     */
    constructor(theModifier?: string);
    /**
     * Modifier name.
     */
    modifier: string;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines the type of modification applied to a tolerance with additional value (e.g. minor diameter, etc.).
 */
class ModelData_PMIModifierWithValueAttribute extends ModelData_PMIModifierAttribute {
    /**
     * Creates an instance of ModelData_PMIModifierWithValueAttribute.
     */
    constructor(theModifier?: string, theValue?: number);
    /**
     * Modifier name. Default is `0`.
     */
    value: number;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Base class for various outline types.
 */
abstract class ModelData_PMIOutline {
    /**
     * Accepts a visitor.
     */
    abstract accept(theVisitor: ModelData_PMIOutlineVisitor): void;
}
/**
 * Defines a visitor of the outlines.
 *
 * Method {@link ModelData_PMIOutlineVisitor#visitCompositeOutlineEnter visitCompositeOutlineEnter()} returns `true`
 * if the child outlines of the composite outline should be visited.
 * If the method returns `false` then they will not be visited.
 * The method {@link ModelData_PMIOutlineVisitor#visitCompositeOutlineLeave visitCompositeOutlineLeave()} will always be called.
 */
abstract class ModelData_PMIOutlineVisitor {
    abstract visitPolyOutline(theOutline: ModelData_PMIPolyOutline): void;
    abstract visitPoly2dOutline(theOutline: ModelData_PMIPoly2dOutline): void;
    abstract visitCurveOutline(theOutline: ModelData_PMICurveOutline): void;
    abstract visitCurve2dOutline(theOutline: ModelData_PMICurve2dOutline): void;
    abstract visitCompositeOutlineEnter(theOutline: ModelData_PMICompositeOutline): boolean;
    abstract visitCompositeOutlineLeave(theOutline: ModelData_PMICompositeOutline): void;
}
/**
 * Defines a component represented by outline.
 *
 * A component stores PMI graphical data expressed by outline(s), like: unfilled text, lines, frames, arrows, etc.
 *
 * The 2d elements of a component are defined by coordinates in the plane of its parent graphical element.
 */
class ModelData_PMIOutlinedComponent extends ModelData_PMIGraphicalElementComponent {
    /**
     * Creates an instance of ModelData_PMIOutlinedComponent.
     */
    constructor(theOutline?: ModelData_PMIOutline | null);
    /**
     * Associated outline.
     */
    outline: ModelData_PMIOutline | null;
    override accept(theVisitor: ModelData_PMIGraphicalElementComponentVisitor): void;
}
/**
 * Defines a plane where 2d graphical elements should be placed.
 *
 * A plane allows to position 2d graphical data on the screen.
 * A plane can be attached to graphical element. But note that it only makes sense in case
 * where element contains 2d data or text.
 *
 * > __Note__: The plane used by graphical element should be present in its table.
 */
class ModelData_PMIPlane extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_PMITable.
     */
    constructor(thePlane?: ModelData_Plane | null, theName?: string);
    /**
     * A geometrical plane.
     */
    plane: ModelData_Plane | null;
}
/**
 * Defines a plus and minus bounds (deviations) of a tolerance.
 */
class ModelData_PMIPlusMinusBoundsAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIPlusMinusBoundsAttribute.
     */
    constructor(theLowerBound?: number, theUpperBound?: number);
    /**
     * Returns upper bound. Default is `0`.
     */
    lowerBound: number;
    /**
     * Returns lower bound. Default is `0`;
     */
    upperBound: number;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines an outline consist of polylines.
 */
class ModelData_PMIPoly2dOutline extends ModelData_PMIOutline {
    /**
     * Creates an instance of ModelData_PMIPoly2dOutline.
     */
    constructor(theLineSet?: ModelData_PolyLine2dSet);
    lineSet: ModelData_PolyLine2dSet | null;
    override accept(theVisitor: ModelData_PMIOutlineVisitor): void;
}
/**
 * Defines an outline consist of polylines.
 */
class ModelData_PMIPolyOutline extends ModelData_PMIOutline {
    /**
     * Creates an instance of ModelData_PMIPolyOutline.
     */
    constructor(theLineSet?: ModelData_PolyLineSet | null);
    lineSet: ModelData_PolyLineSet | null;
    override accept(theVisitor: ModelData_PMIOutlineVisitor): void;
}
/**
 * Defines the type of qualifier that can limit a tolerance (e.g. max, min, etc.).
 */
class ModelData_PMIQualifierAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIQualifierAttribute.
     */
    constructor(theQualifier?: string);
    qualifier: string;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines range of value.
 */
class ModelData_PMIRangeAttribute extends ModelData_PMISemanticAttribute {
    /**
     * Creates an instance of ModelData_PMIRangeAttribute.
     */
    constructor(theLowerLimit?: number, theUpperLimit?: number);
    /**
     * Lower limit. Default is `0`.
     */
    lowerLimit: number;
    /**
     * Upper limit. Default is `0`.
     */
    upperLimit: number;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a view orientation of the model and a subset of visible PMI elements.
 *
 * A saved view facilitates presentation of the model and its annotation.
 * It defines a set of elements that should be visible from the specific point of view.
 *
 * A saved view contains graphical elements and camera.
 */
class ModelData_PMISavedView extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_PMISavedView.
     */
    constructor(theCamera?: ModelData_PMICamera, theName?: string);
    camera: ModelData_PMICamera | null;
    /**
     * Number of added graphical elements.
     */
    get numberOfGraphicalElements(): number;
    /**
     * Adds a graphical element.
     */
    addGraphicalElement(theElement: ModelData_PMIGraphicalElement): void;
    /**
     * Returns a specified element.
     */
    graphicalElement(theIndex: number): ModelData_PMIGraphicalElement | undefined;
    /**
     * Returns iterable to outlines.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const element of savedView.graphicalElement()) {
     *   // Access to the element
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___p_m_i_saved_view.html ModelData_PMISavedView::GraphicalElementIterator} class.
     */
    graphicalElements(): Iterable<ModelData_PMIGraphicalElement>;
}
/**
 * Base class for various attribute types.
 */
abstract class ModelData_PMISemanticAttribute extends ModelData_BaseObject {
    abstract accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines a visitor of the attributes.
 */
abstract class ModelData_PMISemanticAttributeVisitor {
    abstract visitAngleUnitAttribute(theAttribute: ModelData_PMIAngleUnitAttribute): void;
    abstract visitDatumRefAttribute(theAttribute: ModelData_PMIDatumRefAttribute): void;
    abstract visitDatumRefCompartmentAttribute(theAttribute: ModelData_PMIDatumRefCompartmentAttribute): void;
    abstract visitDatumTargetAttribute(theAttribute: ModelData_PMIDatumTargetAttribute): void;
    abstract visitDisplacementAttribute(theAttribute: ModelData_PMIDisplacementAttribute): void;
    abstract visitLengthUnitAttribute(theAttribute: ModelData_PMILengthUnitAttribute): void;
    abstract visitLimitsAndFitsAttribute(theAttribute: ModelData_PMILimitsAndFitsAttribute): void;
    abstract visitMaximumValueAttribute(theAttribute: ModelData_PMIMaximumValueAttribute): void;
    abstract visitModifierAttribute(theAttribute: ModelData_PMIModifierAttribute): void;
    abstract visitModifierWithValueAttribute(theAttribute: ModelData_PMIModifierWithValueAttribute): void;
    abstract visitPlusMinusBoundsAttribute(theAttribute: ModelData_PMIPlusMinusBoundsAttribute): void;
    abstract visitQualifierAttribute(theAttribute: ModelData_PMIQualifierAttribute): void;
    abstract visitRangeAttribute(theAttribute: ModelData_PMIRangeAttribute): void;
    abstract visitMachiningAllowanceAttribute(theAttribute: ModelData_PMIMachiningAllowanceAttribute): void;
    abstract visitSurfaceTextureRequirementAttribute(theAttribute: ModelData_PMISurfaceTextureRequirementAttribute): void;
}
/**
 * Defines PMI semantic element type.
 *
 * The ModelData_PMISemanticElement class stores a presentation of PMI entity in semantic form.
 * The stored information may be annotations associated with a CAD model's edges and faces such as dimensional/geometric tolerances or datum features.
 * Like graphical element semantic element may consists of several components ({@link ModelData_PMISemanticElementComponent} subclasses) each one representing specific semantic notation.
 */
class ModelData_PMISemanticElement extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_PMISemanticElement.
     */
    constructor(theName?: string);
    /**
     * Number of added components.
     */
    get numberOfComponents(): number;
    /**
     * Adds a component.
     */
    addComponent(theComponent: ModelData_PMISemanticElementComponent): void;
    /**
     * Returns `true` if no semantic element components were added.
     */
    isEmpty(): boolean;
    /**
     * Returns iterable to added components.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const component of semanticElement.components()) {
     *   // Access to the component
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___p_m_i_semantic_element_1_1_component_iterator.html ModelData_PMISemanticElement::ComponentIterator} class.
     */
    components(): Iterable<ModelData_PMISemanticElementComponent>;
}
/**
 * Base class for various component types.
 */
abstract class ModelData_PMISemanticElementComponent extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_PMISemanticElementComponent.
     */
    constructor();
    /**
     * Number of added attributes.
     */
    get numberOfAttributes(): number;
    /**
     * Adds an attribute.
     */
    addAttribute(theAttribute: ModelData_PMISemanticAttribute): void;
    /**
     * Returns `true` if semantic attributes were added and `false` otherwise.
     */
    hasAttributes(): boolean;
    /**
     * Returns iterable to added attributes.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const attribute of semanticElementComponent.attributes()) {
     *   // Access to the attribute
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___p_m_i_semantic_element_component_1_1_attribute_iterator.html ModelData_PMISemanticElementComponent::AttributeIterator} class.
     */
    attributes(): Iterable<ModelData_PMISemanticAttribute>;
    /**
     * Accepts an attribute visitor.
     */
    acceptAttributeVisitor(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
    /**
     * Accepts a visitor.
     */
    abstract accept(theVisitor: ModelData_PMISemanticElementComponentVisitor): void;
}
/**
 * Defines a visitor of the components.
 */
abstract class ModelData_PMISemanticElementComponentVisitor {
    abstract visitDatumComponent(theComponent: ModelData_PMIDatumComponent): void;
    abstract visitDimensionComponent(theComponent: ModelData_PMIDimensionComponent): void;
    abstract visitGeometricToleranceComponent(theComponent: ModelData_PMIGeometricToleranceComponent): void;
    abstract visitSurfaceFinishComponent(theComponent: ModelData_PMISurfaceFinishComponent): void;
}
/**
 * Defines a component represented by a surface texture.
 *
 * Specifies the texture of a surface through surface texture requirements, manufacturing method
 * surface lay and its orientation, possible machining allowance, etc.
 *
 * > *Note*: The component can store multiple {@link ModelData_PMISurfaceTextureRequirementAttribute},
 *    so {@link ModelData_PMISurfaceTextureRequirementAttribute#Precedence} method is used
 *    to determine the order of precedence (primary, secondary, tertiary, etc).
 */
class ModelData_PMISurfaceFinishComponent extends ModelData_PMISemanticElementComponent {
    constructor(theValue?: ModelData_PMISurfaceFinishMaterialRemovalType);
    /**
     * Type of material removal.
     */
    materialRemoval: ModelData_PMISurfaceFinishMaterialRemovalType;
    /**
     * Type of lay direction.
     */
    layDirection: ModelData_PMISurfaceFinishLayDirectionType;
    /**
     * Indicates whether the same surface texture is required on all surfaces around a workpiece outline.
     */
    isAllAround: boolean;
    /**
     * Manufacturing method.
     */
    manufacturingMethod: string;
    override accept(theVisitor: ModelData_PMISemanticElementComponentVisitor): void;
}
/**
 * Defines type of lay direction.
 */
enum ModelData_PMISurfaceFinishLayDirectionType {
    Undefined = 0,
    Parallel = 1,
    Multidirectional = 2,
    Perpendicular = 3,
    Circular = 4,
    Particulate = 5,
    Crossed = 6,
    Radial = 7,
}
/**
 * Defines type of material removal.
 */
enum ModelData_PMISurfaceFinishMaterialRemovalType {
    Any = 0,
    Required = 1,
    Prohibited = 2,
}
/**
 * Defines a surface texture requirement.
 *
 * See {@link https://docs.cadexchanger.com/sdkcsharp/classcadex_1_1_model_data___p_m_i_surface_texture_requirement_attribute.html#details C++ documentation} for more details.
 */
class ModelData_PMISurfaceTextureRequirementAttribute extends ModelData_PMISemanticAttribute {
    constructor(thePrecedence?: number, theParameter?: ModelData_PMISurfaceTextureRequirementAttributeSurfaceParameterType, theLimitValue?: number);
    /**
     * Precedence value. Default is `1`.
     */
    precedence: number;
    /**
     * Specification limit. Default is {@link ModelData_PMISurfaceTextureRequirementAttributeSpecificationLimitType UndefinedLimit}.
     */
    specificationLimit: ModelData_PMISurfaceTextureRequirementAttributeSpecificationLimitType;
    /**
     * Filter name. Default is empty string.
     */
    filterName: string;
    /**
     * Short wave filter value. Default is `0`.
     */
    shortWaveFilter: number;
    /**
     * Long wave filter value. Default is `0`.
     */
    longWaveFilter: number;
    /**
     * Surface parameter. Default is {@link ModelData_PMISurfaceTextureRequirementAttributeSurfaceParameterType UndefinedParameter}
     */
    surfaceParameter: ModelData_PMISurfaceTextureRequirementAttributeSurfaceParameterType;
    /**
     * Evaluation length value. Default is `5`.
     */
    evaluationLength: number;
    /**
     * Comparison rule. Default is {@link ModelData_PMISurfaceTextureRequirementAttributeComparisonRuleType SixteenPercent}.
     */
    comparisonRule: ModelData_PMISurfaceTextureRequirementAttributeComparisonRuleType;
    /**
     * Limit value. Default is `0`.
     * > Note: value is given in micrometers or microinches (depends on the value of {@link ModelData_PMILengthUnitAttribute})
     */
    limitValue: number;
    override accept(theVisitor: ModelData_PMISemanticAttributeVisitor): void;
}
/**
 * Defines comparison rule types.
 */
enum ModelData_PMISurfaceTextureRequirementAttributeComparisonRuleType {
    SixteenPercent = 0,
    Max = 1,
}
/**
 * Defines specification limit types.
 */
enum ModelData_PMISurfaceTextureRequirementAttributeSpecificationLimitType {
    UndefinedLimit = 0,
    Upper = 1,
    Lower = 2,
}
/**
 * Defines surface parameter types.
 */
enum ModelData_PMISurfaceTextureRequirementAttributeSurfaceParameterType {
    UndefinedParameter = 0,
    Rp = 1,
    Rv = 2,
    Rz = 3,
    Rc = 4,
    Rt = 5,
    Ra = 6,
    Rq = 7,
    Rsk = 8,
    Rku = 9,
    Wp = 100,
    Wv = 101,
    Wz = 102,
    Wc = 103,
    Wt = 104,
    Wa = 105,
    Wq = 106,
    Wsk = 107,
    Wku = 108,
    Pp = 200,
    Pv = 201,
    Pz = 202,
    Pc = 203,
    Pt = 204,
    Pa = 205,
    Pq = 206,
    Psk = 207,
    Pku = 208,
    N1 = 300,
    N2 = 301,
    N3 = 302,
    N4 = 303,
    N5 = 304,
    N6 = 305,
    N7 = 306,
    N8 = 307,
    N9 = 308,
    N10 = 309,
    N11 = 310,
    N12 = 311,
}
/**
 * Provides {@link ModelData_PMITable PMI table}.
 *
 * Used to defer population of the PMI data and thereby to reduce peak memory footprint.
 */
abstract class ModelData_PMITableProvider {
    /**
     * Feeds the PMI table.
     * This method can depend of the network requests which makes asynchronously, so returns Promise.
     * Subclasses must redefine this method to call {@link ModelData_PMITable#add add()} of thePMITable.
     */
    abstract feed(thePMITable: ModelData_PMITable, theScope: Base_ProgressScope): Promise<void>;
}
/**
 * Defines a container storing PMI data.
 *
 * A table stores PMI specific information.
 *
 * A table can be attached to any scene graph element ({@link ModelData_SceneGraphElement} subclass).
 */
class ModelData_PMITable extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_PMITable.
     */
    constructor(theName?: string);
    /**
     * Number of added PMI data objects.
     */
    get numberOfPMIData(): number;
    /**
     * The number of Saved View objects.
     */
    get numberOfSavedViews(): number;
    /**
     * Number of PMI planes.
     */
    get numberOfPlanes(): number;
    /**
     * Returns list of graphical elements.
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___p_m_i_table.html ModelData_PMITable::PMIDataIterator} class.
     */
    pmiDataItems(theScope?: Base_ProgressScope): Promise<ReadonlyArray<ModelData_PMIData>>;
    /**
     * Returns list of saved view.
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___p_m_i_table.html ModelData_PMITable::SavedViewIterator} class.
     */
    views(theScope?: Base_ProgressScope): Promise<ReadonlyArray<ModelData_PMISavedView>>;
    /**
     * Returns list of saved view.
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___p_m_i_table.html ModelData_PMITable::PlaneIterator} class.
     */
    planes(theScope?: Base_ProgressScope): Promise<ReadonlyArray<ModelData_PMIPlane>>;
    /**
     * Adds PMI table provider.
     */
    addProvider(theProvider: ModelData_PMITableProvider): void;
    /**
     * Appends a collection of PMI entities.
     */
    addTable(theTable: ModelData_PMITable): void;
    /**
     * Adds a graphical element.
     */
    add(theData: ModelData_PMIData): void;
    /**
     * Adds a saved view.
     */
    addView(theView: ModelData_PMISavedView): void;
    /**
     * Adds a plane.
     */
    addPlane(thePlane: ModelData_PMIPlane): void;
    /**
     * Returns `true` if no PMI items or providers have been added.
     */
    isEmpty(): boolean;
    /**
     * Clear all data in the PMI table.
     */
    clear(): void;
}
/**
 * Defines a component represented by text.
 *
 * A component stores PMI graphical data expressed by a string.
 *
 * A component contains text and outline. Outline is optional and may be used as fallback,
 * when target format doesn't support PMI text entities. It also may be used for fast screen rendering.
 *
 * The elements of a component are defined by coordinates in the plane of its parent graphical element.
 */
class ModelData_PMITextComponent extends ModelData_PMIGraphicalElementComponent {
    /**
     * Creates an instance of ModelData_PMITextComponent.
     */
    constructor();
    /**
     * Text to display. Default is `null`.
     */
    text: string | null;
    /**
     * Size of the font. Default is `16`.
     */
    fontSize: number;
    /**
     * Position of the text on the 2d plane. Default is `(0, 0)`.
     */
    textOrigin: ModelData_Point2d;
    /**
     * Direction of the text on the 2d plane. Default is `(1, 0)`.
     */
    textDirection: ModelData_Direction2d;
    /**
     * Outline for text. Default is `null`.
     */
    outline: ModelData_PMIOutline | null;
    override accept(theVisitor: ModelData_PMIGraphicalElementComponentVisitor): void;
}
/**
 * Defines a component represented by triangulation.
 *
 * A component stores PMI graphical data expressed by triangulation,
 * like filled text or any other triangulated planar shape.
 *
 * A component contains indexed triangle set.
 */
class ModelData_PMITriangulatedComponent extends ModelData_PMIGraphicalElementComponent {
    /**
     * Creates an instance of ModelData_PMITriangulatedComponent.
     */
    constructor(theTriangleSet?: ModelData_IndexedTriangleSet);
    triangleSet: ModelData_IndexedTriangleSet | null;
    override accept(theVisitor: ModelData_PMIGraphicalElementComponentVisitor): void;
}
/**
 * Defines a PMI graphical element type.
 */
enum ModelData_PMIType {
    Note = 0,
    Dimension = 1,
    Tolerance = 2,
    CoordinateSystem = 3,
    CenterLine = 4,
    ReferenceGeometry = 5,
    MeasurementPoint = 6,
    Datum = 7,
    FeatureControlFrame = 8,
    Weld = 9,
    SurfaceFinish = 10,
    Section = 11,
    Undefined = 1000,
    UserDefined = 1001,
}
/**
 * Defines a 3D point.
 */
class ModelData_Point {
    /**
     * Creates an instance of ModelData_Point.
     */
    constructor(x?: number, y?: number, z?: number);
    /**
     * A tolerance to compare coincidence of two points.
     */
    static readonly coincidenceTolerance: number;
    /**
     * X coord.
     */
    get x(): number;
    set x(x: number);
    /**
     * Y coord.
     */
    get y(): number;
    set y(y: number);
    /**
     * Z coord.
     */
    get z(): number;
    set z(z: number);
    /**
     * Creates point from object implementing {@link ModelData_XYZ} interface.
     */
    static fromXYZ(theXYZ: ModelData_XYZ): ModelData_Point;
    /**
     * Sets coordinates.
     */
    setCoord(x: number, y: number, z: number): this;
    /**
     * Adds point to current.
     */
    add(thePoint: ModelData_XYZ): this;
    /**
     * Subtracts point from current.
     */
    sub(thePoint: ModelData_XYZ): this;
    /**
     * Multiplies point by scalar.
     */
    multiplyScalar(theScalar: number): this;
    divideScalar(theScalar: number): this;
    translate(theVector: ModelData_Vector): this;
    translated(theVector: ModelData_Vector): this;
    /**
     * Transforms the point with a transformation matrix.
     */
    transform(theTransformation: ModelData_Transformation): this;
    /**
     * Returns a new point which is transformed with a transformation matrix.
     */
    transformed(theTransformation: ModelData_Transformation): ModelData_Point;
    /**
     * Returns distance between current point and other point.
     *
     * > __Note__: Method uses Math.Sqrt() what may incur performance penalty,
     * so favor using `squareDistance` method in performance-sensitive algorithms.
     */
    distance(thePoint: ModelData_XYZ): number;
    /**
     * Returns square distance between current point and other point.
     */
    squareDistance(theOther: ModelData_XYZ): number;
    /**
     * Compares two points with specified tolerance.
     */
    isEqual(thePoint: ModelData_Point, theTolerance?: number): boolean;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Point;
    /**
     * Copies values from another point.
     */
    copy(thePoint: ModelData_XYZ): this;
    fromArray(theArray: Array<number>, theOffset?: number): this;
    /**
     * Returns string representation of the point.
     */
    toString(): string;
    /**
     * Returns JSON representation of the point.
     */
    toJSON(): ModelData_XYZ;
}
/**
 * Represents point in 2D space.
 */
class ModelData_Point2d {
    /**
     * Creates an instance of ModelData_Point2d.
     */
    constructor(x?: number, y?: number);
    /**
     * A tolerance to compare coincidence of two points.
     */
    static readonly coincidenceTolerance: number;
    /**
     * X coord.
     */
    get x(): number;
    set x(x: number);
    /**
     * Y coord.
     */
    get y(): number;
    set y(y: number);
    /**
     * Creates point from object implementing {@link ModelData_XY} interface.
     */
    static fromXY(theXY: ModelData_XY): ModelData_Point2d;
    /**
     * Sets coordinates.
     */
    setCoord(x: number, y: number): this;
    /**
     * Adds point to current.
     */
    add(thePoint: ModelData_XY): this;
    /**
     * Subtract point from current.
     */
    sub(thePoint: ModelData_XY): this;
    /**
     * Multiplies point by scalar.
     */
    multiplyScalar(theScalar: number): this;
    translate(theVector: ModelData_Vector2d): this;
    /**
     * Returns distance between current point and other point.
     *
     * > __Note__: Method uses Math.Sqrt() what may incur performance penalty,
     * so favor using `squareDistance` method in performance-sensitive algorithms.
     */
    distance(thePoint: ModelData_XY): number;
    /**
     * Returns square distance between current point and other point.
     */
    squareDistance(theOther: ModelData_XY): number;
    /**
     * Compares two points with specified tolerance.
     */
    isEqual(thePoint: ModelData_Point2d, theTolerance?: number): boolean;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Point2d;
    /**
     * Copies values from another point.
     */
    copy(thePoint: ModelData_XY): this;
    fromArray(theArray: Array<number>, theOffset?: number): this;
    /**
     * Returns string representation of the point.
     */
    toString(): string;
    /**
     * Returns JSON representation of the point.
     */
    toJSON(): ModelData_XY;
}
class ModelData_PointProperties {
    /**
     * Creates an instance of ModelData_PointProperties.
     */
    constructor(theWidthScaleFactor?: number);
    /**
     * A multiplicative value that scales nominal point size.
     * A value less than or equal to zero refers to the minimum available point size.
     */
    widthScaleFactor: number;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_PointProperties;
    /**
     * Copies parameters from another object.
     */
    copy(thePointProperties: ModelData_PointProperties): ModelData_PointProperties;
    /**
     * Equality operator.
     */
    isEqual(thePointProperties: ModelData_PointProperties | null | undefined): boolean;
}
/**
 * Defines a polygonal shape consisting of 2d polylines.
 */
class ModelData_PolyLine2dSet extends ModelData_PolyVertex2dSet {
    /**
     * Number of poly lines.
     */
    get numberOfPolylines(): number;
    /**
     * Returns a number of vertices in polyline.
     */
    numberOfLineVertices(thePolyline: number): number;
    /**
     * Returns a coordinate index for a vertex in a line.
     */
    coordinateIndex(thePolyline: number, theVertexSlot: number): number;
    /**
     * Adds a polyline into the set.
     */
    addPolyLine(theVertices: Array<ModelData_Point2d>): void;
}
/**
 * Defines a polygonal shape consisting of poly lines.
 */
class ModelData_PolyLineSet extends ModelData_PolyVertexSet {
    /**
     * Number of poly lines.
     */
    get numberOfPolylines(): number;
    /**
     * Creates a new instance of ModelData_PolyLineSet with the given attributes
     */
    static create(theLines: Array<Array<ModelData_Point>>, theColors?: Array<ModelData_ColorObject>): ModelData_PolyLineSet;
    /**
     * Returns a number of vertices in polyline.
     */
    numberOfLineVertices(thePolyline: number): number;
    /**
     * Returns a coordinate index for a vertex in a line.
     */
    coordinateIndex(thePolyline: number, theVertexSlot: number): number;
}
/**
 * Defines a polygonal shape consisting of individual 2d points.
 */
class ModelData_PolyPoint2dSet extends ModelData_PolyVertex2dSet {
    /**
     * Adds points into a point set.
     */
    add(thePoints: ModelData_Point2d | Array<ModelData_Point2d>): void;
}
/**
 * Defines a polygonal shape consisting of individual points.
 */
class ModelData_PolyPointSet extends ModelData_PolyVertexSet {
    /**
     * Creates a new instance of ModelData_PolyPointSet with the given attributes
     */
    static create(theCoords: Array<ModelData_Point>, theColors?: Array<ModelData_ColorObject>): ModelData_PolyPointSet;
}
/**
 * Provides {@link ModelData_PolyRepresentation polygonal representation}.
 *
 * Used to defer construction of the polygonal representation and thereby to reduce peak memory footprint.
 */
abstract class ModelData_PolyRepresentationProvider {
    /**
     * Feeds the representation.
     * This method can depend of the network requests which makes asynchronously, so it returns Promise.
     * Subclasses must redefine this method to call {@link ModelData_PolyRepresentation#add add()}
     * of theRepresentation.
     */
    abstract feed(theRepresentation: ModelData_PolyRepresentation, theScope: Base_ProgressScope): Promise<void>;
}
/**
 * Defines polygonal (faceted or tessellated) representation of part.
 */
class ModelData_PolyRepresentation extends ModelData_Representation {
    /**
     * Creates an instance of ModelData_PolyRepresentation.
     */
    constructor(theShape?: ModelData_PolyVertexSet);
    /**
     * Adds a polygonal shape.
     */
    add(theShape: ModelData_PolyVertexSet): void;
    /**
     * Adds Poly representations provider.
     */
    addProvider(theProvider: ModelData_PolyRepresentationProvider): void;
    /**
     * Returns Promise for list of shapes. This method flushes providers.
     */
    polyShapeList(theScope?: Base_ProgressScope): Promise<ModelData_PolyShapeList>;
    override accept(theVisitor: ModelData_RepresentationVisitor): Promise<void>;
}
/**
 * Base class for polygonal meshes.
 */
abstract class ModelData_PolyShape extends ModelData_BaseObject {
}
/**
 * Base class for polygonal 2d shapes.
 */
class ModelData_PolyShape2d extends ModelData_BaseObject {
}
/**
 * Defines a list of poly shapes.
 * Body list is used by ModelData_PolyRepresentation to hold root shapes.
 */
class ModelData_PolyShapeList extends ModelData_PolyShape implements Iterable<ModelData_PolyVertexSet> {
    /**
     * Adds body to the body list.
     */
    append(thePolyShape: ModelData_PolyVertexSet): void;
    /**
     * Returns the number of added poly shapes.
     */
    size(): number;
    /**
     * Retrieve poly shape by index.
     */
    element(theIndex: number): ModelData_PolyVertexSet;
    [Symbol.iterator](): Iterator<ModelData_PolyVertexSet>;
}
/**
 * Base class for particular 2d vertex sets - polyline sets, point sets.
 *
 * The vertex set defines a set of vertex coordinates which is used by subclass.
 */
class ModelData_PolyVertex2dSet extends ModelData_PolyShape2d {
    /**
     * Number of vertices.
     */
    get numberOfVertices(): number;
    /**
     * Returns a vertex coordinate per its index.
     *
     * > __Note__: changing returned object does not change original coord.
     */
    coordinate(theIndex: number, theTarget?: ModelData_Point2d): ModelData_Point2d | undefined;
}
/**
 * Base class for particular vertex sets - triangle sets, polyline sets, point sets.
 *
 * The vertex set can optionally contain colors which are defined per each vertex.
 *
 *
 * Unlike {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___poly_vertex_set.html C++ SDK Poly Vertex Set}
 * it allows specify appearance for groups of the primitives.
 * Typically multi appearance poly vertex set represents array of colored poly vertex sets
 * merged into one big poly vertex set to save memory and faster rendering.
 * CAD Exchanger SDK generates such geometry by default for WEB Toolkit.
 */
abstract class ModelData_PolyVertexSet extends ModelData_PolyShape {
    /**
     * Creates an instance of ModelData_PolyVertexSet.
     */
    constructor();
    /**
     * Optional appearance assigned with poly vertex set.
     */
    appearance: ModelData_Appearance | null;
    /**
     * Array of coords in {x,y,z} format [x,y,z,x,y,z,...].
     */
    coords: Float32Array | null;
    /**
     * Array of colors in RGB ([R,G,B,R,G,B,...]) or RGBA [R,G,B,A,R,G,B,A,...] format.
     */
    colors: Float32Array | null;
    /**
     * List of appearances referenced by groups.
     * If no one group has been explicitly added, returns an array with poly vertex set appearance.
     */
    get appearances(): Array<(ModelData_Appearance|undefined)>;
    /**
     * Render groups.
     * If no one group has been explicitly added, returns an array with group corresponds to whole poly vertex set.
     */
    get groups(): ReadonlyArray<ModelData_PrimitivesGroup>;
    /**
     * Number of coord elements.
     */
    get numberOfVertices(): number;
    /**
     * Number of color elements.
     */
    get numberOfColors(): number;
    /**
     * Adds a render group.
     */
    addGroup(theStart: number, theCount: number, theAppearance?: ModelData_Appearance): ModelData_PrimitivesGroup;
    /**
     * Remove all groups and appearances associated with current poly vertex set.
     */
    removeGroups(): void;
    /**
     * Returns a vertex coordinate per its index.
     *
     * > __Note__: changing returned object does not change original coord.
     */
    coordinate(theIndex: number, theTarget?: ModelData_Point): ModelData_Point | undefined;
    /**
     * Returns a color per its index.
     *
     * > __Note__: changing returned object does not change original color.
     */
    color(theIndex: number, theTarget?: ModelData_ColorObject): ModelData_ColorObject | undefined;
    /**
     * Returns `true` if object has colors per vertex.
     */
    hasColors(): boolean;
}
/**
 * Defines primitives group with appearance.
 *
 * > __Note__: the group always refers to appearance array item, which can have `undefined` value.
 */
interface ModelData_PrimitivesGroup {
    /**
     * Start index of the group.
     */
    start: number;
    /**
     * Number of elements in the group.
     */
    count: number;
    /**
     * Index of appearance associated with current group.
     */
    appearanceIndex: number;
}
/**
 * Provides properties to the {@link ModelData_PropertyTable property table}.
 *
 * Used to defer population of the properties and thereby to reduce peak memory footprint.
 */
abstract class ModelData_PropertyTableProvider {
    /**
     * Feeds the property table.
     * This method can depend of the network request which makes asynchronously, so it should return Promise.
     * Subclasses must redefine this method to call {@link ModelData_PropertyTable#add add()} of theTable.
     */
    abstract feed(theTable: ModelData_PropertyTable, theScope: Base_ProgressScope): Promise<void>;
}
/**
 * Defines a collection of properties.
 *
 * Property table is a list of pairs {property_name, property_value}. A property name is defined with a string.
 * A value can have any of the following types:
 * * number
 * * string
 * * timestamp
 * * 3D Point
 * * 3D Box
 *
 * In addition, validation properties (surface area, volume, centroid, etc) can be registered using
 * reserved property names.
 *
 * A property table can be attached to any scene graph element ({@link ModelData_SceneGraphElement ModelData_SceneGraphElement} subclass)
 * or a subshape in {@link ModelData_BRepRepresentation ModelData_BRepRepresentation}.
 *
 * A property table can be checked with IsEmpty(), Size() and cleared with Clear() method.
 */
class ModelData_PropertyTable extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_PropertyTable.
     */
    constructor(theName?: string);
    /**
     * Key to designate a 'name' property.
     */
    static readonly NamePropertyName: string;
    /**
     * Key to designate a 'surface area' validation property.
     */
    static readonly SurfaceAreaPropertyName: string;
    /**
     * Key to designate a 'volume' validation property.
     */
    static readonly VolumePropertyName: string;
    /**
     * Key to designate a 'centroid' validation property.
     */
    static readonly CentroidPropertyName: string;
    /**
     * Key to designate a 'bounding box' property.
     */
    static readonly BoundingBoxPropertyName: string;
    /**
     * Adds property to table.
     */
    add(theKey: string, theValue: string | number | Date | ModelData_Point | ModelData_Box): void;
    /**
     * Appends properties from another property table.
     */
    addTable(thePropertyTable: ModelData_PropertyTable): void;
    /**
     * Adds property table provider.
     */
    addProvider(theProvider: ModelData_PropertyTableProvider): void;
    /**
     * Returns dictionary object. Keys are properties names, values are properties values.
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___property_table.html ModelData_PropertyTable::Accept()} method.
     */
    properties(theScope?: Base_ProgressScope): Promise<Record<string, (string|number|Date|ModelData_Point|ModelData_Box)>>;
    /**
     * Returns `true` if no property items or providers have been added.
     */
    isEmpty(): boolean;
    /**
     * Returns the number of added properties.
     */
    size(): number;
    /**
     * Returns `Promise` for property value. If property is not existed, the promise will be resolved with `undefined`.
     */
    findProperty(theName: string, theFlushProviders?: boolean): Promise<(string|number|Date|ModelData_Point|ModelData_Box|undefined)>;
    /**
     * Discards associated data.
     * Providers which have been added with their deleters will be deleted.
     */
    clear(): void;
}
/**
 * Holds the ratio of 2 positive integers in the form of (n:m).
 *
 * Can be used to define the scale factor in certain situations.
 * The numerator and denominator are not reduced, they are stored as is.
 */
class ModelData_Ratio {
    constructor(theNumerator?: number, theDenominator?: number);
    /**
     * The numerator of the ratio. Default is `1`.
     */
    numerator: number;
    /**
     * The denominator of the ratio. Default is `1`.
     */
    denominator: number;
    /**
     * Converts the ratio to double floating-point number.
     */
    toDouble(): number;
    /**
     * Given 2 ratios (n:m) and (p:q) their equality is tested as equality of
     * rational numbers (n/m) and (p/q).
     */
    isEqual(theOther: ModelData_Ratio): boolean;
}
/**
 * Base class for all representations.
 */
abstract class ModelData_Representation extends ModelData_BaseObject {
    /**
     * Accepts a representation visitor.
     */
    abstract accept(theVisitor: ModelData_RepresentationVisitor): Promise<void>;
}
/**
 * Defines a visitor of the representations.
 * This visitor corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___part.html ModelData_Part::RepresentationVisitor} class.
 */
abstract class ModelData_RepresentationVisitor {
    abstract visitBRepRepresentation(theBRepRep: ModelData_BRepRepresentation): Promise<void> | void;
    abstract visitPolyRepresentation(thePolyRep: ModelData_PolyRepresentation): Promise<void> | void;
}
/**
 * Defines a mask to filter part representations.
 */
enum ModelData_RepresentationMask {
    /**
     * Filter only B-Rep representation.
     */
    ModelData_RM_BRep = 1,
    /**
     * Filter coarse Poly representation.
     */
    ModelData_RM_CoarseLOD = 16,
    /**
     * Filter medium Poly representation.
     */
    ModelData_RM_MediumLOD = 32,
    /**
     * Filter fine Poly representation.
     */
    ModelData_RM_FineLOD = 64,
    /**
     * Filter any Poly representation.
     */
    ModelData_RM_Poly = 112,
    /**
     * Filter any representation.
     */
    ModelData_RM_Any = 113,
}
/**
 * Base class for part, instance and assembly.
 */
abstract class ModelData_SceneGraphElement extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_SceneGraphElement.
     */
    constructor(theName?: string | null);
    /**
     * Appearance of the element.
     *
     * > __Note:__ By convention assemblies may not have appearances.
     * If one is set using this function it can be ignored by other CAD Exchanger functions (e.g. export).
     */
    appearance: ModelData_Appearance | null;
    /**
     * Property table of the element.
     */
    get properties(): ModelData_PropertyTable | null;
    /**
     * PMI table of the element.
     */
    get pmi(): ModelData_PMITable | null;
    /**
     * Adds a property table to the element.
     * If this object did not have properties yet, then theProperties are just linked to this object.
     * Otherwise, the elements of theProperties are appended to existing property table
     * (see {@link ModelData_PropertyTable#add ModelData_PropertyTable#add()} method).
     */
    addProperties(theProperties: ModelData_PropertyTable): void;
    /**
     * Adds a PMI table to the element.
     *
     * If this object did not have PMI yet, then thePMI are just linked to this object.
     * Otherwise, the elements of thePMI are appended to existing PMI table
     * (see {@link ModelData_PMITable#add ModelData_PMITable#add()}).
     */
    addPMI(thePMI: ModelData_PMITable): void;
    /**
     * Adds element to layer.
     */
    addToLayer(theLayer: ModelData_Layer): void;
    /**
     * Returns iterable to layers.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const layer of sge.layers()) {
     *   // Access to the layer
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___scene_graph_element_1_1_layer_iterator.html ModelData_SceneGraphElement::LayerIterator} class.
     */
    layers(): Iterable<ModelData_Layer>;
    /**
     * Accepts an element visitor.
     *
     * The order of visiting depends on the type of this object:
     *
     * * for a part invokes a visitor for the part ({@link ModelData_SceneGraphElementVisitor#visitPart visitPart()});
     *
     * * for an instance first enters the instance by calling
     * {@link ModelData_SceneGraphElementVisitor#visitInstanceEnter visitInstanceEnter()}.
     * If it returns `true` then visits the referred element. Then leaves the instance calling
     * {@link ModelData_SceneGraphElementVisitor#visitInstanceLeave visitInstanceLeave()}.
     *
     * * for an assembly first enters the assembly by calling
     * {@link ModelData_SceneGraphElementVisitor#visitAssemblyEnter visitAssemblyEnter()}.
     * If it returns `true` then visits all the assembly children. Then leaves the assembly calling
     * {@link ModelData_SceneGraphElementVisitor#visitAssemblyLeave visitAssemblyLeave()}.
     */
    abstract accept(theVisitor: ModelData_SceneGraphElementVisitor): Promise<void>;
}
/**
 * @deprecated
 */
type ModelData_CombinedSceneGraphElementVisitor = ModelData_SceneGraphElementCombinedVisitor;
/**
 * Provides combined methods `visitElementEnter` and `visitElementLeave` to visit all elements.
 *
 * This visitor corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___model_1_1_combined_element_visitor.html ModelData_Model::CombinedElementVisitor} class.
 */
abstract class ModelData_SceneGraphElementCombinedVisitor extends ModelData_SceneGraphElementVisitor {
    visitElementEnter(theElement: ModelData_SceneGraphElement): Promise<boolean> | boolean;
    visitElementLeave(theElement: ModelData_SceneGraphElement): Promise<void> | void;
    override visitPart(thePart: ModelData_Part): Promise<void> | void;
    override visitInstanceEnter(theInstance: ModelData_Instance): Promise<boolean> | boolean;
    override visitInstanceLeave(theInstance: ModelData_Instance): Promise<void> | void;
    override visitAssemblyEnter(theAssembly: ModelData_Assembly): Promise<boolean> | boolean;
    override visitAssemblyLeave(theAssembly: ModelData_Assembly): Promise<void> | void;
}
/**
 * Defines a visitor that visits each unique element only once.
 *
 * Children of already visited assemblies or referred element of already visited instances,
 * or already visited parts will never be visited again.
 *
 * Already visited instances or assemblies will be entered with `reference` visitor `visitEnter()` as well
 * as left with `visitLeave()`.
 *
 * Uses the associated visitor (`reference`) to really visit an element.
 */
class ModelData_SceneGraphElementUniqueVisitor extends ModelData_SceneGraphElementVisitor {
    constructor(theRef: ModelData_SceneGraphElementVisitor);
    /**
     * The associated visitor.
     */
    reference: ModelData_SceneGraphElementVisitor;
    /**
     * Invokes `visitPart` for the `reference` visitor if the part has not been already visited.
     */
    override visitPart(thePart: ModelData_Part): Promise<void> | void;
    /**
     * Enters an instance.
     *
     * Always invokes `visitInstanceEnter` for the `reference` visitor, but returns `true` only if the instance is
     * being visited for the first time and `reference` visitor returned `true`.
     * Thus referred element of already visited instance won't be visited again.
     */
    override visitInstanceEnter(theInstance: ModelData_Instance): Promise<boolean> | boolean;
    /**
     * Leaves an instance.
     *
     * Always invokes `visitInstanceLeave` of the `reference` visitor.
     */
    override visitInstanceLeave(theInstance: ModelData_Instance): Promise<void> | void;
    /**
     * Enters an assembly.
     *
     * Always invokes `visitAssemblyEnter` for the `reference` visitor, but returns `true` only if the assembly is
     * being visited for the first time and `reference` visitor returned `true`.
     * Thus children of already visited assembly won't be visited again.
     */
    override visitAssemblyEnter(theAssembly: ModelData_Assembly): Promise<boolean> | boolean;
    /**
     * Leaves an assembly.
     *
     * Always invokes `visitAssemblyLeave` of the `reference` visitor.
     */
    override visitAssemblyLeave(theAssembly: ModelData_Assembly): Promise<void> | void;
}
/**
 * Defines a visitor of the scene graph elements.
 * This visitor corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___model_1_1_void_element_visitor.html ModelData_Model::VoidElementVisitor} class.
 *
 * The Visitor follows a hierarchical visitor pattern (see
 * http://c2.com/cgi/wiki?HierarchicalVisitorPattern) what enables to track entering
 * and leaving composite entities - assemblies and instances.
 *
 * Methods visitInstanceEnter() and visitAssemblyEnter() returns Promise resolved to `true` if the child elements
 * of the assembly or a referred element of the instance should be visited.
 * If the method returns Promise resolved to `false` they will not be visited.
 * The methods `visitInstanceLeave()` and `visitAssemblyEnter()` will always be called.
 */
abstract class ModelData_SceneGraphElementVisitor {
    abstract visitPart(thePart: ModelData_Part): Promise<void> | void;
    abstract visitInstanceEnter(theInstance: ModelData_Instance): Promise<boolean> | boolean;
    abstract visitInstanceLeave(theInstance: ModelData_Instance): Promise<void> | void;
    abstract visitAssemblyEnter(theAssembly: ModelData_Assembly): Promise<boolean> | boolean;
    abstract visitAssemblyLeave(theAssembly: ModelData_Assembly): Promise<void> | void;
}
/**
 * Element visitor with empty implementation.
 *
 * VoidElementVisitor can be used as a base class for subclasses that only needs to redefine a few methods.
 *
 * This visitor corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___model_1_1_void_element_visitor.html ModelData_Model::VoidElementVisitor } class.
 */
class ModelData_SceneGraphElementVoidVisitor extends ModelData_SceneGraphElementVisitor {
    override visitPart(thePart: ModelData_Part): Promise<void> | void;
    override visitInstanceEnter(theInstance: ModelData_Instance): Promise<boolean> | boolean;
    override visitInstanceLeave(theInstance: ModelData_Instance): Promise<void> | void;
    override visitAssemblyEnter(theAssembly: ModelData_Assembly): Promise<boolean> | boolean;
    override visitAssemblyLeave(theAssembly: ModelData_Assembly): Promise<void> | void;
}
/**
 * Base class of topological shapes.
 *
 * Topological shapes define boundaries of the geometrical entities (curves and surfaces) in
 * {@link ModelData_BRepRepresentation B-Rep representation}.
 *
 * Some topological entities refer to geometrical entities (e.g. edge refers to curve and
 * face refers to surface), whereas some only refer to child topological entities (e.g. wire refers.
 * to edges it consists of or shell refers to faces it consists of).
 *
 * Children subshapes can be retrieved using {@link ModelData_ShapeIterator Iterator}.
 */
abstract class ModelData_Shape implements Iterable<ModelData_Shape> {
    /**
     * Shape type.
     */
    get type(): ModelData_ShapeType;
    [Symbol.iterator](): Iterator<ModelData_Shape>;
}
/**
 * Iterates over subshapes in a shape.
 *
 * Iterator supports two usage scenario:
 * * iteration over direct children;
 * * iteration over subshapes of a specified type.
 *
 * To retrieve direct children of a shape, `ModelData_ShapeIterator` should be used as follows:
 *
 * <span class="doxygen-js"></span>
 * ```js
 * const aWire = ...;
 * for (val aChild of new ModelData_ShapeIterator (aWire)) {
 *   const aType = aChild.type;
 * }
 * ```
 *
 * To retrieve children of a particular type, `ModelData_ShapeIterator` should be used by specifying
 * a type of interest as follows:
 *
 * <span class="doxygen-js"></span>
 * ```js
 * const aSolid = ...;
 * for (let aChild of new ModelData_ShapeIterator (aSolid, ModelData_ShapeType.Edge)) {
 *   // "aChild" has ModelData_Edge type
 *   // For typescript cast to ModelData_Edge:
 *   // const anEdge = aChild as ModelData_Edge;
 * }
 * ```
 *
 * When using the latter approach exploration is done traversing the graph of subshapes in a depth-first manner.
 * Each subshape will be found as many times as it is registered in the parent subshape.
 * For instance, a seam-edge will be encountered twice, with forward and reversed orientations.
 *
 * The order of returned subshapes is deterministic and corresponds to the order
 * in which the subshapes were added during construction.
 *
 * This class corresponds to the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___shape_1_1_iterator.html ModelData_Shape::Iterator} class
 * and implements {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Iteration_protocols JS iteration protocols}.
 */
class ModelData_ShapeIterator implements Iterable<ModelData_Shape> {
    /**
     * Creates an instance of ModelData_ShapeIterator.
     */
    constructor(theShape: ModelData_Shape, theType?: ModelData_ShapeType);
    [Symbol.iterator](): Iterator<ModelData_Shape>;
}
/**
 * Defines shape type.
 */
enum ModelData_ShapeType {
    BodyList = 0,
    Body = 1,
    Solid = 2,
    Shell = 3,
    Face = 4,
    Wire = 5,
    Edge = 6,
    Vertex = 7,
    Undefined = 8,
}
/**
 * Defines a connected set of faces.
 */
class ModelData_Shell extends ModelData_Shape {
    /**
     * Creates an instance of ModelData_Shell.
     */
    constructor(theFace?: ModelData_Face);
    /**
     * Adds a face to the shell.
     */
    append(theFace: ModelData_Face): boolean;
    [Symbol.iterator](): Iterator<ModelData_Face>;
}
/**
 * Defines a topological solid.
 *
 * As a rule, a solid has a single external shell.
 * Solids with voids (i.e. multiple shells) are also supported.
 */
class ModelData_Solid extends ModelData_Shape {
    /**
     * Creates an instance of ModelData_Solid.
     */
    constructor(theShell?: ModelData_Shell);
    /**
     * Adds a shell to the solid.
     */
    append(theShell: ModelData_Shell): boolean;
    [Symbol.iterator](): Iterator<ModelData_Shell>;
}
/**
 * Base class for geometrical surfaces.
 *
 * Surfaces in 3D space are used to represent boundaries of a body (e.g. solid or sheet body).
 * Each {@link ModelData_Face "face"} must refer to a surface.
 *
 * > __Note__: This class is defined for SDK API compatibility only.
 * Unlike {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___surface.html C++ SDK Surface}
 * it contains only type of the surface.
 */
abstract class ModelData_Surface {
    /**
     * Surface type.
     */
    get type(): ModelData_SurfaceType;
}
/**
 * Defines surface type.
 */
enum ModelData_SurfaceType {
    Plane = 0,
    Cylinder = 1,
    Cone = 2,
    Sphere = 3,
    Torus = 4,
    LinearExtrusion = 5,
    Revolution = 6,
    Bezier = 7,
    BSpline = 8,
    Offset = 9,
    Trimmed = 10,
    Undefined = 100,
    UserDefined = 101,
}
/**
 * Defines text justification type.
 */
enum ModelData_TextJustificationType {
    TopLeft = 0,
    TopCenter = 1,
    TopRight = 2,
    MiddleLeft = 3,
    MiddleCenter = 4,
    MiddleRight = 5,
    BottomLeft = 6,
    BottomCenter = 7,
    BottomRight = 8,
}
class ModelData_TextProperties extends ModelData_BaseObject {
    /**
     * The size of the font in points.
     * Default is `16`.
     */
    fontSize: number;
    /**
     * The text rotation in degrees.
     */
    rotation: number;
    /**
     * The text justification. Default is {@link ModelData_TextJustificationType TopLeft}
     */
    justification: ModelData_TextJustificationType;
}
/**
 * Provides an interface to the 2D texture.
 */
abstract class ModelData_Texture extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_Texture.
     */
    constructor(theTextureType?: ModelData_TextureType, theTextureParameters?: ModelData_TextureParameters | null);
    /**
     * Texture type. Default is {@link ModelData_TextureType.ModelData_TT_Unknown ModelData_TT_Unknown}.
     */
    type: ModelData_TextureType;
    /**
     * Texture parameters.
     */
    parameters: ModelData_TextureParameters;
    /**
     * Accepts a texture visitor.
     */
    abstract accept(theVisitor: ModelData_TextureVisitor): Promise<void>;
    /**
     * Equality operator.
     * Returns `true` if type and all parameters of two textures are equal.
     */
    isEqual(theTexture: ModelData_Texture | null | undefined): boolean;
}
/**
 * Texture blending mode.
 */
enum ModelData_TextureBlendMode {
    ModelData_TBM_Decal = 0,
    ModelData_TBM_Modulate = 1,
    ModelData_TBM_Replace = 2,
}
/**
 * Texture magnification filter.
 */
enum ModelData_TextureMagnificationFilter {
    ModelData_TMagF_Nearest = 0,
    ModelData_TMagF_Linear = 1,
}
/**
 * Texture mapping mode.
 */
enum ModelData_TextureMappingMode {
    ModelData_TMM_UVMapping = 0,
    ModelData_TMM_SphereMapping = 1,
}
/**
 * Texture minification filter.
 */
enum ModelData_TextureMinificationFilter {
    ModelData_TMinF_Nearest = 0,
    ModelData_TMinF_NearestMipMapNearest = 1,
    ModelData_TMinF_NearestMipMapLinear = 2,
    ModelData_TMinF_Linear = 3,
    ModelData_TMinF_LinearMipMapNearest = 4,
    ModelData_TMinF_LinearMipMapLinear = 5,
}
/**
 * Provides an interface to set of textures.
 */
class ModelData_TextureParameters extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_Texture.
     */
    constructor();
    /**
     * Indicates to generate mipmaps (if possible) for a texture or not.
     * Default value is `true`.
     */
    generateMipmaps: boolean;
    /**
     * A magnification filter which indicates how the texture is sampled when a texel covers more than one pixel.
     * Should be one of {@link ModelData_TextureMagnificationFilter ModelData_TextureMagnificationFilter}.
     * Default value is ModelData_TextureMagnificationFilter.ModelData_TMagF_Linear,
     * which takes the four closest texels and bilinearly interpolates among them.
     */
    magnificationFilter: ModelData_TextureMagnificationFilter;
    /**
     * A minification filter which indicates how the texture is sampled when a texel covers less than one pixel.
     * Should be one of {@link ModelData_TextureMinificationFilter ModelData_TextureMinificationFilter}.
     * Default value is ModelData_TextureMinificationFilter.ModelData_TMinF_LinearMipMapLinear,
     * which uses mipmapping and a trilinear filter.
     */
    minificationFilter: ModelData_TextureMinificationFilter;
    /**
     * A wrap mode which defines how the texture is wrapped horizontally and corresponds to U in UV mapping.
     * Should be one of {@link ModelData_TextureWrapMode ModelData_TextureWrapMode}.
     * Default value is ModelData_TextureWrapMode.ModelData_TWM_Repeat.
     */
    wrapModeU: ModelData_TextureWrapMode;
    /**
     * A wrap mode which defines how the texture is wrapped horizontally and corresponds to V in UV mapping.
     * Should be one of {@link ModelData_TextureWrapMode ModelData_TextureWrapMode}.
     * Default value is ModelData_TextureWrapMode.ModelData_TWM_Repeat.
     */
    wrapModeV: ModelData_TextureWrapMode;
    /**
     * A blending mode describes how to mix texture colors. Should be one of {@link ModelData_TextureBlendMode ModelData_TextureBlendMode}.
     * Default value is ModelData_TextureBlendMode.ModelData_TBM_Decal.
     */
    blendMode: ModelData_TextureBlendMode;
    /**
     * A mapping mode describes how to calculate textures coords. Should be one of {@link ModelData_TextureMappingMode ModelData_TextureMappingMode}.
     * Default value is ModelData_TextureMappingMode.ModelData_TMM_UVMapping.
     */
    mappingMode: ModelData_TextureMappingMode;
    /**
     * A rotation is describe angle to rotate texture coords to.
     * Default value is `0`
     */
    rotation: number;
    /**
     * A scale applied to the texture width.
     * Default value is `1`
     */
    scaleU: number;
    /**
     * A scale applied to the texture height.
     * Default value is `1`
     */
    scaleV: number;
    /**
     * A translation which indicates how much a single repetition of the texture is offset from the beginning in U direction.
     * Default value is `0`
     */
    translationU: number;
    /**
     * A translation which indicates how much a single repetition of the texture is offset from the beginning in V direction.
     * Default value is `0`
     */
    translationV: number;
    /**
     * Equality operator.
     * Returns `true` if all values of two texture parameters are equal.
     */
    isEqual(theOther: ModelData_TextureParameters | null | undefined): boolean;
}
/**
 * Provides an interface to set of textures.
 */
class ModelData_TextureSet extends ModelData_BaseObject {
    /**
     * Creates an instance of ModelData_TextureSet.
     */
    constructor();
    /**
     * Number of textures in texture set.
     */
    get numberOfTextures(): number;
    /**
     * Returns `true` if no textures in texture set.
     */
    isEmpty(): boolean;
    /**
     * Returns iterable to added components.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const texture of textureSet.textures()) {
     *   // Access to the texture
     * }
     * ```
     *
     * This method performs the same function as the C++ {@link https://docs.cadexchanger.com/sdk/classcadex_1_1_model_data___texture_set_1_1_texture_iterator.html ModelData_TextureSet::TextureIterator} class.
     */
    textures(): Iterable<ModelData_Texture>;
    /**
     * Adds texture to texture set.
     *
     * Returns `true` if insertion was successfully.
     */
    add(theTexture: ModelData_Texture): boolean;
    /**
     * Removes form texture set if exist.
     * Returns `true` if deleting was successfully.
     */
    remove(theTexture: ModelData_Texture): boolean;
    /**
     * Checks texture to be in texture set.
     */
    contains(theTexture: ModelData_Texture): boolean;
    /**
     * Accepts a texture visitor.
     */
    accept(theVisitor: ModelData_TextureVisitor): Promise<void>;
    combineWith(theOther: ModelData_TextureSet): void;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_TextureSet;
    /**
     * Copies data from another texture set.
     */
    copy(theTextureSet: ModelData_TextureSet): this;
    /**
     * Equality operator.
     * Returns `true` if all textures of two sets are equal.
     */
    isEqual(theTextureSet: ModelData_TextureSet | null | undefined): boolean;
}
/**
 * Defines types of texture.
 */
enum ModelData_TextureType {
    ModelData_TT_Unknown = 0,
    ModelData_TT_DiffuseMap = 1,
    ModelData_TT_SpecularMap = 2,
    ModelData_TT_EmissiveMap = 3,
    ModelData_TT_BumpMap = 4,
    ModelData_TT_NormalMap = 5,
    ModelData_TT_DisplacementMap = 6,
    ModelData_TT_SphereEnvironmentMap = 7,
}
/**
 * Defines a visitor of the textures.
 */
abstract class ModelData_TextureVisitor {
    abstract visitFileTexture(theFileTexture: ModelData_FileTexture): Promise<void> | void;
    abstract visitPixMapTexture(thePixMapTexture: ModelData_PixMapTexture): Promise<void> | void;
}
/**
 * Texture wrapping mode.
 */
enum ModelData_TextureWrapMode {
    ModelData_TWM_ClampToEdge = 0,
    ModelData_TWM_Repeat = 1,
    ModelData_TWM_MirroredRepeat = 2,
}
/**
 * Defines a transformation matrix.
 */
class ModelData_Transformation {
    /**
     * Creates an instance of ModelData_Transformation.
     */
    constructor(theMatrix?: Array<number>);
    /**
     * Elements of transformation 4x4 matrix.
     */
    get elements(): ReadonlyArray<number>;
    /**
     * Returns `true` if the matrix is identity.
     */
    get isIdentity(): boolean;
    /**
     * Scale factor. Default is `1`.
     */
    get scaleFactor(): number;
    set scaleFactor(theScaleFactor: number);
    /**
     * Resets this transformation to be identity.
     */
    makeIdentity(): this;
    /**
     * Retrieves individual matrix elements.
     */
    data(theRow: number, theCol: number): number;
    /**
     * Sets rotation part of transformation.
     */
    setRotationPart(theV00: number, theV01: number, theV02: number, theV10: number, theV11: number, theV12: number, theV20: number, theV21: number, theV22: number): this;
    /**
     * Sets translation part of transformation.
     */
    setTranslationPart(theX: number, theY: number, theZ: number): this;
    /**
     * Multiplies this matrix with another one.
     */
    multiply(theTrsf: ModelData_Transformation): this;
    /**
     * Compares if two transformation are equal within a tolerance.
     */
    isEqual(theTrsf: ModelData_Transformation): boolean;
    /**
     * Creates copy of the current object.
     */
    clone(): ModelData_Transformation;
    /**
     * Copies values from another transformation.
     */
    copy(theTrsf: ModelData_Transformation): this;
    /**
     * Returns string representation of the transformation.
     */
    toString(): string;
    /**
     * Returns JSON representation of the transformation.
     */
    toJSON(): Array<number>;
}
/**
 * Defines a 3D vector.
 */
class ModelData_Vector {
    constructor(x?: number, y?: number, z?: number);
    /**
     * A tolerance to compare coincidence of two vectors.
     */
    static get coincidenceTolerance(): number;
    /**
     * A tolerance to compare if two vectors are collinear.
     */
    static get angularTolerance(): number;
    /**
     * X coord.
     */
    get x(): number;
    set x(x: number);
    /**
     * Y coord.
     */
    get y(): number;
    set y(y: number);
    /**
     * Z coord.
     */
    get z(): number;
    set z(z: number);
    /**
     * Creates vector from object implementing {@link ModelData_XY} interface.
     */
    static fromXYZ(theXYZ: ModelData_XYZ): ModelData_Vector;
    /**
     * Sets coordinates.
     */
    setCoord(x: number, y: number, z: number): this;
    /**
     * Returns a square modulus.
     */
    squareModulus(): number;
    /**
     * Returns a modulus.
     */
    modulus(): number;
    /**
     * Set modulus of the vector.
     */
    setModulus(theModulus: number): this;
    /**
     * Normalizes the vector.
     */
    normalize(): this;
    /**
     * Returns a vector which is a normalized vector of this.
     */
    normalized(): ModelData_Vector;
    /**
     * Reverses the vector.
     */
    reverse(): this;
    /**
     * Returns a vector which is a reversed vector of this.
     */
    reversed(): ModelData_Vector;
    add(theOther: ModelData_XYZ): this;
    addScaled(theOther: ModelData_XYZ, theScale: number): this;
    addVectors(theOther1: ModelData_XYZ, theOther2: ModelData_XYZ): this;
    subtract(theOther: ModelData_XYZ): this;
    subtractVectors(theOther1: ModelData_XYZ, theOther2: ModelData_XYZ): this;
    multiply(theOther: ModelData_XYZ): this;
    multiplyScalar(theScalar: number): this;
    divide(theOther: ModelData_XYZ): this;
    divideScalar(theScalar: number): this;
    dot(theOther: ModelData_XYZ): number;
    cross(theOther: ModelData_XYZ): this;
    crossVectors(a: ModelData_XYZ, b: ModelData_XYZ): this;
    /**
     * Returns the angle between this vector and the other in radians.
     */
    angle(theOther: ModelData_Vector): number;
    /**
     * Transforms the vector with a transformation matrix.
     */
    transform(theTrsf: ModelData_Transformation): this;
    /**
     * Returns a vector which is transformed with a transformation matrix.
     */
    transformed(theTrsf: ModelData_Transformation): ModelData_Vector;
    /**
     * Compares if two vectors are parallel within a distance tolerance.
     */
    isParallel(theOther: ModelData_Vector, theTolerance: number): boolean;
    /**
     * Compares if two vectors are equal within a distance tolerance.
     */
    isEqual(theXYZ: ModelData_XYZ, theTolerance?: number): boolean;
    clone(): ModelData_Vector;
    copy(theXYZ: ModelData_XYZ): this;
    /**
     * Returns string representation of the vector.
     */
    toString(): string;
    /**
     * Returns JSON representation of the vector.
     */
    toJSON(): ModelData_XYZ;
}
/**
 * Defines a 2D vector.
 */
class ModelData_Vector2d {
    constructor(x?: number, y?: number);
    static get coincidenceTolerance(): number;
    /**
     * A tolerance to compare if two vectors are collinear.
     */
    static get angularTolerance(): number;
    /**
     * X coord.
     */
    get x(): number;
    set x(x: number);
    /**
     * Y coord.
     */
    get y(): number;
    set y(y: number);
    /**
     * Creates vector from object implementing {@link ModelData_XY} interface.
     */
    static fromXY(theXY: ModelData_XY): ModelData_Vector2d;
    /**
     * Sets coordinates.
     */
    setCoord(x: number, y: number): this;
    /**
     * Returns a square modulus.
     */
    squareModulus(): number;
    /**
     * Returns a modulus.
     */
    modulus(): number;
    /**
     * Set modulus of the vector.
     */
    setModulus(theModulus: number): this;
    /**
     * Normalizes the vector.
     */
    normalize(): this;
    /**
     * Returns a vector which is a normalized vector of this.
     */
    normalized(): ModelData_Vector2d;
    /**
     * Reverses the vector.
     */
    reverse(): this;
    /**
     * Returns a vector which is a reversed vector of this.
     */
    reversed(): ModelData_Vector2d;
    add(theOther: ModelData_XY): this;
    addScaled(theOther: ModelData_XY, theScale: number): this;
    addVectors(theOther1: ModelData_XY, theOther2: ModelData_XY): this;
    subtract(theOther: ModelData_XY): this;
    subtractVectors(theOther1: ModelData_XY, theOther2: ModelData_XY): this;
    multiply(theOther: ModelData_XY): this;
    multiplyScalar(theScalar: number): this;
    divide(theOther: ModelData_XY): this;
    divideScalar(theScalar: number): this;
    dot(theOther: ModelData_XY): number;
    cross(theOther: ModelData_XY): number;
    /**
     * Computes the angle in radians with respect to the positive x-axis.
     */
    angle(): number;
    /**
     * Compares if two direction are parallel within a distance tolerance.
     */
    isParallel(theOther: ModelData_Vector2d, theTolerance: number): boolean;
    /**
     * Compares if two vectors are equal within a distance tolerance.
     */
    isEqual(theXY: ModelData_XY, theTolerance?: number): boolean;
    clone(): ModelData_Vector2d;
    copy(theXY: ModelData_XY): this;
    /**
     * Returns string representation of the vector.
     */
    toString(): string;
    /**
     * Returns JSON representation of the vector.
     */
    toJSON(): ModelData_XY;
}
/**
 * Defines topological vertex.
 */
abstract class ModelData_Vertex extends ModelData_Shape {
    /**
     * Point assigned to this vertex.
     */
    get point(): ModelData_Point;
}
/**
 * Defines a connected set of edges.
 *
 * Wire can be used to define a {@link ModelData_Face face} boundary
 * or just a set of {@link ModelData_Edge edges}.
 */
class ModelData_Wire extends ModelData_Shape {
    /**
     * Creates an instance of ModelData_Wire.
     */
    constructor(theEdge?: ModelData_Edge);
    /**
     * Adds an edge to the wire.
     */
    append(theEdge: ModelData_Edge): boolean;
    [Symbol.iterator](): Iterator<ModelData_Edge>;
}
/**
 * 2D Point interface.
 */
interface ModelData_XY {
    x: number;
    y: number;
}
/**
 * 3D Point interface.
 */
interface ModelData_XYZ {
    x: number;
    y: number;
    z: number;
}
/**
 * The light which globally illuminates all objects in the scene equally.
 */
class ModelPrs_AmbientLight extends ModelPrs_Light {
    /**
     * Creates an instance of ModelPrs_AmbientLight.
     */
    constructor(theColor?: ModelData_ColorObject, theIntensity?: number);
}
/**
 * Stores result of the angular measurement.
 */
abstract class ModelPrs_AngularMeasurement extends ModelPrs_Measurement {
    /**
     * Angle unit.
     */
    get angleUnit(): Base_AngleUnit;
    set angleUnit(theAngleUnit: Base_AngleUnit);
    override toString(): string;
}
/**
 * {@link ModelPrs_Annotation} configuration interface.
 */
interface ModelPrs_AnnotationConfig {
    /**
     * World-space position of the annotation.
     */
    position: ModelData_Point;
    /**
     * Optional view3d object associated with the Annotation.
     */
    sceneNode: ModelPrs_SceneNode | undefined;
    /**
     * DOM element to render the marker.
     */
    markerElement: HTMLElement;
    /**
     * DOM element to render the label.
     */
    labelElement: HTMLElement | undefined;
    /**
     * Initial value for {@link ModelPrs_Annotation#markerShown} property
     */
    markerShown: boolean | undefined;
    /**
     * Initial value for {@link ModelPrs_Annotation#labelShown} property
     */
    labelShown: boolean | undefined;
}
/**
 * A {@link ModelPrs_Marker} with an HTML marker and label attached to it.
 */
class ModelPrs_Annotation extends ModelPrs_Marker {
    constructor(theConfig: ModelPrs_AnnotationConfig);
    /**
     * Scene object associated with the Annotation.
     */
    get sceneNode(): ModelPrs_SceneNode | undefined;
    /**
     * Marker element associated with Annotation.
     */
    get markerElement(): HTMLElement;
    /**
     * Optional label element associated with Annotation.
     */
    get labelElement(): HTMLElement | undefined;
    /**
     * Indicates whether the marker element is shown or not.
     */
    get isMarkerShown(): boolean;
    set isMarkerShown(theShown: boolean);
    /**
     * Indicates whether the label element is shown or not.
     */
    get isLabelShown(): boolean;
    set isLabelShown(theShown: boolean);
    /**
     * Updates marker and label position according to the camera parameters.
     */
    override update(theCamera: ModelPrs_Camera): void;
}
/**
 * Defines a camera paddings in units of the camera's normalized device coordinate (NDC) space (should be in range [0, 1]).
 */
type ModelPrs_CameraPaddings = {
    left?: number;
    right?: number;
    top?: number;
    bottom?: number;
}
/**
 * Defines a camera frustum.
 */
type ModelData_CameraFrustumType = {
    left: ModelData_Plane;
    right: ModelData_Plane;
    top: ModelData_Plane;
    bottom: ModelData_Plane;
    near: ModelData_Plane;
    far: ModelData_Plane;
}
type ModelPrs_CameraChangedEvent<CameraType extends ModelPrs_Camera = ModelPrs_Camera> = ModelPrs_Event<"changed",CameraType> & {
}
type ModelPrs_CameraEventMap<CameraType extends ModelPrs_Camera> = {
    changed: ModelPrs_CameraChangedEvent<CameraType>;
}
class ModelPrs_Camera extends ModelPrs_EventDispatcher {
    /**
     * Initializes camera with the default properties.
     */
    constructor();
    /**
     * Camera projection type. Default is {@link ModelPrs_CameraProjectionType Isometric}
     */
    get projectionType(): ModelPrs_CameraProjectionType;
    set projectionType(theType: ModelPrs_CameraProjectionType);
    /**
     * Camera scale.
     * For orthographic projection the scale factor corresponds to parallel scale of view mapping.
     * For perspective camera scale factor relates to distance between position and target.
     */
    get scale(): number;
    /**
     * Camera frustum vertical field of view, from bottom to top of view, in degrees. Default is 45.
     * Relevant for {@link ModelPrs_CameraProjectionType Perspective} projection type only.
     */
    get fov(): number;
    set fov(theFov: number);
    /**
     * The near Z-clipping plane position.
     */
    get near(): number;
    /**
     * The far Z-clipping plane position.
     */
    get far(): number;
    /**
     * Position of the camera. Default is (0, 0, 0).
     */
    get position(): ModelData_Point;
    /**
     * To preserve correct orthographic/perspective projection type switching on set the `scale` is automatically recalculated.
     */
    set position(thePosition: ModelData_Point);
    /**
     * Target of the camera. Default is (0, 1, 0).
     */
    get target(): ModelData_Point;
    set target(theTarget: ModelData_Point);
    /**
     * Up vector of the camera. Default is (0, 0, 1).
     */
    get up(): ModelData_Direction;
    set up(theUp: ModelData_Direction);
    /**
     * Sets position, target and up direction of the camera.
     * To preserve correct orthographic/perspective projection type switching the camera `scale` is automatically recalculated.
     */
    set(thePosition: ModelData_Point, theTarget: ModelData_Point, theUp: ModelData_Direction, theNear?: number, theFar?: number): void;
    /**
     * Calculates camera look direction.
     */
    getDirection(theTarget?: ModelData_Direction): ModelData_Direction;
    /**
     * Changes the near and far Z-clipping plane positions.
     * For orthographic projection, theNear, theFar can be negative or positive.
     * For perspective projection, only positive values are allowed.
     *
     * Exception is raised if non-positive values are specified for perspective projection
     * or `theNear` >= `theFar`.
     */
    setZRange(theNear: number, theFar: number): void;
    /**
     * Changes position to `thePoint` and updates camera target
     * to preserve direction and distance between position and target.
     */
    moveTo(thePoint: ModelData_XYZ): void;
    /**
     * Translates the center of the view along `x` and `y` axes of camera space
     * preserving camera direction and distance between `position` and `target`.
     */
    pan(dx: number, dy: number): void;
    /**
     * Rotates the center of the view about the coordinate system of reference of the screen
     * for which the origin is gravity point, with a relative angular value in radians.
     */
    rotate(ax: number, ay: number, az: number, theGravity: ModelData_XYZ): void;
    /**
     * Scales the camera according to a zoom factor about the coordinate system of reference of the screen
     * for which the origin is gravity point.
     * If the gravity point is omitted, the camera position is used (which corresponds to the zooming about center of the screen).
     */
    zoom(theFactor: number, theGravity?: ModelData_XYZ): void;
    /**
     * Estimate Z-min and Z-max planes of projection volume to match the displayed objects bounding box.
     * For orthogonal projection the view volume contains the bounding box completely.
     * For perspective projection the view volume is adjusted such that it contains the bounding box part, located in front of the camera.
     */
    zFitAll(theBox: ModelData_Box): void;
    /**
     * Orthogonalize up direction vector.
     */
    orthogonalizeUp(): void;
    /**
     * Calculates orthogonalized up direction vector.
     */
    orthogonalizedUp(): ModelData_Direction;
    /**
     * Fit camera to see whole box.
     */
    fitBox(theBox: ModelData_Box, thePaddings?: ModelPrs_CameraPaddings): void;
    /**
     * Translates the point from world space into the camera's normalized device coordinate (NDC) space.
     */
    translatePointFromWorldSpaceToNDCSpace(thePoint: ModelData_Point, theTarget?: ModelData_Point): ModelData_Point;
    /**
     * Translates the point from the camera's normalized device coordinate (NDC) space into world space.
     */
    translatePointFromNDCSpaceToWorldSpace(thePoint: ModelData_Point, theTarget?: ModelData_Point): ModelData_Point;
    /**
     * Calculate frustum planes for the camera projection volume.
     * Frustum is a convex volume determined by six planes directing inwards.
     */
    getFrustum(): ModelData_CameraFrustumType;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_CameraEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_CameraEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_Camera#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_CameraEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_CameraEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
class ModelPrs_CameraManipulationHandler extends ModelPrs_InputHandler {
    /**
     * Creates an instance of ModelPrs_CameraManipulationHandler.
     */
    constructor();
    multiTouchHandler: ModelPrs_MultiPointInputHandler;
    rotateHandler: ModelPrs_SinglePointInputHandler;
    zoomHandler: ModelPrs_SinglePointInputHandler;
    panHandler: ModelPrs_SinglePointInputHandler;
    /**
     * Gravity point which the camera rotates around.
     * If `null`, the center of the viewport's scene bounding box is used.
     */
    get gravityPoint(): ModelData_Point | null;
    set gravityPoint(theGravityPoint: ModelData_Point | null);
    override pointerDown(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUp(theEvent: ModelPrs_PointerInputEvent): void;
    override pointerCancel(theEvent: ModelPrs_PointerInputEvent): void;
    override wheel(theEvent: ModelPrs_WheelInputEvent): boolean;
    override keyDown(theEvent: ModelPrs_KeyboardInputEvent): boolean;
    override keyUp(theEvent: ModelPrs_KeyboardInputEvent): void;
}
class ModelPrs_CameraMultiTouchHandler extends ModelPrs_PinchInputHandler {
    /**
     * Creates an instance of ModelPrs_CameraMultiTouchHandler.
     */
    constructor();
    /**
     * Default value is `1`.
     */
    panSpeed: number;
    /**
     * Default value is `1`.
     */
    zoomSpeed: number;
    /**
     * Default value is `1`.
     */
    zAxisRotationSpeed: number;
}
class ModelPrs_CameraPanHandler extends ModelPrs_SinglePointInputHandler {
    /**
     * Creates an instance of ModelPrs_CameraPanHandler.
     */
    constructor();
    /**
     * Default value is `1`.
     */
    panSpeed: number;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
}
/**
 * Defines a direction where the camera is located.
 *
 * Camera views are relative to the local coordinate system of the scene. Its origin is in the center of the scene
 * and its axes are aligned with world coordinate system axes. All camera views are set up so that the camera looks
 * at the origin of local coordinate system, i.e. looks at the scene. Various preset views then describe the possible
 * positions of camera around the scene. "Positive X" means that camera is located at the positive end of X axis.
 * "Negative Y" means that camera is located at the negative end of Y axis. There are also combined presets that place
 * the camera in the middle between 2 or all 3 axes.
 */
enum ModelPrs_CameraPositionType {
    Xpos = 0,
    Ypos = 1,
    Zpos = 2,
    Xneg = 3,
    Yneg = 4,
    Zneg = 5,
    XposYpos = 6,
    XposZpos = 7,
    YposZpos = 8,
    XnegYneg = 9,
    XnegYpos = 10,
    XnegZneg = 11,
    XnegZpos = 12,
    YnegZneg = 13,
    YnegZpos = 14,
    XposYneg = 15,
    XposZneg = 16,
    YposZneg = 17,
    XposYposZpos = 18,
    XposYnegZpos = 19,
    XposYposZneg = 20,
    XnegYposZpos = 21,
    XposYnegZneg = 22,
    XnegYposZneg = 23,
    XnegYnegZpos = 24,
    XnegYnegZneg = 25,
    /**
     * Alias to Yneg
     */
    Front = 4,
    /**
     * Alias to Ypos
     */
    Back = 1,
    /**
     * Alias to Zpos
     */
    Top = 2,
    /**
     * Alias to Zneg
     */
    Bottom = 5,
    /**
     * Alias to Xneg
     */
    Left = 3,
    /**
     * Alias to Xpos
     */
    Right = 0,
    /**
     * Alias to XposYnegZpos
     */
    Default = 19,
}
/**
 * ViewPort camera projection mode.
 */
enum ModelPrs_CameraProjectionType {
    Isometric = 0,
    Perspective = 1,
}
class ModelPrs_CameraRotateHandler extends ModelPrs_SinglePointInputHandler {
    /**
     * Creates an instance of ModelPrs_CameraRotateHandler.
     */
    constructor(theGravityPoint?: ModelData_Point | null);
    /**
     * Default value is `1.5`.
     */
    rotateSpeed: number;
    /**
     * The target the camera rotates around.
     * If `null`, the center of the viewport's scene bounding box is used.
     */
    gravityPoint: ModelData_Point | null;
    override pointerDown(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override keyDown(theEvent: ModelPrs_KeyboardInputEvent): boolean;
}
/**
 * Defines the type of the gravity point using by {@link ModelPrs_CameraZoomHandler}.
 */
enum ModelPrs_CameraZoomHandlerGravityPoint {
    ScreenCenter = 0,
    PointerPosition = 1,
}
class ModelPrs_CameraZoomHandler extends ModelPrs_SinglePointInputHandler {
    /**
     * Creates an instance of ModelPrs_CameraZoomHandler.
     */
    constructor();
    /**
     * Default value is {@link ModelPrs_CameraZoomHandlerGravityPoint.ScreenCenter `ScreenCenter`};
     */
    wheelZoomGravityPoint: ModelPrs_CameraZoomHandlerGravityPoint;
    /**
     * Default value is `false`;
     */
    wheelZoomReverseDirection: boolean;
    /**
     * Default value is `1`.
     */
    zoomSpeed: number;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override wheel(theEvent: ModelPrs_WheelInputEvent): boolean;
}
type ModelPrs_ClipPlaneEventMap<ClipPlaneType extends ModelPrs_ClipPlane> = ModelPrs_FieldChangedEventMap<ClipPlaneType> & {
    changed: ModelPrs_Event<"changed", ClipPlaneType>;
}
/**
 * Class represents "clipping plane" in 3D space.
 */
class ModelPrs_ClipPlane extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_ClipPlane.
     */
    constructor(thePlane: ModelData_Plane);
    /**
     * A plane located in 3D space with an axis placement three axis.
     */
    get plane(): ModelData_Plane;
    set plane(thePlane: ModelData_Plane);
    /**
     * Indicates to show transparent plane or not. Default is `true`.
     */
    get isShowPlane(): boolean;
    set isShowPlane(theIsShowPlane: boolean);
    /**
     * Indicates to show controls (arrow) or not. Default is `true`.
     */
    get isShowControls(): boolean;
    set isShowControls(theIsShowControls: boolean);
    /**
     * Indicates to generate capping plane (if possible) or not. Default is `true`.
     */
    get isCappingEnabled(): boolean;
    set isCappingEnabled(theIsCappingEnabled: boolean);
    /**
     * Capping appearance.
     */
    get cappingAppearance(): ModelData_Appearance;
    set cappingAppearance(theCappingAppearance: ModelData_Appearance);
    /**
     * Indicates to render hatch for capping plane (if possible) or not.
     */
    get isHatchingEnabled(): boolean;
    /**
     * Indicates to render hatch for capping plane (if possible) or not.
     */
    set isHatchingEnabled(theIsHatchingEnabled: boolean);
    /**
     * Releases the data allocated by this instance.
     * Disposed instance cannot be used anymore.
     * Call this method whenever the instance is no longer used in your app.
     */
    override dispose(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_ClipPlaneEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ClipPlaneEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_ClipPlane#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_ClipPlaneEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ClipPlaneEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
type ModelPrs_ClipPlaneInputHandlerClipPlaneMovedEvent<ClipPlaneInputHandlerType extends ModelPrs_ClipPlaneInputHandler = ModelPrs_ClipPlaneInputHandler> = ModelPrs_Event<"clipPlaneMoved",ClipPlaneInputHandlerType> & {
    clipPlane: ModelPrs_ClipPlane;
}
type ModelPrs_ClipPlaneInputHandlerEventMap<ClipPlaneInputHandlerType extends ModelPrs_ClipPlaneInputHandler = ModelPrs_ClipPlaneInputHandler> = {
    clipPlaneMoved: ModelPrs_ClipPlaneInputHandlerClipPlaneMovedEvent<ClipPlaneInputHandlerType>;
}
class ModelPrs_ClipPlaneInputHandler extends ModelPrs_SinglePointInputHandler {
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_ClipPlaneInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ClipPlaneInputHandlerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_ClipPlaneInputHandler#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_ClipPlaneInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ClipPlaneInputHandlerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    override pointerDown(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUp(theEvent: ModelPrs_PointerInputEvent): void;
}
/**
 * Event emitted when the clip plane is added to the clip planes manager.
 */
type ModelPrs_GlobalClipPlaneAddedEvent<ClipPlanesManagerType extends ModelPrs_ClipPlanesManager = ModelPrs_ClipPlanesManager> = ModelPrs_Event<"globalClipPlaneAdded",ClipPlanesManagerType> & {
    clipPlane: ModelPrs_ClipPlane;
}
/**
 * Event emitted when the clip plane is removed from the clip planes manager.
 */
type ModelPrs_GlobalClipPlaneRemovedEvent<ClipPlanesManagerType extends ModelPrs_ClipPlanesManager = ModelPrs_ClipPlanesManager> = ModelPrs_Event<"globalClipPlaneRemoved",ClipPlanesManagerType> & {
    clipPlane: ModelPrs_ClipPlane;
}
/**
 * Event emitted when all clip planes are removed from the clip planes manager.
 */
type ModelPrs_GlobalClipPlanesRemovedAllEvent<ClipPlanesManagerType extends ModelPrs_ClipPlanesManager = ModelPrs_ClipPlanesManager> = ModelPrs_Event<"globalClipPlanesRemovedAll",ClipPlanesManagerType> & {
    clipPlanes: Array<ModelPrs_ClipPlane>;
}
type ModelPrs_ClipPlanesManagerEventMap<ClipPlanesManagerType extends ModelPrs_ClipPlanesManager> = {
    globalClipPlaneAdded: ModelPrs_GlobalClipPlaneAddedEvent<ClipPlanesManagerType>;
    globalClipPlaneRemoved: ModelPrs_GlobalClipPlaneRemovedEvent<ClipPlanesManagerType>;
    globalClipPlanesRemovedAll: ModelPrs_GlobalClipPlanesRemovedAllEvent<ClipPlanesManagerType>;
    viewportChanged: ModelPrs_Event<"viewportChanged", ClipPlanesManagerType>;
}
/**
 * Manager of global clip planes for the {@link ModelPrs_ViewPort ViewPort}.
 */
class ModelPrs_ClipPlanesManager extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_ClipPlanesManager.
     */
    constructor();
    get viewport(): ModelPrs_ViewPort | null;
    /**
     * The box used for creating clipping plane visual geometry.
     * If `null`, the viewport's scene bounding box is used.
     */
    get bbox(): ModelData_Box | null;
    set bbox(theBBox: ModelData_Box | null);
    /**
     * Number of added global clip planes.
     */
    get numberOfGlobalClipPlanes(): number;
    /**
     * Returns iterable to stored global clip planes.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const clipPlane of clipPlanesManager.globalClipPlanes()) {
     *   // Access to the clip plane
     * }
     * ```
     */
    globalClipPlanes(): Iterable<ModelPrs_ClipPlane>;
    /**
     * Returns `true` if the manager contains the global clip plane. Otherwise returns `false`.
     */
    containsGlobalClipPlane(theClipPlane: ModelPrs_ClipPlane): boolean;
    /**
     * Adds global clip plane to the manager.
     */
    addGlobalClipPlane(theClipPlane: ModelPrs_ClipPlane): void;
    /**
     * Removes global clip plane from the manager.
     */
    removeGlobalClipPlane(theClipPlane: ModelPrs_ClipPlane): void;
    /**
     * Removes all global clip planes from the manager.
     */
    removeAllGlobalClipPlanes(): void;
    /**
     * Releases the data allocated by this instance.
     * Disposed instance cannot be used anymore.
     * Call this method whenever the instance is no longer used in your app.
     */
    override dispose(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_ClipPlanesManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ClipPlanesManagerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_ClipPlanesManager#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_ClipPlanesManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ClipPlanesManagerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
type ModelPrs_ContextMenuHandlerEventMap<ContextMenuHandlerType extends ModelPrs_ContextMenuHandler> = ModelPrs_SinglePointInputHandlerEventMap<ContextMenuHandlerType> & {
    contextMenu: ModelPrs_Event<"contextMenu", ContextMenuHandlerType>;
}
/**
 * Handler that interprets a right click or long touch press as context menu event.
 */
class ModelPrs_ContextMenuHandler extends ModelPrs_TapInputHandler {
    /**
     * Creates an instance of ModelPrs_SelectionHandler.
     */
    constructor();
    /**
     * Handles context menu event fires when the user attempts to open a context menu.
     *
     * By default it just dispatches `contextMenu` {@link ModelPrs_PointerInputEvent event}.
     */
    contextMenu(theEvent: ModelPrs_PointerInputEvent): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_ContextMenuHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ContextMenuHandlerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_ContextMenuHandler#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_ContextMenuHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ContextMenuHandlerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    override tap(theEvent: ModelPrs_PointerInputEvent): void;
    override doubleTap(theEvent: ModelPrs_PointerInputEvent): void;
    override longPress(theEvent: ModelPrs_PointerInputEvent): void;
}
/**
 * A light that gets emitted in a specific direction.
 * This light will behave as though it is infinitely far away
 * and the rays produced from it are all parallel.
 * The common use case for this is to simulate daylight.
 */
class ModelPrs_DirectionalLight extends ModelPrs_Light {
    /**
     * Creates an instance of ModelPrs_DirectionalLight.
     */
    constructor(theDirection: ModelData_Direction, theColor?: ModelData_ColorObject, theIsHeadlight?: boolean);
    /**
     * Direction of the light source.
     */
    get direction(): ModelData_Direction;
    set direction(theDirection: ModelData_Direction);
}
/**
 * Display mode for visualization.
 */
enum ModelPrs_DisplayMode {
    Wireframe = 0,
    Shaded = 1,
    ShadedWithBoundaries = 2,
    Undefined = 100,
}
/**
 * Event dispatcher listener type.
 */
type ModelPrs_EventDispatcherListenerType<EventTypeType extends string, EventDispatcherType extends ModelPrs_EventDispatcher, EventType = ModelPrs_Event<EventTypeType, EventDispatcherType>> = (theEvent: EventType) => any;
/**
 * Base interface for all events.
 */
interface ModelPrs_Event<TypeType extends string, TargetType extends ModelPrs_EventDispatcher> {
    readonly type: TypeType;
    readonly target: TargetType;
    [index: string]: any;
}
/**
 * Implements {@link https://developer.mozilla.org/en-US/docs/Web/API/EventTarget EventTarget} interface.
 * Base class which allow objects receive events and may have listeners for them.
 */
class ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_EventDispatcher.
     */
    constructor();
    /**
     * Releases the data allocated by this instance.
     * Disposed instance cannot be used anymore.
     * Call this method whenever the instance is no longer used in your app.
     */
    dispose(): void;
    /**
     * Use @ignore to declare method with readable signature is (j|t)sdoc files
     */
    addEventListener<EventDispatcherType extends ModelPrs_EventDispatcher>(theType: string, theListener: ModelPrs_EventDispatcherListenerType<string, EventDispatcherType>): void;
    /**
     * Use @ignore to declare method with readable signature is (j|t)sdoc files
     */
    removeEventListener<EventDispatcherType extends ModelPrs_EventDispatcher>(theType: string, theListener: ModelPrs_EventDispatcherListenerType<string, EventDispatcherType>): void;
    /**
     * Dispatches an event, invoking the affected event listeners in the appropriate order.
     */
    dispatchEvent(theEvent: {[index: string]: any, type: string}): void;
}
type ModelPrs_ExploderEventMap<ExploderType extends ModelPrs_Exploder> = ModelPrs_FieldChangedEventMap<ExploderType> & {
    viewportChanged: ModelPrs_Event<"viewportChanged", ExploderType>;
    explodedBBoxChanged: ModelPrs_Event<"explodedBBoxChanged", ExploderType>;
}
/**
 * Provides the ability to get an 'exploded' model view - when model parts separated by distance.
 * Temporarily changes transformation of displayed objects according to the `value` of explosion.
 */
class ModelPrs_Exploder extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_Exploder.
     */
    constructor();
    get viewport(): ModelPrs_ViewPort | null;
    get explodedBBox(): ModelData_Box;
    /**
     * Explosion value. It may be between 0.0 (unexploded) and 1.0 (fully exploded).
     */
    get value(): number;
    set value(theValue: number);
    /**
     * Indicates whether the exploder is currently active.
     */
    get isActive(): boolean;
    set isActive(theIsActive: boolean);
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_ExploderEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ExploderEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_Exploder#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_ExploderEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ExploderEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Provides the ability to generate mesh from text.
 */
class ModelPrs_Font {
    /**
     * Construction from "typeface.js" font has been deprecated. Use {@link ModelPrs_Font.fromTypefaceFont} method instead.
     * @deprecated
     */
    constructor(theFont: Object);
    /**
     * The font's family name.
     */
    get familyName(): string;
    /**
     * The font's sub-family name.
     */
    get subfamilyName(): string;
    /**
     * The size of the font's internal coordinate grid.
     */
    get unitsPerEm(): number;
    /**
     * Distance from baseline of highest ascender. In font units, not pixels.
     */
    get ascender(): number;
    /**
     * Distance from baseline of lowest descender. In font units, not pixels.
     */
    get descender(): number;
    /**
     * The amount of space that should be included between lines.
     */
    get lineGap(): number;
    /**
     * Creates font from {@link https://github.com/gero3/facetype.js typeface.js} font.
     */
    static fromTypefaceFont(theFont: Object): ModelPrs_Font;
    /**
     * Creates font from {@link https://github.com/opentypejs/opentype.js opentype.js} font instance.
     */
    static fromOpentypeFont(theFont: Object): ModelPrs_Font;
    /**
     * Creates font from {@link https://github.com/foliojs/fontkit fontkit} font instance.
     */
    static fromFontkitFont(theFont: Object): ModelPrs_Font;
    /**
     * Generates mesh for text.
     * If `theSplitIntoSubGroups` is `true`, the result mesh will include appearance groups for every symbol.
     * `theJustification` controls position of the text relative to (0, 0) point.
     */
    generateMesh(theText: string, theSize: number, theSplitIntoSubGroups?: boolean, theJustification?: ModelData_TextJustificationType): ModelData_IndexedTriangleSet;
    /**
     * Returns the advance width and height of a text.
     *
     * This is something different than bounding box of generated mesh as for example a
     * suffixed whitespace increases the advanceWidth but not the bounding box
     * or an overhanging letter like a calligraphic 'f' might have a quite larger
     * bounding box than its advance width.
     */
    computeAdvanceWidth(theText: string, theSize: number): number;
}
type ModelPrs_GeometryEventMap<GeometryType extends ModelPrs_Geometry> = ModelPrs_FieldChangedEventMap<GeometryType> & {
}
abstract class ModelPrs_Geometry extends ModelPrs_EventDispatcher {
    /**
     * Geometry type.
     */
    get type(): ModelPrs_GeometryType;
    /**
     * Geometry state.
     */
    get state(): ModelPrs_GeometryState;
    /**
     * Calculates bounding box of the geometry with applied transformation.
     */
    boundingBox(theTransformation?: ModelData_Transformation | null, theTarget?: ModelData_Box | null): ModelData_Box;
    /**
     * Accepts a geometry visitor.
     */
    abstract accept(theVisitor: ModelPrs_GeometryVisitor): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_GeometryEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_GeometryEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_Geometry#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_GeometryEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_GeometryEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Defines geometry state.
 */
enum ModelPrs_GeometryState {
    NotLoaded = 0,
    Loading = 1,
    Completed = 2,
    Failed = 3,
}
/**
 * Defines geometry type.
 */
enum ModelPrs_GeometryType {
    Body = 0,
    BRepRepresentation = 1,
    PolyVertexSet = 2,
    PolyRepresentation = 3,
    Measurement = 4,
    PMI = 5,
    Drawing = 6,
    Undefined = 100,
}
/**
 * Defines a visitor of the geometry.
 */
abstract class ModelPrs_GeometryVisitor {
    abstract visitBody(theBody: ModelData_Body, theRep: ModelData_BRepRepresentation | undefined): void;
    abstract visitBRepRepresentation(theBRep: ModelData_BRepRepresentation): void;
    abstract visitPolyVertexSet(thePVS: ModelData_PolyVertexSet): void;
    abstract visitPolyRepresentation(theRep: ModelData_PolyRepresentation): void;
    abstract visitPMIGraphicalElement(theElement: ModelData_PMIGraphicalElement): void;
    abstract visitMeasurement(theMeasurement: ModelPrs_Measurement): void;
    abstract visitDrawingElement(theElement: ModelData_DrawingElement): void;
}
/**
 * A handler for highlight on viewport.
 */
class ModelPrs_HighlightingHandler extends ModelPrs_SinglePointInputHandler {
    /**
     * Creates an instance of ModelPrs_HighlightingHandler.
     */
    constructor();
    override hoverEnter(theEvent: ModelPrs_PointerInputEvent): boolean;
    override hoverMove(theEvent: ModelPrs_PointerInputEvent): boolean;
    override hoverLeave(theEvent: ModelPrs_PointerInputEvent): void;
}
/**
 * Base class for all input events.
 */
class ModelPrs_InputEvent<TypeType extends string, TargetType extends ModelPrs_InputHandler> implements ModelPrs_Event<TypeType, TargetType> {
    /**
     * Creates an instance of ModelPrs_InputEvent.
     */
    constructor(theType: string, theViewport: ModelPrs_ViewPort, theModifiers: number, theTimeStamp?: number);
    get viewport(): ModelPrs_ViewPort;
    /**
     * The time (in milliseconds) at which the event was created.
     *
     * See also {@link https://developer.mozilla.org/en-US/docs/Web/API/Event/timeStamp Event.timeStamp} for more details.
     */
    get timeStamp(): number;
    /**
     * Indicates which keys were pressed on the keyboard to trigger the event.
     * It stores an OR combination of {@link ModelPrs_KeyboardModifier KeyboardModifier} values.
     */
    get modifiers(): number;
    /**
     * Indicates whether the alt key was pressed or not when a given pointer event occurs.
     *
     * See also {@link https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent/altKey MouseEvent.altKey} documentation.
     */
    get altKey(): boolean;
    /**
     * Indicates whether the ctrl key was pressed or not when a given pointer event occurs.
     *
     * See also {@link https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent/ctrlKey MouseEvent.ctrlKey} documentation.
     */
    get ctrlKey(): boolean;
    /**
     * Indicates whether the meta key was pressed or not when a given pointer event occurs.
     *
     * See also {@link https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent/metaKey MouseEvent.metaKey} documentation.
     */
    get metaKey(): boolean;
    /**
     * Indicates whether the shift key was pressed or not when a given pointer event occurs.
     *
     * See also {@link https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent/shiftKey MouseEvent.shiftKey} documentation.
     */
    get shiftKey(): boolean;
    /**
     * A string containing the input event's type.
     */
    readonly type: TypeType;
    /**
     * The input handler which has dispatched the event.
     */
    readonly target: TargetType;
}
/**
 * Base class for input handlers registered in {@link ModelPrs_InputManager}.
 */
class ModelPrs_InputHandler extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_InputHandler.
     */
    constructor();
    /**
     * If a input handler is disabled, it will reject all events.
     */
    enabled: boolean;
    /**
     * Indicates whether pointer buttons events are accepted by this handler. Default is `false`.
     */
    isAcceptPointerEvents: boolean;
    /**
     * Indicates whether hover events are accepted by this handler. Default is `false`.
     */
    isAcceptHoverEvents: boolean;
    /**
     * Indicates whether mouse wheel events are accepted by this handler. Default is `false`.
     */
    isAcceptWheelEvents: boolean;
    /**
     * Indicates whether keyboard events are accepted by this handler. Default is `false`.
     */
    isAcceptKeyEvents: boolean;
    /**
     * Handles event fired when a pointer becomes _active_.
     * For mouse, it is fired when the device transitions from no buttons pressed to at least one button pressed.
     * For touch, it is fired when physical contact is made with the digitizer.
     * For pen, it is fired when the stylus makes physical contact with the digitizer.
     */
    pointerDown(theEvent: ModelPrs_PointerInputEvent): boolean;
    /**
     * Handles event fired when an _active_ pointer changes coordinates.
     */
    pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    /**
     * Handles event is fired when a pointer is no longer _active_.
     */
    pointerUp(theEvent: ModelPrs_PointerInputEvent): void;
    /**
     * Handles event is fired when a pointer is no longer be able to generate events.
     */
    pointerCancel(theEvent: ModelPrs_PointerInputEvent): void;
    /**
     * Handles event fired when a pointing device is moved into a viewport boundaries without pressed any buttons.
     */
    hoverEnter(theEvent: ModelPrs_PointerInputEvent): boolean;
    /**
     * Handles event fired when a pointer device is moved within a viewport boundaries without pressed any buttons.
     */
    hoverMove(theEvent: ModelPrs_PointerInputEvent): boolean;
    /**
     * Handles event fired when a pointer device is moved out a viewport boundaries without pressed any buttons.
     */
    hoverLeave(theEvent: ModelPrs_PointerInputEvent): void;
    /**
     * Handles wheel event fires when the user rotates a wheel button on a pointing device (typically a mouse).
     */
    wheel(theEvent: ModelPrs_WheelInputEvent): boolean;
    /**
     * Handles key down event.
     */
    keyDown(theEvent: ModelPrs_KeyboardInputEvent): boolean;
    /**
     * Handles key up event.
     */
    keyUp(theEvent: ModelPrs_KeyboardInputEvent): void;
    /**
     * Handles resize event.
     */
    resize(theWidth: number, theHeight: number): void;
}
/**
 * Provides information about a point in a {@link ModelPrs_PointerInputEvent}.
 */
class ModelPrs_InputHandlerPoint {
    /**
     * Creates an instance of ModelPrs_InputHandlerPoint.
     */
    constructor(x: number, y: number);
    /**
     * A unique identifier of the point.
     */
    get id(): number;
    /**
     * The point coordinates (in pixels).
     */
    get position(): ModelData_Point2d;
    /**
     * The position of this point from the previous press or move event.
     */
    get lastPosition(): ModelData_Point2d;
    /**
     * The position at which this point was pressed.
     */
    get pressPosition(): ModelData_Point2d;
    /**
     * The most recent time at which this point was included in a {@link ModelPrs_PointerInputEvent}.
     */
    get timestamp(): number;
    /**
     * The time from the previous {@link ModelPrs_PointerInputEvent} that contained this point.
     */
    get lastTimestamp(): number;
    /**
     * The most recent time at which this point was pressed.
     */
    get pressTimestamp(): number;
}
type ModelPrs_InputHandlerAddedEvent<InputManagerType extends ModelPrs_InputManager = ModelPrs_InputManager> = ModelPrs_Event<"inputHandlerAdded",InputManagerType> & {
    inputHandler: ModelPrs_InputHandler;
}
type ModelPrs_InputHandlerReplacedEvent<InputManagerType extends ModelPrs_InputManager = ModelPrs_InputManager> = ModelPrs_Event<"inputHandlerReplaced",InputManagerType> & {
    previousInputHandler: ModelPrs_InputHandler;
    newInputHandler: ModelPrs_InputHandler;
}
type ModelPrs_InputHandlerRemovedEvent<InputManagerType extends ModelPrs_InputManager = ModelPrs_InputManager> = ModelPrs_Event<"inputHandlerRemoved",InputManagerType> & {
    inputHandler: ModelPrs_InputHandler;
}
type ModelPrs_InputHandlersRemovedAllEvent<InputManagerType extends ModelPrs_InputManager = ModelPrs_InputManager> = ModelPrs_Event<"inputHandlersRemovedAll",InputManagerType> & {
    inputHandlers: Array<ModelPrs_InputHandler>;
}
type ModelPrs_InputManagerEventMap<InputManagerType extends ModelPrs_InputManager> = {
    inputHandlerAdded: ModelPrs_InputHandlerAddedEvent<InputManagerType>;
    inputHandlerReplaced: ModelPrs_InputHandlerReplacedEvent<InputManagerType>;
    inputHandlerRemoved: ModelPrs_InputHandlerRemovedEvent<InputManagerType>;
    inputHandlersRemovedAll: ModelPrs_InputHandlersRemovedAllEvent<InputManagerType>;
    viewportChanged: ModelPrs_Event<"viewportChanged", InputManagerType>;
}
/**
 * Function which is called once for each input handler of the input manager in ascending order,
 * until it finds one where predicate returns `true`.
 */
type ModelPrs_InputManagerFindPredicate = (inputHandler: ModelPrs_InputHandler) => unknown;
/**
 * Manager of user input handlers of the {@link ModelPrs_ViewPort ViewPort}.
 */
class ModelPrs_InputManager extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_InputManager.
     */
    constructor();
    /**
     * Indicates enabled hover events handling or not. Default is `false` due performance reason.
     */
    get isHoverEnabled(): boolean;
    set isHoverEnabled(theIsHoverEnabled: boolean);
    /**
     * Number of added input handlers.
     */
    get numberOfInputHandlers(): number;
    /**
     * Returns iterable to stored input handles.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const inputHandler of inputManager.inputHandlers()) {
     *   // Access to the input handler
     * }
     * ```
     */
    inputHandlers(): IterableIterator<ModelPrs_InputHandler>;
    /**
     * Returns `true` if the manager contains the input handler. Otherwise returns `false`.
     */
    containsInputHandler(theInputHandler: ModelPrs_InputHandler): boolean;
    /**
     * Adds an input handler on the stack if it's not already on the stack.
     * Returns `true` if the input handler is successfully added to top of the stack.
     */
    pushInputHandler(theInputHandler: ModelPrs_InputHandler): boolean;
    /**
     * Removes an input handler from the top of the stack.
     */
    popInputHandler(): ModelPrs_InputHandler | undefined;
    /**
     * Removes specific input handler from the stack.
     * Returns `true` if the input handler has been successfully removed.
     */
    removeInputHandler(theInputHandler: ModelPrs_InputHandler): boolean;
    /**
     * Replace the input handler with another one.
     * Returns `true` if the input handler has been successfully replaced.
     */
    replaceInputHandler(thePreviousInputHandler: ModelPrs_InputHandler, theNewInputHandler: ModelPrs_InputHandler): boolean;
    /**
     * Removes all input handlers from the manager.
     */
    removeAllInputHandlers(): void;
    /**
     * Releases the data allocated by this instance.
     * Disposed instance cannot be used anymore.
     * Call this method whenever the instance is no longer used in your app.
     */
    override dispose(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_InputManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_InputManagerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_InputManager#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_InputManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_InputManagerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Returns the first input handler in the manager
     * where predicate is `true`, and `undefined` otherwise.
     */
    findInputHandler(theType: ModelPrs_InputManagerFindPredicate): ModelPrs_InputHandler | undefined;
}
class ModelPrs_KeyboardInputEvent<TypeType extends string = string, TargetType extends ModelPrs_InputHandler = ModelPrs_InputHandler> extends ModelPrs_InputEvent<TypeType,TargetType> {
    /**
     * Creates instance of ModelPrs_KeyboardInputEvent.
     */
    constructor(theType: string, theCode: string, theKey: string, theModifiers: number, theViewPort: ModelPrs_ViewPort, theTimeStamp?: number);
    /**
     * A physical key on the keyboard (as opposed to the character generated by pressing the key).
     *
     * See {@link https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/code KeyboardEvent.code} documentation for more details.
     */
    get code(): string;
    /**
     * The value of the key pressed by the user,
     * taking into consideration the state of modifier keys such as "Shift" as well as the keyboard locale and layout.
     *
     * See {@link https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key KeyboardEvent.key} documentation for more details.
     */
    get key(): string;
    /**
     * Creates keyboard input event from system keyboard event.
     */
    static fromKeyboardEvent(theEvent: KeyboardEvent, theViewport: ModelPrs_ViewPort): ModelPrs_KeyboardInputEvent;
}
/**
 * This enum describes the modifier keys.
 *
 * > __Notes__:
 * * On Macintosh keyboards, the `Alt` key is also known as the "option" key,
 *   the `Ctrl` key is the "control" key, the `Meta` key is the "command" key (`⌘`).
 * * On Windows keyboards, the `Meta` key is the "Windows" key (`⊞`).
 */
enum ModelPrs_KeyboardModifier {
    /**
     * No modifier key is pressed.
     */
    NoModifier = 0,
    /**
     * A Shift key on the keyboard is pressed.
     */
    ShiftModifier = 2,
    /**
     * A Ctrl key on the keyboard is pressed.
     */
    CtrlModifier = 4,
    /**
     * An Alt key on the keyboard is pressed.
     */
    AltModifier = 8,
    /**
     * A Meta key on the keyboard is pressed.
     */
    MetaModifier = 16,
}
type ModelPrs_LightEventMap<LightType extends ModelPrs_Light> = ModelPrs_FieldChangedEventMap<LightType> & {
    changed: ModelPrs_Event<"changed", LightType>;
}
/**
 * Base class for light sources.
 */
abstract class ModelPrs_Light extends ModelPrs_EventDispatcher {
    /**
     * The color of the light source. Default is white.
     */
    get color(): ModelData_ColorObject;
    set color(theColor: ModelData_ColorObject);
    /**
     * Numeric value of the light's strength/intensity. Default is `1`.
     */
    get intensity(): number;
    set intensity(theIntensity: number);
    /**
     * Indicates the light position and direction are defined relative to the camera orientation or not.
     */
    get isHeadlight(): boolean;
    set isHeadlight(theIsHeadlight: boolean);
    /**
     * Releases the data allocated by this instance.
     * Disposed instance cannot be used anymore.
     * Call this method whenever the instance is no longer used in your app.
     */
    override dispose(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_LightEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_LightEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_Light#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_LightEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_LightEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Event emitted when the light source is added to the lights manager.
 */
type ModelPrs_LightAddedEvent<LightsManagerType extends ModelPrs_LightsManager = ModelPrs_LightsManager> = ModelPrs_Event<"lightAdded",LightsManagerType> & {
    light: ModelPrs_Light;
}
/**
 * Event emitted when the light removed from the lights manager.
 */
type ModelPrs_LightRemovedEvent<LightsManagerType extends ModelPrs_LightsManager = ModelPrs_LightsManager> = ModelPrs_Event<"lightRemoved",LightsManagerType> & {
    light: ModelPrs_Light;
}
/**
 * Event emitted when all lights removed from the lights manager.
 */
type ModelPrs_LightsRemovedAllEvent<LightsManagerType extends ModelPrs_LightsManager = ModelPrs_LightsManager> = ModelPrs_Event<"lightsRemovedAll",LightsManagerType> & {
    lights: Array<ModelPrs_Light>;
}
type ModelPrs_LightsManagerEventMap<LightsManagerType extends ModelPrs_LightsManager> = {
    lightAdded: ModelPrs_LightAddedEvent<LightsManagerType>;
    lightRemoved: ModelPrs_LightRemovedEvent<LightsManagerType>;
    lightsRemovedAll: ModelPrs_LightsRemovedAllEvent<LightsManagerType>;
    viewportChanged: ModelPrs_Event<"viewportChanged", LightsManagerType>;
}
/**
 * Manager of light sources of the {@link ModelPrs_ViewPort ViewPort}.
 */
class ModelPrs_LightsManager extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_LightsManager.
     */
    constructor();
    get viewport(): ModelPrs_ViewPort | null;
    /**
     * Number of added lights.
     */
    get numberOfLights(): number;
    /**
     * Returns iterable to stored lights.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const light of lightManager.lights()) {
     *   // Access to the light
     * }
     * ```
     */
    lights(): Iterable<ModelPrs_Light>;
    /**
     * Returns `true` if the manager contains the light. Otherwise returns `false`.
     */
    containsLight(theLight: ModelPrs_Light): boolean;
    /**
     * Adds the light to the manager.
     */
    addLight(theLight: ModelPrs_Light): void;
    /**
     * Removes the light from the manager.
     */
    removeLight(theLight: ModelPrs_Light): void;
    /**
     * Removes all lights from the manager.
     */
    removeAllLights(): void;
    /**
     * Adds default lights to the manager.
     */
    addDefaultLights(): void;
    /**
     * Releases the data allocated by this instance.
     * Disposed instance cannot be used anymore.
     * Call this method whenever the instance is no longer used in your app.
     */
    override dispose(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_LightsManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_LightsManagerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_LightsManager#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_LightsManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_LightsManagerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Type of light source.
 */
enum ModelPrs_LightType {
    Ambient = 0,
    Directional = 1,
    Positional = 2,
    Spot = 3,
    Undefined = 100,
}
/**
 * Stores result of the linear measurement.
 */
abstract class ModelPrs_LinearMeasurement extends ModelPrs_Measurement {
    get points(): Array<ModelData_Point>;
    /**
     * Length unit. Default is {@link Base_LengthUnit.Base_LU_Millimeters}.
     */
    get lengthUnit(): Base_LengthUnit;
    set lengthUnit(theLengthUnit: Base_LengthUnit);
    /**
     * Direction of extension line. Chosen automatically if the value is `null`. Default is `null`.
     */
    get extensionLineDirection(): ModelData_Direction | null;
    set extensionLineDirection(theExtensionLineDirection: ModelData_Direction | null);
    override toString(): string;
}
type ModelPrs_MarkerEventMap<MarkerType extends ModelPrs_Marker> = ModelPrs_FieldChangedEventMap<MarkerType> & {
    canvasPositionChanged: ModelPrs_Event<"canvasPositionChanged", MarkerType>;
    zOrderChanged: ModelPrs_Event<"zOrderChanged", MarkerType>;
}
/**
 * Class represents "marker" in 3D space, which tracks the world and canvas coordinates.
 */
class ModelPrs_Marker extends ModelPrs_EventDispatcher {
    constructor(thePosition: ModelData_Point);
    /**
     * Marker position in 3D space.
     */
    get position(): ModelData_Point;
    set position(thePosition: ModelData_Point);
    /**
     * Marker position in canvas space.
     */
    get canvasPosition(): ModelData_Point2d;
    /**
     * Z-order of marker. The higher value means the further marker from the camera.
     */
    get zOrder(): number;
    /**
     * Calculates canvas position and zOrder from camera.
     */
    update(theCamera: ModelPrs_Camera): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_MarkerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_MarkerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_Marker#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_MarkerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_MarkerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Event emitted when the clip plane is added to the clip planes manager.
 */
type ModelPrs_MarkerAddedEvent<MarkersManagerType extends ModelPrs_MarkersManager = ModelPrs_MarkersManager> = ModelPrs_Event<"markerAdded",MarkersManagerType> & {
    marker: ModelPrs_Marker;
}
/**
 * Event emitted when the clip plane is removed from the clip planes manager.
 */
type ModelPrs_MarkerRemovedEvent<MarkersManagerType extends ModelPrs_MarkersManager = ModelPrs_MarkersManager> = ModelPrs_Event<"markerRemoved",MarkersManagerType> & {
    marker: ModelPrs_Marker;
}
/**
 * Event emitted when all clip planes are removed from the clip planes manager.
 */
type ModelPrs_MarkersRemovedAllEvent<MarkersManagerType extends ModelPrs_MarkersManager = ModelPrs_MarkersManager> = ModelPrs_Event<"markersRemovedAll",MarkersManagerType> & {
    markers: Array<ModelPrs_Marker>;
}
type ModelPrs_MarkersManagerEventMap<MarkersManagerType extends ModelPrs_MarkersManager> = {
    markerAdded: ModelPrs_MarkerAddedEvent<MarkersManagerType>;
    markerRemoved: ModelPrs_MarkerRemovedEvent<MarkersManagerType>;
    markersRemovedAll: ModelPrs_MarkersRemovedAllEvent<MarkersManagerType>;
    viewportChanged: ModelPrs_Event<"viewportChanged", MarkersManagerType>;
}
/**
 * Manager of markers of the {@link ModelPrs_ViewPort ViewPort}.
 */
class ModelPrs_MarkersManager extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_MarkersManager.
     */
    constructor();
    get viewport(): ModelPrs_ViewPort | null;
    /**
     * Number of added markers.
     */
    get numberOfMarkers(): number;
    /**
     * Returns iterable to added markers.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const marker of markersManager.markers()) {
     *   // Access to the marker
     * }
     * ```
     */
    markers(): Iterable<ModelPrs_Marker>;
    /**
     * Returns `true` if the manager contains the marker. Otherwise returns `false`.
     */
    containsMarker(theMarker: ModelPrs_Marker): boolean;
    /**
     * Adds the marker to the manager.
     */
    addMarker(theMarker: ModelPrs_Marker): void;
    /**
     * Removes the marker from the manager.
     */
    removeMarker(theMarker: ModelPrs_Marker): void;
    /**
     * Removes all markers from the manager.
     */
    removeAllMarkers(): void;
    /**
     * Releases the data allocated by this instance.
     * Disposed instance cannot be used anymore.
     * Call this method whenever the instance is no longer used in your app.
     */
    override dispose(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_MarkersManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_MarkersManagerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_MarkersManager#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_MarkersManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_MarkersManagerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
type ModelPrs_MeasurementEventMap<MeasurementType extends ModelPrs_Measurement> = ModelPrs_FieldChangedEventMap<MeasurementType> & {
    valueChanged: ModelPrs_Event<"valueChanged", MeasurementType>;
}
/**
 * Stores the source data and result of the measurement.
 * Objects of this class can be created using {@link ModelPrs_MeasurementFactory}.
 *
 * To display the measurement you need to create scene node from it using {@link ModelPrs_SceneNodeFactory}
 * and add that to the scene.
 * When the measurement settings are changed, it's necessary to call {@link ModelPrs_SceneNode ModelPrs_SceneNode#invalidate()}
 * or re-create a scene node for that measurement.
 */
abstract class ModelPrs_Measurement extends ModelPrs_EventDispatcher {
    /**
     * Measurement type.
     */
    get type(): ModelPrs_MeasurementType;
    /**
     * Measurement result in current units.
     */
    get value(): number;
    /**
     * Number of digits after the decimal point for value. Default is `2`.
     */
    get precision(): number;
    set precision(thePrecision: number);
    /**
     * Font size in millimeters.
     */
    get fontSize(): number;
    set fontSize(theFontSize: number);
    /**
     * Length of extension line in millimeters. Default is `40`.
     */
    get extensionLineLength(): number;
    set extensionLineLength(theExtensionLineSize: number);
    /**
     * Length of extension overhang in millimeters. Default is `4`.
     */
    get extensionOverhangLength(): number;
    set extensionOverhangLength(theExtensionOverhangLength: number);
    toString(): string;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_MeasurementEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_MeasurementEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_Measurement#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_MeasurementEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_MeasurementEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Creates measurement objects that store the result of a measurement.
 */
class ModelPrs_MeasurementFactory {
    /**
     * Creates a distance measurement between any two points.
     */
    createDistanceFromPoints(thePoint1: ModelData_Point, thePoint2: ModelData_Point): ModelPrs_LinearMeasurement | null;
    /**
     * Creates minimum distance measurement between any two shapes.
     */
    createDistanceFromShapes(theShape1: ModelData_Shape, theShape2: ModelData_Shape, theTrsf1?: ModelData_Transformation | null, theTrsf2?: ModelData_Transformation | null): ModelPrs_LinearMeasurement | null;
    /**
     * Creates an angle measurement between three points.
     */
    createAngleFromPoints(thePoint1: ModelData_Point, thePoint2: ModelData_Point, thePoint3: ModelData_Point): ModelPrs_AngularMeasurement | null;
}
/**
 * Defines a type of measurement.
 */
enum ModelPrs_MeasurementType {
    Distance = 0,
    AngleBetweenVertexes = 4,
    Undefined = 100,
}
class ModelPrs_MultiCameraManipulationHandler extends ModelPrs_CameraManipulationHandler {
    constructor(theViewPorts: Array<ModelPrs_ViewPort>);
    get viewports(): Array<ModelPrs_ViewPort>;
    override pointerDown(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUp(theEvent: ModelPrs_PointerInputEvent): void;
    override pointerCancel(theEvent: ModelPrs_PointerInputEvent): void;
    override wheel(theEvent: ModelPrs_WheelInputEvent): boolean;
    override keyDown(theEvent: ModelPrs_KeyboardInputEvent): boolean;
    override keyUp(theEvent: ModelPrs_KeyboardInputEvent): void;
}
class ModelPrs_MultiCameraMultiTouchHandler extends ModelPrs_PinchInputHandler {
    /**
     * Creates an instance of ModelPrs_CameraMultiTouchHandler.
     */
    constructor(theViewPorts: Array<ModelPrs_ViewPort>);
    /**
     * Default value is `1`.
     */
    panSpeed: number;
    /**
     * Default value is `1`.
     */
    zoomSpeed: number;
    /**
     * Default value is `1`.
     */
    zAxisRotationSpeed: number;
    get viewports(): Array<ModelPrs_ViewPort>;
}
class ModelPrs_MultiCameraPanHandler extends ModelPrs_SinglePointInputHandler {
    /**
     * Creates an instance of ModelPrs_MultiCameraPanHandler.
     */
    constructor(theViewPorts: Array<ModelPrs_ViewPort>);
    /**
     * Default value is `1`.
     */
    panSpeed: number;
    get viewports(): Array<ModelPrs_ViewPort>;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
}
class ModelPrs_MultiCameraRotateHandler extends ModelPrs_SinglePointInputHandler {
    /**
     * Creates an instance of ModelPrs_CameraRotateHandler.
     */
    constructor(theViewPorts: Array<ModelPrs_ViewPort>, theGravityPoint?: ModelData_Point | null);
    /**
     * Default value is `1.5`.
     */
    rotateSpeed: number;
    /**
     * The target the camera rotates around.
     * If `null`, the center of the viewport's scene bounding box is used.
     */
    gravityPoint: ModelData_Point | null;
    get viewports(): Array<ModelPrs_ViewPort>;
    override pointerDown(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override keyDown(theEvent: ModelPrs_KeyboardInputEvent): boolean;
}
class ModelPrs_MultiCameraZoomHandler extends ModelPrs_SinglePointInputHandler {
    /**
     * Creates an instance of ModelPrs_CameraZoomHandler.
     */
    constructor(theViewPorts: Array<ModelPrs_ViewPort>);
    /**
     * Default value is `false`;
     */
    wheelZoomReverseDirection: boolean;
    /**
     * Default value is `1`.
     */
    zoomSpeed: number;
    get viewports(): Array<ModelPrs_ViewPort>;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override wheel(theEvent: ModelPrs_WheelInputEvent): boolean;
}
type ModelPrs_MultiPointInputHandlerEventMap<MultiPointInputHandlerType extends ModelPrs_MultiPointInputHandler> = ModelPrs_PointerInputHandlerEventMap<MultiPointInputHandlerType> & {
    isActiveChanged: ModelPrs_Event<"isActiveChanged", MultiPointInputHandlerType>;
    currentPointsChanged: ModelPrs_Event<"currentPointsChanged", MultiPointInputHandlerType>;
    centroidChanged: ModelPrs_Event<"centroidChanged", MultiPointInputHandlerType>;
}
/**
 * An intermediate class for the handlers which expect a specific number of multiple points.
 */
class ModelPrs_MultiPointInputHandler extends ModelPrs_PointerInputHandler {
    /**
     * Creates an instance of ModelPrs_MultiPointInputHandler.
     */
    constructor();
    /**
     * The minimum number of points required to activate this handler. Default is 2.
     */
    minimumPointCount: number;
    /**
     * The maximum number of points this handler can utilize. Default is 2.
     */
    maximumPointCount: number;
    /**
     * Indicates whenever it has taken sole responsibility for handing one or more input points.
     */
    get isActive(): boolean;
    /**
     * A point exactly in the middle of the currently-pressed points.
     * `null` if the handler is not active.
     */
    get centroid(): ModelData_Point2d | null;
    /**
     * Returns iterable to stored current points.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const point of inputHandler.currentPoints()) {
     *   // Access to the point
     * }
     * ```
     */
    currentPoints(): Iterable<ModelPrs_InputHandlerPoint>;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_MultiPointInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_MultiPointInputHandlerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_MultiPointInputHandler#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_MultiPointInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_MultiPointInputHandlerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    override pointerDown(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUp(theEvent: ModelPrs_PointerInputEvent): void;
    override pointerCancel(theEvent: ModelPrs_PointerInputEvent): void;
}
/**
 * Encapsulates a picked clip plane.
 */
class ModelPrs_PickedClipPlaneEntity extends ModelPrs_PickedEntity {
    /**
     * Creates an instance of ModelPrs_PickedClipPlaneEntity.
     */
    constructor(thePoint: ModelData_Point, theTolerance: number, theCliPlane: ModelPrs_ClipPlane);
    /**
     * The picked clip plane.
     */
    clipPlane: ModelPrs_ClipPlane;
    override accept(theVisitor: ModelPrs_PickedEntityVisitor): void;
}
/**
 * Encapsulates a picked item.
 */
abstract class ModelPrs_PickedEntity {
    /**
     * World space point of the intersection.
     */
    point: ModelData_Point;
    /**
     * Tolerance in millimeters used on current entity detection.
     */
    tolerance: number;
    /**
     * Accepts an entity visitor.
     */
    abstract accept(theVisitor: ModelPrs_PickedEntityVisitor): void;
}
/**
 * Defines a visitor of the picked items.
 */
abstract class ModelPrs_PickedEntityVisitor {
    abstract visitPickedNodeEntity(theEntity: ModelPrs_PickedNodeEntity): void;
    abstract visitPickedShapeEntity(theEntity: ModelPrs_PickedShapeEntity): void;
    abstract visitPickedPolyShapeEntity(theEntity: ModelPrs_PickedPolyShapeEntity): void;
    abstract visitPickedPolyVertexEntity(theEntity: ModelPrs_PickedPolyVertexEntity): void;
    abstract visitPickedClipPlaneEntity(theEntity: ModelPrs_PickedClipPlaneEntity): void;
}
/**
 * Encapsulates a picked node.
 */
abstract class ModelPrs_PickedNodeEntity extends ModelPrs_PickedEntity {
    /**
     * The picked node.
     */
    node: ModelPrs_SceneNode;
    /**
     * World space normal at the point of intersection.
     */
    get normal(): ModelData_Direction | null;
    override accept(theVisitor: ModelPrs_PickedEntityVisitor): void;
}
/**
 * Encapsulates a picked Poly shape.
 */
abstract class ModelPrs_PickedPolyShapeEntity extends ModelPrs_PickedEntity {
    /**
     * The node that contains the picked poly shape.
     */
    node: ModelPrs_SceneNode;
    /**
     * The picked shape.
     */
    polyShape: ModelData_PolyVertexSet;
    /**
     * Index of the intersected element inside poly shape:
     * * index of face if the polyShape is {@link ModelData_IndexedTriangleSet}
     * * index of line segment if the polyShape is {@link ModelData_PolyLineSet}
     * * index of vertex if the polyShape is {@link ModelData_PolyPointSet}
     */
    elementIndex: number;
    /**
     * World space normal at the point of intersection.
     *
     * > __Note__: the value is available for {@link ModelData_IndexedTriangleSet Indexed Triangle Sets} only.
     */
    get normal(): ModelData_Direction | null;
    /**
     * U,V coordinates at point of intersection.
     *
     * > __Note__: the value is available for {@link ModelData_IndexedTriangleSet Indexed Triangle Sets} with UV coordinates.
     */
    get uv(): ModelData_Point2d | null;
    override accept(theVisitor: ModelPrs_PickedEntityVisitor): void;
}
/**
 * Encapsulates a picked specific vertex of poly shape.
 */
abstract class ModelPrs_PickedPolyVertexEntity extends ModelPrs_PickedEntity {
    /**
     * The node that contains the picked shaped.
     */
    node: ModelPrs_SceneNode;
    /**
     * The poly shape that contains the picked vertex.
     */
    polyShape: ModelData_PolyVertexSet;
    /**
     * Index of the intersected vertex inside poly shape.
     */
    vertexIndex: number;
    override accept(theVisitor: ModelPrs_PickedEntityVisitor): void;
}
/**
 * Encapsulates a picked B-Rep shape.
 */
abstract class ModelPrs_PickedShapeEntity extends ModelPrs_PickedEntity {
    /**
     * The node that contains the picked shape.
     */
    node: ModelPrs_SceneNode;
    /**
     * The picked shape.
     */
    shape: ModelData_Shape;
    /**
     * World space normal at the point of intersection.
     */
    get normal(): ModelData_Direction | null;
    override accept(theVisitor: ModelPrs_PickedEntityVisitor): void;
}
/**
 * Encapsulates a result of pick operation.
 */
abstract class ModelPrs_PickResult {
    /**
     * Picked entity detected on intersecting.
     */
    pickedEntity: ModelPrs_PickedEntity;
    /**
     * World space point of the intersection.
     */
    get point(): ModelData_Point;
    /**
     * World space normal at the point of intersection.
     */
    get normal(): ModelData_Direction | null;
    /**
     * Distance between the origin of the ray and the intersection.
     */
    get distance(): number;
    /**
     * The node which has been intersected on pick operation.
     */
    get node(): ModelPrs_SceneNode | null;
}
/**
 * PinchInputHandler "updated" event.
 */
type ModelPrs_PinchInputHandlerUpdatedEvent = {
    type: "updated";
    target: ModelPrs_PinchInputHandler;
}
type ModelPrs_PinchInputHandlerEventMap<PinchInputHandlerType extends ModelPrs_PinchInputHandler> = ModelPrs_MultiPointInputHandlerEventMap<PinchInputHandlerType> & {
    updated: ModelPrs_Event<"updated", PinchInputHandlerType>;
}
/**
 * Handler that interprets a multi-finger gesture to interactively rotate, zoom, and drag.
 */
class ModelPrs_PinchInputHandler extends ModelPrs_MultiPointInputHandler {
    /**
     * Creates an instance of ModelPrs_PinchInputHandler.
     */
    constructor();
    /**
     * The scale factor while the pinch gesture is being performed.
     * It is 1.0 when the gesture begins, increases as the points are spread apart,
     * and decreases as the points are brought together.
     */
    get scale(): number;
    /**
     * The rotation of the pinch gesture in degrees, with positive values clockwise.
     * It is 0 when the gesture begins.
     */
    get rotation(): number;
    /**
     * The translation of the gesture centroid.
     * It is (0, 0) when the gesture begins.
     */
    get translation(): ModelData_Vector2d;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_PinchInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_PinchInputHandlerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_PinchInputHandler#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_PinchInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_PinchInputHandlerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Defines pointer buttons.
 */
enum ModelPrs_PointerButton {
    NoButton = 0,
    LeftButton = 1,
    RightButton = 2,
    MiddleButton = 4,
    Undefined = -1,
}
/**
 * The universal input event produced by a hardware agnostic representation of input devices
 * (such as a mouse, pen or contact point on a touch-enable surface).
 */
class ModelPrs_PointerInputEvent<TypeType extends string = string, TargetType extends ModelPrs_InputHandler = ModelPrs_InputHandler> extends ModelPrs_InputEvent<TypeType,TargetType> {
    /**
     * Creates instance of ModelPrs_PointerInputEvent.
     */
    constructor(theType: string, thePoint: ModelPrs_InputHandlerPoint, theButton: ModelPrs_PointerButton, theButtons: number, theModifiers: number, theViewPort: ModelPrs_ViewPort, theTimeStamp?: number, thePointerType?: ModelPrs_PointerType);
    /**
     * The pointer position when pointer event is dispatched.
     */
    get point(): ModelPrs_InputHandlerPoint;
    /**
     * The device type (mouse, pen, or touch) that caused a given pointer event.
     */
    get pointerType(): ModelPrs_PointerType;
    /**
     * Indicates the button was pressed/released on the pointer device to trigger the event.
     * If neither buttons nor touch/pen contact changed since last event returns {@link ModelPrs_PointerButton ModelPrs_PointerButton.NoButton}.
     */
    get button(): ModelPrs_PointerButton;
    /**
     * Indicates which buttons are pressed on the pointer when a event is triggered.
     * It stores an OR combination of {@link ModelPrs_PointerButton} values.
     */
    get buttons(): number;
    /**
     * Creates pointer input event from system event.
     */
    static fromPointerEvent(theEvent: PointerEvent, theViewport: ModelPrs_ViewPort, thePoint?: ModelPrs_InputHandlerPoint): ModelPrs_PointerInputEvent;
}
type ModelPrs_PointerInputHandlerEventMap<PointerInputHandlerType extends ModelPrs_PointerInputHandler> = ModelPrs_FieldChangedEventMap<PointerInputHandlerType> & {
}
/**
 * An intermediate class for handlers which allow filtering based pressed buttons, keyboard modifiers and pointer type.
 */
class ModelPrs_PointerInputHandler extends ModelPrs_InputHandler {
    /**
     * Creates an instance of ModelPrs_PointerInputHandler.
     */
    constructor();
    /**
     * The maximum distance in pixels that the user must drag an event point
     * in order to have it treated as a drag gesture.
     * Default is 5 pixels.
     */
    get dragDistance(): number;
    set dragDistance(theDragDistance: number);
    /**
     * Specifies the pointer states required to accept pointer event.
     * If keyboard modifier is not specified, the handler does not care which modifiers are pressed.
     * If pointer type is not specified, the handler does not care which device dispatches the event.
     */
    addAcceptedPointerButton(theButton: ModelPrs_PointerButton, theKeyboardModifiers?: number, thePointerType?: string): void;
    removeAcceptedPointerButton(theButton: ModelPrs_PointerButton, theKeyboardModifiers?: number, thePointerType?: string): void;
    clearAcceptedPointerButtons(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_PointerInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_PointerInputHandlerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_PointerInputHandler#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_PointerInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, ModelPrs_PointerInputHandler, ModelPrs_PointerInputHandlerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Defines pointer type.
 */
enum ModelPrs_PointerType {
    Mouse = "mouse",
    Pen = "pen",
    Touch = "touch",
    Undefined = "",
}
type ModelPrs_SceneEventMap<SceneType extends ModelPrs_Scene> = {
    boundingBoxChanged: ModelPrs_Event<"boundingBoxChanged", SceneType>;
    updated: ModelPrs_Event<"updated", SceneType>;
}
/**
 * Provides CAD Exchanger visualization graph.
 */
class ModelPrs_Scene extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_Scene3D.
     */
    constructor();
    /**
     * Scene selection manager.
     */
    selectionManager: ModelPrs_SelectionManager;
    /**
     * Bounding box of the visible nodes on the scene.
     */
    get boundingBox(): ModelData_Box;
    /**
     * Number of added roots.
     */
    get numberOfRoots(): number;
    /**
     * Adds a root scene node.
     */
    addRoot(theRoot: ModelPrs_SceneNode): boolean;
    /**
     * Removes a root scene node.
     */
    removeRoot(theRoot: ModelPrs_SceneNode): boolean;
    /**
     * Removes all root scene nodes.
     */
    clear(): void;
    /**
     * Returns iterable to the root nodes.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const root of scene.roots()) {
     *   // Access to the root
     * }
     * ```
     */
    roots(): Iterable<ModelPrs_SceneNode>;
    /**
     * Accepts a visitor.
     */
    accept(theVisitor: ModelPrs_SceneNodeVisitor): void;
    /**
     * Applies changes of attached Scene Nodes.
     *
     * Updating the scene can be time-consuming and therefore should be delayed until changes to the underlying scene node's data have been applied.
     *
     * This method is asynchronous. It means that changes will be applied piecemeal.
     *
     * Any manipulations with scene nodes forbidden while changes are applied.
     * Resolved promise indicates all changes were applied.
     */
    update(theProgressScope?: Base_ProgressScope): Promise<void>;
    /**
     * Releases the data allocated by this instance.
     * Disposed instance cannot be used anymore.
     * Call this method whenever the instance is no longer used in your app.
     */
    override dispose(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_SceneEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_SceneEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_Scene#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_SceneEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_SceneEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Represents a node in the visual scene graph.
 *
 * Scene nodes constitute hierarchy of the logical objects on the 3D scene.
 * A node has a single parent node and may have zero, one or multiple child nodes.
 *
 * The scene node is not typed: ModelPrs_SceneNode may not have subclasses.
 * Semantics is determined by the application.
 *
 * Each node can carry various attributes that define how the object or a sub-tree is displayed.
 * Implementation has been optimized to allocate memory only for those attributes items
 * that have been explicitly created (e.g. transformation matrix or a material).
 * When data items are not explicitly defined then no extra memory footprint is incurred.
 *
 * Scene nodes may not be shared among parents, sharing of heavy-weight data (e.g. geometry)
 * is provided via shareable data attributes. For instance, two instances of the same part in an assembly
 * will be mapped to two scene nodes referring to the same geometry and distinct transformations.
 */
class ModelPrs_SceneNode extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_SceneNode.
     */
    constructor();
    /**
     * Parent node.
     */
    get parent(): ModelPrs_SceneNode | null;
    /**
     * The scene which the node is attached to.
     *
     * > __Note__: the property is calculated on every access, iterating up to the root node.
     */
    get scene(): ModelPrs_Scene | null;
    /**
     * Geometry attribute.
     */
    get geometry(): ModelPrs_Geometry | null;
    set geometry(theGeometry: ModelPrs_Geometry | null);
    /**
     * Appearance attribute.
     */
    get appearance(): ModelData_Appearance | null;
    set appearance(theAppearance: ModelData_Appearance | null);
    /**
     * Appearance attribute combined with parents appearances.
     */
    get combinedAppearance(): ModelData_Appearance | null;
    /**
     * Transformation attribute.
     */
    get transformation(): ModelData_Transformation | null;
    set transformation(theTransformation: ModelData_Transformation | null);
    /**
     * Transformation attribute combined with parents transformations.
     */
    get combinedTransformation(): ModelData_Transformation | null;
    /**
     * Style attribute.
     */
    get style(): ModelPrs_Style | null;
    set style(theStyle: ModelPrs_Style | null);
    /**
     * Style attribute combined with parents styles.
     */
    get combinedStyle(): ModelPrs_Style | null;
    /**
     * Display mode attribute.
     */
    get displayMode(): ModelPrs_DisplayMode;
    set displayMode(theDisplayMode: ModelPrs_DisplayMode);
    /**
     * Display mode attribute combined with parents display modes.
     */
    get combinedDisplayMode(): ModelPrs_DisplayMode;
    /**
     * Selection mode attribute.
     */
    get selectionMode(): ModelPrs_SelectionMode;
    set selectionMode(theSelectionMode: ModelPrs_SelectionMode);
    /**
     * Selection mode attribute combined with parents selection modes.
     */
    get combinedSelectionMode(): ModelPrs_SelectionMode;
    /**
     * Visibility mode attribute.
     */
    get visibilityMode(): ModelPrs_VisibilityMode;
    set visibilityMode(theVisibilityMode: ModelPrs_VisibilityMode);
    /**
     * Visibility mode attribute combined with parents visibility modes.
     */
    get combinedVisibilityMode(): ModelPrs_VisibilityMode;
    /**
     * Detaches node from parent.
     *
     * > __Note__: this method does not remove roots from the scene.
     */
    detach(): ModelPrs_SceneNode | null;
    /**
     * Adds child node. Returns `false` if the node already has a parent, otherwise returns `true`.
     */
    addChildNode(theChild: ModelPrs_SceneNode): boolean;
    /**
     * Removes child node. Returns `true` if the node has been successfully removed.
     */
    removeChildNode(theChild: ModelPrs_SceneNode): boolean;
    /**
     * Removes all children nodes.
     */
    removeChildNodes(): void;
    /**
     * Returns iterable to child nodes.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const childNode of node.childNodes()) {
     *   // Access to the child node
     * }
     * ```
     */
    childNodes(): Iterable<ModelPrs_SceneNode>;
    /**
     * Accepts a visitor.
     */
    accept(theVisitor: ModelPrs_SceneNodeVisitor): void;
    /**
     * Marks scene nodes sub tree to be forcibly updated on the next scene `update()` call.
     */
    invalidate(): void;
}
/**
 * Creates a scene nodes and its children from input data model objects.
 *
 * The factory supports various input types to create scene nodes from:
 * * Entire ModelData_Model;
 * * Assembly, part or instance;
 * * B-Rep or polygonal representation;
 * * Individual body inside B-Rep representation;
 * * Individual mesh (triangle set, polyline set or point set).
 *
 * There are two ways to create scene nodes:
 * 1. Use createGraph* API that traverses a model entirely, creates scene nodes and their children.
 *    This prevents you from influencing the creation of the scene nodes. Therefore, it is more suitable for
 *    simply displaying the entire model. For example, to take screenshots.
 * 2. Use createNode* API that creates a single scene node. It gives you the ability to process
 *    each scene node and manually create a scene graph. For example, to create items for tree view and add
 *    a relation to scene nodes.
 */
class ModelPrs_SceneNodeFactory {
    /**
     * Creates scene graph from whole model. Returns `null` if the model is empty.
     */
    createGraphFromModel(theModel: ModelData_Model, theMaskOrName: ModelData_RepresentationMask | string): Promise<(ModelPrs_SceneNode|null)>;
    /**
     * Creates scene node from scene graph element.
     */
    createNodeFromSceneGraphElement(theSceneGraphElement: ModelData_SceneGraphElement): ModelPrs_SceneNode;
    /**
     * Creates scene node from polygonal or B-Rep representation.
     */
    createNodeFromRepresentation(theRepresentation: ModelData_Representation): ModelPrs_SceneNode;
    /**
     * Creates scene node from B-Rep body and optional B-Rep representation.
     */
    createNodeFromBody(theBody: ModelData_Body, theRepresentation?: ModelData_BRepRepresentation): ModelPrs_SceneNode;
    /**
     * Creates scene node from poly vertex set.
     */
    createNodeFromPolyVertexSet(thePolyVertexSet: ModelData_PolyVertexSet): ModelPrs_SceneNode;
    /**
     * Creates scene node from PMI graphical element.
     */
    createNodeFromPMIGraphicalElement(thePMIGraphicalElement: ModelData_PMIGraphicalElement): ModelPrs_SceneNode;
    /**
     * Creates scene node from measurement.
     */
    createNodeFromMeasurement(theMeasurement: ModelPrs_Measurement): ModelPrs_SceneNode;
    /**
     * Creates scene graph from drawing sheet.
     */
    createGraphFromDrawingSheet(theDrawingSheet: ModelData_DrawingSheet, theSwitchWhiteColorToBlack?: boolean): ModelPrs_SceneNode;
    /**
     * Creates scene node for drawing element.
     */
    createNodeFromDrawingElement(theElement: ModelData_DrawingElement): ModelPrs_SceneNode;
    /**
     * Creates scene graph from whole BIM model. Returns `null` if the model is empty.
     */
    createGraphFromBIMModel(theBIMModel: ModelData_BIMModel, theMaskOrName: ModelData_RepresentationMask | string): Promise<(ModelPrs_SceneNode|null)>;
    /**
     * Creates scene graph from BIM geometry element.
     */
    createGraphFromBIMGeometryElement(theGeometryElement: ModelData_BIMGeometryElement, theMaskOrName: ModelData_RepresentationMask | string): Promise<ModelPrs_SceneNode>;
    /**
     * Creates scene node for drawing element.
     */
    createNodeFromBIMElement(theElement: ModelData_BIMElement): ModelPrs_SceneNode;
}
/**
 * Defines a visitor of the scene node.
 */
abstract class ModelPrs_SceneNodeVisitor {
    abstract visitEnter(theNode: ModelPrs_SceneNode): boolean;
    abstract visitLeave(theNode: ModelPrs_SceneNode): void;
}
/**
 * Encapsulates a selected entity.
 */
abstract class ModelPrs_SelectedEntity {
    /**
     * Accepts an entity visitor.
     */
    abstract accept(theVisitor: ModelPrs_SelectedEntityVisitor): void;
    /**
     * Compares two selection entity to be equal by value.
     */
    isEqual(theEntity: ModelPrs_SelectedEntity): boolean;
}
/**
 * Defines a visitor of the selected entities.
 */
abstract class ModelPrs_SelectedEntityVisitor {
    abstract visitShapeEntity(theShapeEntity: ModelPrs_SelectedShapeEntity): void;
    abstract visitPolyShapeEntity(thePolyShapeEntity: ModelPrs_SelectedPolyShapeEntity): void;
    abstract visitPolyVertexEntity(thePolyVertexEntity: ModelPrs_SelectedPolyVertexEntity): void;
}
/**
 * Encapsulates a selected poly vertex set.
 */
class ModelPrs_SelectedPolyShapeEntity extends ModelPrs_SelectedEntity {
    /**
     * Creates an instance of ModelPrs_SelectedPolyShapeEntity.
     */
    constructor(thePolyShape: ModelData_PolyVertexSet);
    /**
     * Selected poly vertex set.
     */
    get polyShape(): ModelData_PolyVertexSet;
    override accept(theVisitor: ModelPrs_SelectedEntityVisitor): void;
    override isEqual(theEntity: ModelPrs_SelectedEntity): boolean;
}
/**
 * Encapsulates a selected poly vertex of the {@link ModelData_PolyVertexSet poly shape}.
 */
class ModelPrs_SelectedPolyVertexEntity extends ModelPrs_SelectedEntity {
    /**
     * Creates an instance of ModelPrs_SelectedPolyVertexEntity.
     */
    constructor(thePolyShape: ModelData_PolyVertexSet, theVertexIndex: number);
    /**
     * Selected poly vertex set.
     */
    get polyShape(): ModelData_PolyVertexSet;
    /**
     * Index of the selected vertex inside poly shape.
     */
    get vertexIndex(): number;
    override accept(theVisitor: ModelPrs_SelectedEntityVisitor): void;
    override isEqual(theEntity: ModelPrs_SelectedEntity): boolean;
}
/**
 * Encapsulates a selected BRep shape.
 */
class ModelPrs_SelectedShapeEntity extends ModelPrs_SelectedEntity {
    /**
     * Creates an instance of ModelPrs_SelectedShapeEntity.
     */
    constructor(theShape: ModelData_Shape);
    /**
     * Selected B-Rep shape.
     */
    get shape(): ModelData_Shape;
    override accept(theVisitor: ModelPrs_SelectedEntityVisitor): void;
    override isEqual(theEntity: ModelPrs_SelectedEntity): boolean;
}
/**
 * A handler for selection on viewport.
 */
class ModelPrs_SelectionHandler extends ModelPrs_TapInputHandler {
    /**
     * Creates an instance of ModelPrs_SelectionHandler.
     */
    constructor();
    /**
     * Primary pointer button. Default is {@link ModelPrs_SelectionHandler ModelPrs_PointerButton.LeftButton}
     */
    get primaryButton(): ModelPrs_PointerButton;
    set primaryButton(thePrimaryButton: ModelPrs_PointerButton);
    /**
     * The keyboard modifiers used to determine which kind of selection (multiple or single) should applied.
     * `undefined` value disables multiple selection.
     * Default is {@link ModelPrs_KeyboardModifier ModelPrs_KeyboardModifier.CtrlModifier}.
     */
    get multipleSelectionModifier(): number | undefined;
    set multipleSelectionModifier(theMultipleSelectionModifier: number | undefined);
    /**
     * Picks the objects on the scene at the pixel coordinates of the event.
     */
    pick(theEvent: ModelPrs_PointerInputEvent): ModelPrs_PickResult | null;
    override tap(theEvent: ModelPrs_PointerInputEvent): void;
}
type ModelPrs_HighlightingItem = ModelPrs_SelectionItem;
/**
 * Encapsulates a selection of specific node or part of node.
 */
class ModelPrs_SelectionItem {
    /**
     * Creates an instance of ModelPrs_SelectionItem.
     */
    constructor(theNode: ModelPrs_SceneNode, theEntities?: ModelPrs_SelectedEntity | Array<ModelPrs_SelectedEntity> | undefined);
    /**
     * The node associated with this selection item.
     */
    get node(): ModelPrs_SceneNode;
    /**
     * Indicates whether the item represents whole selected node or not.
     */
    get isWholeSelectedNode(): boolean;
    /**
     * Number of selected entities associated with current selection item.
     * `0` if the whole node is selected.
     */
    get numberOfEntities(): number;
    /**
     * Creates SelectionItem from PickResult object.
     */
    static fromPickResult(thePickResult: ModelPrs_PickResult): ModelPrs_SelectionItem | null;
    /**
     * Returns iterable to associated entities.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const entity of item.entities()) {
     *   // Access to the entity
     * }
     * ```
     */
    entities(): Iterable<ModelPrs_SelectedEntity>;
}
/**
 * The event dispatched on selection changing.
 */
type ModelPrs_SelectionChangedEvent<SelectionManagerType extends ModelPrs_SelectionManager = ModelPrs_SelectionManager> = ModelPrs_Event<"selectionChanged",SelectionManagerType> & {
    added: Array<ModelPrs_SelectionItem>;
    removed: Array<ModelPrs_SelectionItem>;
}
/**
 * The event dispatched on highlighting changing
 */
type ModelPrs_HighlightingChangedEvent<SelectionManagerType extends ModelPrs_SelectionManager = ModelPrs_SelectionManager> = ModelPrs_Event<"highlightingChanged",SelectionManagerType> & {
    added: Array<ModelPrs_HighlightingItem>;
    removed: Array<ModelPrs_HighlightingItem>;
}
type ModelPrs_SelectionManagerEventMap<SelectionManagerType extends ModelPrs_SelectionManager> = {
    selectionChanged: ModelPrs_SelectionChangedEvent<SelectionManagerType>;
    highlightingChanged: ModelPrs_HighlightingChangedEvent<SelectionManagerType>;
}
/**
 * Manager of selected items on the {@link ModelPrs_Scene Scene}.
 */
class ModelPrs_SelectionManager extends ModelPrs_EventDispatcher {
    /**
     * Creates instance of ModelPrs_SelectionManager.
     */
    constructor(theScene: ModelPrs_Scene);
    /**
     * Pick tolerance in pixels for line and point picking. Default is 5px.
     */
    pickTolerance: number;
    /**
     * Number of selected items.
     */
    get numberOfSelectedItems(): number;
    /**
     * Number of highlighted items.
     */
    get numberOfHighlightedItems(): number;
    /**
     * Returns iterable to selected items.
     *
     * <span class="doxygen-js"></span>
     * ```js
     * for (const item of selectionManager.selectedItems()) {
     *   // Access to the selected item
     * }
     * ```
     */
    selectedItems(): Iterable<ModelPrs_SelectionItem>;
    /**
     * Returns a highlighted item.
     */
    highlightedItem(): ModelPrs_HighlightingItem | null;
    /**
     * Picks the objects on the scene from the word-space point and direction.
     */
    pick(theOrigin: ModelData_XYZ, theDirection: ModelData_XYZ): ModelPrs_PickResult | null;
    /**
     * Picks the objects on the scene at the pixel coordinates of the viewport.
     * (0, 0) coords corresponds to the top-left corner of the viewport.
     */
    pickFromViewport(theX: number, theY: number, theViewport: ModelPrs_ViewPort): ModelPrs_PickResult | null;
    /**
     * Select the objects on the scene at the pixel coordinates of the viewport.
     * (0, 0) coords corresponds to the top-left corner of the viewport.
     */
    selectFromViewport(theX: number, theY: number, theViewport: ModelPrs_ViewPort, theBreakSelection?: boolean): void;
    /**
     * Highlight the object on the scene at the pixel coordinates of the viewport.
     * (0, 0) coords corresponds to the top-left corner of the viewport.
     */
    highlightFromViewport(theX: number, theY: number, theViewport: ModelPrs_ViewPort): void;
    /**
     * Returns `true` if the specific item is selected.
     */
    isSelected(theItem: ModelPrs_SelectionItem): boolean;
    /**
     * Selects a specific selection item. Returns `false` if the item is cannot be selected or already selected.
     */
    select(theItem: ModelPrs_SelectionItem, theBreakSelection?: boolean, theDispatchEvent?: boolean): boolean;
    /**
     * Deselects a specific selection item. Returns `true` if the item has been successfully deselected.
     */
    deselect(theItem: ModelPrs_SelectionItem, theDispatchEvent?: boolean): boolean;
    /**
     * Inverts selection of specific item.
     * If the item is selected it will be deselected. If the item is not selected it will be selected.
     * Returns `false` if the item is cannot be selected/deselected.
     */
    invertSelection(theItem: ModelPrs_SelectionItem, theDispatchEvent?: boolean): boolean;
    /**
     * Deselects any selected items on the scene.
     */
    deselectAll(theDispatchEvent?: boolean): void;
    /**
     * Returns `true` if the specific scene node is selected.
     */
    isNodeSelected(theNode: ModelPrs_SceneNode): boolean;
    /**
     * Selects specific node on the scene including all child nodes.
     * Returns `true` if the node has been selected.
     */
    selectNode(theNode: ModelPrs_SceneNode, theBreakSelection?: boolean, theDispatchEvent?: boolean): boolean;
    /**
     * Deselects node on the scene.
     */
    deselectNode(theNode: ModelPrs_SceneNode, theDispatchEvent?: boolean): boolean;
    /**
     * Invert selection of specific node on the scene.
     * If the node is selected it will be deselected. If the node is not selected it will be selected.
     * Returns `false` if the node is cannot be selected/deselected.
     */
    invertNodeSelection(theNode: ModelPrs_SceneNode, theDispatchEvent?: boolean): boolean;
    /**
     * Returns `true` if the specific item is highlighted.
     */
    isHighlighted(theItem: ModelPrs_HighlightingItem): boolean;
    /**
     * Highlights a specific item. Returns `false` if the item is cannot be highlighted or already highlighted.
     */
    highlight(theItem: ModelPrs_HighlightingItem, theDispatchEvent?: boolean): boolean;
    /**
     * Unhighlights any highlighted items on the scene.
     */
    unhighlightAll(theDispatchEvent?: boolean): void;
    /**
     * Highlights specific node on the scene including all child nodes.
     * Returns `true` if the node has been highlighted.
     */
    highlightNode(theNode: ModelPrs_SceneNode, theDispatchEvent?: boolean): boolean;
    /**
     * Returns `true` if the specific scene node is highlighted.
     */
    isNodeHighlighted(theNode: ModelPrs_SceneNode): boolean;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_SelectionManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_SelectionManagerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_SelectionManager#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_SelectionManagerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_SelectionManagerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Selection mode of the scene node.
 */
enum ModelPrs_SelectionMode {
    None = 0,
    Node = 1,
    Body = 2,
    Solid = 4,
    Shell = 8,
    Face = 16,
    Wire = 32,
    Edge = 64,
    Vertex = 128,
    PolyShape = 256,
    PolyVertex = 2048,
    Undefined = 32768,
}
type ModelPrs_SinglePointInputHandlerEventMap<SinglePointInputHandlerType extends ModelPrs_SinglePointInputHandler> = ModelPrs_PointerInputHandlerEventMap<SinglePointInputHandlerType> & {
    pointChanged: ModelPrs_Event<"pointChanged", SinglePointInputHandlerType>;
    isActiveChanged: ModelPrs_Event<"isActiveChanged", SinglePointInputHandlerType>;
}
/**
 * An intermediate class for the handlers which expect only a single point.
 */
class ModelPrs_SinglePointInputHandler extends ModelPrs_PointerInputHandler {
    /**
     * Creates an instance of ModelPrs_SinglePointInputHandler.
     */
    constructor();
    /**
     * Indicates whenever handler detects drag gesture.
     */
    get isActive(): boolean;
    /**
     * Pressed point data. `null` if handler is inactive.
     */
    get point(): ModelPrs_InputHandlerPoint | null;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_SinglePointInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_SinglePointInputHandlerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_SinglePointInputHandler#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_SinglePointInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_SinglePointInputHandlerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    override pointerDown(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUp(theEvent: ModelPrs_PointerInputEvent): void;
    override pointerCancel(theEvent: ModelPrs_PointerInputEvent): void;
}
type ModelPrs_StyleEventMap<StyleType extends ModelPrs_Style> = ModelPrs_FieldChangedEventMap<StyleType> & {
    changed: ModelPrs_Event<"changed", StyleType>;
}
/**
 * Defines style of the scene node.
 */
class ModelPrs_Style extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_Style.
     */
    constructor();
    /**
     * Default appearance applied to scene node before applying any appearance.
     */
    get defaultAppearance(): ModelData_Appearance | null;
    set defaultAppearance(theDefaultAppearance: ModelData_Appearance | null);
    /**
     * Appearance applied when scene node is highlighted.
     */
    get highlightAppearance(): ModelData_Appearance | null;
    set highlightAppearance(theHighlightAppearance: ModelData_Appearance | null);
    /**
     * Appearance applied when scene node is selected.
     */
    get selectionAppearance(): ModelData_Appearance | null;
    set selectionAppearance(theSelectionAppearance: ModelData_Appearance | null);
    /**
     * Appearance applied when scene node is in ghost mode.
     */
    get ghostModeAppearance(): ModelData_Appearance | null;
    set ghostModeAppearance(theGhostModeAppearance: ModelData_Appearance | null);
    /**
     * Appearance applied to scene node boundaries in {@link ModelPrs_DisplayMode ShadedWithBoundaries} display mode.
     */
    get boundariesDefaultAppearance(): ModelData_Appearance | null;
    set boundariesDefaultAppearance(theDefaultAppearance: ModelData_Appearance | null);
    /**
     * Appearance applied to the scene node boundaries in {@link ModelPrs_DisplayMode ShadedWithBoundaries} display mode when the node is highlighted.
     */
    get boundariesHighlightAppearance(): ModelData_Appearance | null;
    set boundariesHighlightAppearance(theHighlightAppearance: ModelData_Appearance | null);
    /**
     * Appearance applied to the scene node boundaries in {@link ModelPrs_DisplayMode ShadedWithBoundaries} display mode when the node is selected.
     */
    get boundariesSelectionAppearance(): ModelData_Appearance | null;
    set boundariesSelectionAppearance(theSelectionAppearance: ModelData_Appearance | null);
    /**
     * Appearance applied to the scene node boundaries in {@link ModelPrs_DisplayMode ShadedWithBoundaries} display mode when the node is in ghost mode.
     */
    get boundariesGhostModeAppearance(): ModelData_Appearance | null;
    set boundariesGhostModeAppearance(theGhostModeAppearance: ModelData_Appearance | null);
    /**
     * Copies internal contents and combines with attributes from another style.
     */
    copyAndCombineWith(theStyle: ModelPrs_Style): ModelPrs_Style;
    /**
     * Combine style with current.
     */
    combineWith(theStyle: ModelPrs_Style): this;
    /**
     * Creates copy of the current style.
     */
    clone(): ModelPrs_Style;
    /**
     * Copies style attributes from another style.
     */
    copy(theStyle: ModelPrs_Style): this;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_StyleEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_StyleEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_Style#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_StyleEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_StyleEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
type ModelPrs_TapInputHandlerEventMap<TapInputHandlerType extends ModelPrs_SinglePointInputHandler> = ModelPrs_SinglePointInputHandlerEventMap<TapInputHandlerType> & {
    tap: ModelPrs_Event<"tap", TapInputHandlerType>;
    doubleTap: ModelPrs_Event<"doubleTap", TapInputHandlerType>;
    longPress: ModelPrs_Event<"longPress", TapInputHandlerType>;
}
/**
 * A handler for taps on a touchscreen or clicks on a mouse.
 */
class ModelPrs_TapInputHandler extends ModelPrs_SinglePointInputHandler {
    /**
     * Creates an instance of ModelPrs_TapInputHandler.
     */
    constructor();
    /**
     * The time limit in milliseconds that distinguishes a double taps from two consecutive pointer taps.
     * Default is 400ms.
     */
    doubleTapInterval: number;
    /**
     * The time in milliseconds that a point must be pressed in order to trigger a long press gesture.
     * Default is 500ms.
     */
    longPressInterval: number;
    /**
     * The the maximum distance, in pixels, that a pointer can be moved between two consecutive taps
     * and still have it detected as a double-tap. Default is 5 pixels.
     */
    get doubleTapDistance(): number;
    set doubleTapDistance(theDoubleTapDistance: number);
    /**
     * Handles tap event fired when a pointing device button (such as a mouse's primary mouse button)
     * is both pressed and released within a time period less than while the pointer is located inside the element.
     *
     * By default it just dispatches `tap` {@link ModelPrs_PointerInputEvent event}.
     */
    tap(theEvent: ModelPrs_PointerInputEvent): void;
    /**
     * Handles double tap event fires when a pointing device button (such as a mouse's primary button) is double-clicked;
     * that is, when it's rapidly clicked twice on a single element within a very short span of time.
     *
     * By default it just dispatches `doubleTap` {@link ModelPrs_PointerInputEvent event}.
     */
    doubleTap(theEvent: ModelPrs_PointerInputEvent): void;
    /**
     * Handles long press event fires when a pointing device button (such as a mouse's primary button) is pressed and held for a time
     * greater than `longPressInterval`.
     *
     * By default it just dispatches `longPress` {@link ModelPrs_PointerInputEvent event}.
     */
    longPress(theEvent: ModelPrs_PointerInputEvent): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_TapInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_TapInputHandlerEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_TapInputHandler#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_TapInputHandlerEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_TapInputHandlerEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    override pointerDown(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUpdate(theEvent: ModelPrs_PointerInputEvent): boolean;
    override pointerUp(theEvent: ModelPrs_PointerInputEvent): void;
    override pointerCancel(theEvent: ModelPrs_PointerInputEvent): void;
}
/**
 * {@link ModelPrs_ViewCube} configuration interface.
 */
type ModelPrs_ViewCubeConfig = {
    [index: string]: any;
    /**
     * Automatically resize viewcube on viewport resizing. Default is `true`.
     */
    autoResize?: boolean;
    /**
     * The font. Default is built-in font.
     */
    font?: ModelPrs_Font;
    /**
     * The font size used for text rendering. Default is `0.16137`.
     */
    fontSize?: number;
    /**
     * The text on the front cube side. Default is 'Front'.
     */
    frontText?: string;
    /**
     * The text on the back cube side. Default is 'Back'.
     */
    backText?: string;
    /**
     * The text on the left cube side. Default is 'Left'.
     */
    leftText?: string;
    /**
     * The text on the right cube side. Default is 'Right'.
     */
    rightText?: string;
    /**
     * The text on the top cube side. Default is 'Top'.
     */
    topText?: string;
    /**
     * The text on the bottom cube side. Default is 'Bottom'.
     */
    bottomText?: string;
}
type ModelPrs_ViewCubeEventMap<ExploderType extends ModelPrs_ViewCube> = ModelPrs_FieldChangedEventMap<ExploderType> & {
    viewportChanged: ModelPrs_Event<"viewportChanged", ExploderType>;
}
class ModelPrs_ViewCube extends ModelPrs_EventDispatcher {
    /**
     * Creates an instance of ModelPrs_ViewCube.
     */
    constructor(theConfig?: ModelPrs_ViewCubeConfig);
    get viewport(): ModelPrs_ViewPort | null;
    /**
     * Dom element.
     */
    get domElement(): HTMLElement;
    /**
     * Resizes view cube.
     */
    resize(theWidth: number, theHeight: number): void;
    /**
     * The method called, when the one of cube sides is clicked.
     * By default moves the camera to specified position with animation.
     */
    cubeSideClick(thePosition: ModelPrs_CameraPositionType): void;
    /**
     * The method called, when one of the rotate buttons is clicked.
     * By default rotates the camera to `Pi / 8` angle around the center of scene.
     */
    rotateButtonClick(dx: number, dy: number, dz: number): void;
    override dispose(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_ViewCubeEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ViewCubeEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_ViewCube#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_ViewCubeEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ViewCubeEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Defines a viewport paddings in pixels.
 */
type ModelPrs_ViewportPaddings = {
    left?: number;
    right?: number;
    top?: number;
    bottom?: number;
}
/**
 * {@link ModelPrs_ViewPort} configuration interface.
 */
type ModelPrs_ViewPortConfig = {
    [index: string]: any;
    /**
     * @deprecated
     * Deprecated. Use {@link ModelPrs_ViewPortConfig.addDefaultViewCube} instead.
     */
    showViewCube?: boolean;
    /**
     * Automatically resize canvas on window `'resize'` event. Default is `true`.
     */
    autoResize?: boolean;
    /**
     * Tab index which will be assigned to canvas. Default is `0`.
     */
    tabIndex?: number;
    /**
     * Adds default view cube control at the top right corner. Default is `true`.
     */
    addDefaultViewCube?: boolean;
    /**
     * Adds default lights or not. Default is `true`.
     */
    addDefaultLights?: boolean;
    /**
     * Adds default camera and selection input handlers. Default is `true`.
     */
    addDefaultInputHandlers?: boolean;
}
/**
 * Event emitted when the {@link ModelPrs_ViewPort viewport} has been resized.
 */
type ModelPrs_ViewPortResizeEvent<ViewPortType extends ModelPrs_ViewPort = ModelPrs_ViewPort> = ModelPrs_Event<"resized",ViewPortType> & {
    width: number;
    height: number;
}
type ModelPrs_ViewPortEventMap<ViewPortType extends ModelPrs_ViewPort> = {
    resized: ModelPrs_ViewPortResizeEvent<ViewPortType>;
    frameRendered: ModelPrs_Event<"frameRendered", ViewPortType>;
}
/**
 * Add viewer to dom element.
 */
class ModelPrs_ViewPort extends ModelPrs_EventDispatcher {
    constructor(theConfig?: ModelPrs_ViewPortConfig, theDomElement?: HTMLElement);
    /**
     * Dom element.
     */
    get domElement(): HTMLElement;
    /**
     * Canvas element.
     */
    get canvas(): HTMLCanvasElement;
    /**
     * Viewport width.
     */
    get width(): number;
    /**
     * Viewport hight.
     */
    get height(): number;
    /**
     * Viewport camera.
     */
    get camera(): ModelPrs_Camera;
    /**
     * A scene to which the viewport is attached.
     */
    get scene(): ModelPrs_Scene | null;
    /**
     * View cube.
     */
    get viewCube(): ModelPrs_ViewCube | null;
    set viewCube(theViewCube: ModelPrs_ViewCube | null);
    /**
     * Viewport lights manager.
     */
    get lightsManager(): ModelPrs_LightsManager;
    set lightsManager(theLightsManager: ModelPrs_LightsManager);
    /**
     * Viewport clipping planes manager.
     */
    get clipPlanesManager(): ModelPrs_ClipPlanesManager;
    set clipPlanesManager(theClipPlanesManager: ModelPrs_ClipPlanesManager);
    /**
     * Viewport markers manager.
     */
    get markerManager(): ModelPrs_MarkersManager;
    set markerManager(theMarkerManager: ModelPrs_MarkersManager);
    /**
     * Viewport input manager.
     */
    get inputManager(): ModelPrs_InputManager;
    set inputManager(theInputManager: ModelPrs_InputManager);
    /**
     * Model explosion manager.
     */
    get exploder(): ModelPrs_Exploder;
    set exploder(theExploder: ModelPrs_Exploder);
    /**
     * Attach current viewport to the scene to render its content.
     */
    attachToScene(theScene: ModelPrs_Scene): void;
    /**
     * Detach current viewport from the scene.
     */
    detachFromScene(): void;
    /**
     * Resizes viewport.
     */
    resize(theWidth: number, theHeight: number): void;
    /**
     * Schedules new frame rendering.
     */
    update(): void;
    /**
     * Sets a camera position.
     */
    setCameraPosition(thePosition: ModelPrs_CameraPositionType | ModelData_Point, theFitAllPaddings?: ModelPrs_ViewportPaddings | number | boolean, theAnimate?: boolean): void;
    /**
     * Adjust the camera so that the given bounding box is fully in its field of view.
     * `thePaddings` specifies the paddings in pixels or coefficient in % of viewport size for all sides at once.
     */
    fitBox(theBox: ModelData_Box, thePaddings?: ModelPrs_ViewportPaddings | number): void;
    /**
     * Adjust the camera so that the entire scene is fully in its field of view.
     * `thePaddings` specifies the paddings in pixels or padding in % of viewport size for all sides at once.
     */
    fitAll(thePaddings?: ModelPrs_ViewportPaddings | number): void;
    /**
     * Adjust the camera so that the given scene node is fully in its field of view.
     * `thePaddings` specifies the paddings in pixels or padding in % of viewport size for all sides at once.
     */
    fitNode(theNode: ModelPrs_SceneNode, thePaddings?: ModelPrs_ViewportPaddings | number): void;
    /**
     * Estimate Z-min and Z-max camera planes of projection volume to match the displayed objects bounding box.
     */
    zFitAll(): void;
    /**
     * Releases the data allocated by this instance.
     * Disposed instance cannot be used anymore.
     * Call this method whenever the instance is no longer used in your app.
     */
    override dispose(): void;
    /**
     * Adds the specified JavaScript function to the list of event listeners for the specified event type on which it is called.
     */
    override addEventListener<K extends keyof ModelPrs_ViewPortEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ViewPortEventMap<this>[K]>): void;
    override addEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
    /**
     * Removes an event listener previously registered with {@link ModelPrs_ViewPort#addEventListener addEventListener()}.
     */
    override removeEventListener<K extends keyof ModelPrs_ViewPortEventMap<this>>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this, ModelPrs_ViewPortEventMap<this>[K]>): void;
    override removeEventListener<K extends string>(theType: K, theListener: ModelPrs_EventDispatcherListenerType<K, this>): void;
}
/**
 * Visibility mode of the {@link ModelPrs_SceneNode}
 */
enum ModelPrs_VisibilityMode {
    Visible = 0,
    GhostlyHidden = 1,
    Hidden = 2,
    Undefined = 100,
}
class ModelPrs_WheelInputEvent<TypeType extends string = string, TargetType extends ModelPrs_InputHandler = ModelPrs_InputHandler> extends ModelPrs_InputEvent<TypeType,TargetType> {
    /**
     * Creates an instance of ModelPrs_WheelInputEvent.
     */
    constructor(theType: string, thePoint: ModelPrs_InputHandlerPoint, theDelta: number, theButton: ModelPrs_PointerButton, theButtons: number, theModifiers: number, theViewPort: ModelPrs_ViewPort, theTimeStamp?: number);
    /**
     * Returns a double representing the scroll amount.
     */
    get delta(): number;
    /**
     * Creates wheel input event from system event.
     */
    static fromWheelEvent(theEvent: WheelEvent, theViewport: ModelPrs_ViewPort): ModelPrs_WheelInputEvent;
}
}

export = cadex;
/**
 * This module is a UMD module that exposes a global variable 'cadex' when
 * loaded outside a module loader environment
 */
export as namespace cadex;
